using System;
using System.Reflection;
using System.Drawing;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;

namespace RinGOch.HtmlGenerators
{
	public class BBSGenerator
	{
		public static string ThumbnailDirectory = Path.Combine(System.Environment.CurrentDirectory,"Thumbnail");
		static readonly string[] image_exts = new string[]{".jpg",".jpeg",".png",".gif",".ico"};

		static string res_template_text;
		static string res_template_image;
		static string res_template_file;
		static string res_template_text_new;
		static string res_template_image_new;
		static string res_template_file_new;
		static string res_template_abon;

		#region 振り分け
		public static byte[] Generate(WebHeaderInfo info,StringDictionary param,out string mime,string trace_id)
		{
			try
			{
				try
				{
					string fname = Path.GetDirectoryName((string)param[""]);
					Guid id = new Guid(fname);
					return ReadBinary(info,param,id,fname,out mime);
				}
				catch
				{
					Debug.WriteLine(DateTime.Now.ToString() + "\tRequest CheckUpdateMessage",trace_id);
					if(param[""].IndexOf(".") > 0)
					{
						mime = HttpServer.GetMimeType(param[""]);
						if(param[""].ToLower().EndsWith("xml"))
						{
							StringBuilder temp = new StringBuilder(HttpHtmlGenerator.ReadTemplateFile("template\\xml\\bbs\\" + param[""]));
							temp.Replace("&rawData;",XmlGenerator.GenerateBbsXML(info));
							return HttpServerSetting.HtmlEncoding.GetBytes(temp.ToString());
						}
						else
						{
							return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\xml\\bbs\\" + param[""]));
						}
					}
					else
					{
						return GenerateBBS(info,param,out mime,trace_id);
					}
				}
			}
			catch(HttpException)
			{
				throw;
			}
			catch(Exception exp)
			{
				mime = "text/html";
				return ErrorHtml.Generate("BBSをHTMLに変換中に例外が発生しました。",exp);
			}
		}

		#endregion

		#region 非BBSファイル
		public static byte[] ReadBinary(WebHeaderInfo info,StringDictionary param,Guid id,string fname,out string mime)
		{
			mime = "*/*";
			WebPostInfo[] _posts = WebCacheReader.ReadAllPostusingGuid(info.GUID,new Guid[]{id});
			try
			{
				foreach(WebPostInfo post in _posts)
				{
					if(id.Equals(post.GUID))
					{
						mime = HttpServer.GetMimeType(post.FileName);
						if(Path.GetExtension(Path.GetFileNameWithoutExtension((string)param[""])).ToLower() == ".thum")
						{
							// サムネイル保存
							if(post.Data != null && !ExistsThum(info,post))
								SaveThumbnail(post,GetThumPath(info,post));

							// 読み込み
							using(FileStream fs = new FileStream(GetThumPath(info,post),FileMode.Open,FileAccess.Read,FileShare.Read))
							{
								byte[] buffer = new byte[fs.Length];
								fs.Read(buffer,0,buffer.Length);
								return buffer;
							}
						}
						else
						{
							return post.Data;
						}
					}
				}
				return null;
			}
			catch
			{
				return null;
			}
		}
		#endregion

		#region スレッド
		public static byte[] GenerateBBS(WebHeaderInfo info,StringDictionary param,out string mime,string trace_id)
		{
			try
			{
				Debug.WriteLine(DateTime.Now.ToString() + "\tReading All Posts",trace_id);
				DateTime start = DateTime.Now;
				mime = HttpServer.GetMimeType("bbs.p2p");
				WebPostInfo[] _posts = WebCacheReader.ReadAllPost(info.GUID);

				// ソート
				Array.Sort(_posts,new WebPostInfoComparer());

				TimeSpan readed = DateTime.Now.Subtract(start);
				WebPostInfo[] posts = RegetPostRange((string)param[""],_posts,false);
				Guid[] signErrors = WebCertificationCache.GetSignErrorList(info.GUID);
				string _header = HttpHtmlGenerator.ReadTemplateFile("template\\bbs\\header.html");
				string _footer = HttpHtmlGenerator.ReadTemplateFile("template\\bbs\\footer.html");
				string _uploader = HttpHtmlGenerator.ReadTemplateFile("template\\bbs\\uploader.html");
				string _range = HttpHtmlGenerator.ReadTemplateFile("template\\bbs\\viewrange.html");
				string _tripList = GenerateTripList();

				// レステンプレート読み込み
				string temp = HttpHtmlGenerator.ReadTemplateFile("template\\bbs\\res.html");
				res_template_text = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN TEXT-->","<!--END TEXT-->");
				res_template_image = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN IMAGE-->","<!--END IMAGE-->");
				res_template_file = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN FILE-->","<!--END FILE-->");
				res_template_text_new = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN NEW TEXT-->","<!--END NEW TEXT-->");
				res_template_image_new = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN NEW IMAGE-->","<!--END NEW IMAGE-->");
				res_template_file_new = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN NEW FILE-->","<!--END NEW FILE-->");
				res_template_abon = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN ABON-->","<!--END ABON-->");

				_header = _header.Replace("%STYLE%",HttpHtmlGenerator.ReadStyleSheet("template\\bbs\\"));

				Guid _openedRes = WebCertificationCache.GetEndGuid(info.GUID,info.Created); //BBSViewerSetting.GetReadedRes(info.GUID);
				StringBuilder sb = new StringBuilder();

				StringBuilder rangeLink = new StringBuilder();
				// 範囲リンクを作成
				for(int i=0;i<_posts.Length;i+=100)
				{
					string s = ((i+1) + "-" + ((i+100)>_posts.Length ? "" : (i+100).ToString()));
					rangeLink.Append(_range.Replace("%range%",s).Replace("%range_text%",s));
				}

				Debug.WriteLine(DateTime.Now.ToString() + "\tHTML Generating...",trace_id);

				// ヘッダ＆フッタ置き換え
				_header = TemplateFieldReplacer.ReplaceBbsFields(_header,info,null,true,"",0,_uploader,_tripList);
				_header = _header.Replace("%_bbs_rangelink%",rangeLink.ToString());

				// ヘッダを追加
				sb.Append(_header);

				if(posts.Length > 0)
				{
					int noSignCheck = Array.IndexOf(_posts,new WebPostInfo("","",DateTime.Now,WebCertificationCache.GetEndGuid(info.GUID,info.Created),null,null,"",null));
					Guid[] abon = WebBbsAbon.GetList(info.GUID);
					ArrayList errors = new ArrayList(WebCertificationCache.GetSignErrorList(info.GUID));
					int newResNum = int.MaxValue;
					if(_openedRes == Guid.Empty)
						newResNum = -1;

					// 1を表示
					if(Array.IndexOf(posts,_posts[0]) < 0 && (posts.Length>1 || Array.IndexOf(_posts,posts[0])==_posts.Length-1))
					{
						WebPostInfo post = _posts[0];

						if(0 > noSignCheck)
							if(!post.IsEffectiveSignature())
								errors.Add(post.GUID);

						if(Array.IndexOf(abon,post.GUID) < 0)
						{
							sb.Append(TemplateFieldReplacer.ReplaceBbsFields(
								ChoicePostTemplate(post,false,res_template_text,res_template_image,res_template_file,res_template_text_new,res_template_image_new,res_template_file_new),
								info,post,!errors.Contains(post.GUID),GetThumbnailUrl(info,post),1,_uploader,_tripList));
						}
					}

					// レスを追加
					foreach(WebPostInfo post in posts)
					{
						int index = Array.IndexOf(_posts,post);
						if(index > noSignCheck)
							if(!post.IsEffectiveSignature())
								errors.Add(post.GUID);

						if(post.GUID.Equals(_openedRes))
							newResNum = index;

						if(index == newResNum + 1)
						{
							sb.Append("<a id=\"_newres\"></a>\r\n");
						}

						bool isNewRes = (index > newResNum);

						if(Array.IndexOf(abon,post.GUID) < 0)
						{
							sb.Append(
								GenerateAnchor(_posts,post,
								TemplateFieldReplacer.ReplaceBbsFields(
								ChoicePostTemplate(post,isNewRes,res_template_text,res_template_image,res_template_file,res_template_text_new,res_template_image_new,res_template_file_new),
								info,post,!errors.Contains(post.GUID),GetThumbnailUrl(info,post),index+1,"","")));
						}
						else
						{
							//sb.Append(TemplateFieldReplacer.ReplaceBbsFields(res_template_abon,info,post,false,"",index+1,"",""));
							sb.Append(TemplateFieldReplacer.ReplaceBbsFields(res_template_abon,info,post,!errors.Contains(post.GUID),"",index+1,"",""));
						}
					}

					WebCertificationCache.SetEndGuid(info.GUID,info.Created,_posts[_posts.Length-1].GUID,(Guid[])errors.ToArray(typeof(Guid)));
				}
			
				// フッターを追加
				_footer = TemplateFieldReplacer.ReplaceBbsFields(_footer,info,null,true,"",0,_uploader,_tripList);
				_footer = _footer.Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
				_footer = _footer.Replace("&cache_time;",readed.TotalSeconds.ToString("0.00"));
				_footer = _footer.Replace("&html_time;",DateTime.Now.Subtract(start).Subtract(readed).TotalSeconds.ToString("0.00"));
				sb.Append(_footer);

				Debug.WriteLine(DateTime.Now.ToString() + "\tFinished HTML Generating",trace_id);

				return Encoding.UTF8.GetBytes(sb.ToString());
			}
			catch
			{
				throw;
			}
		}
		public static string ChoicePostTemplate(WebPostInfo post,bool isNewRes,string text,string image,string file,string text_new,string image_new,string file_new)
		{
			if(post.Data == null)
			{
				if(isNewRes)
					return text_new;
				else
					return text;
			}
			else if(Array.IndexOf(image_exts,Path.GetExtension(post.FileName).ToLower()) >= 0)
			{
				if(isNewRes)
					return image_new;
				else
					return image;
			}
			else
			{
				if(isNewRes)
					return file_new;
				else
					return file;
			}
		}
		public static string GetThumbnailUrl(WebHeaderInfo info,WebPostInfo post)
		{
			try
			{
				return post.GUID.ToString() + "/" + System.Web.HttpUtility.UrlEncode(Path.GetFileNameWithoutExtension(post.FileName)) + ".thum" + Path.GetExtension(post.FileName);
			}
			catch
			{
				return "";
			}
		}
		#endregion

		#region 置き換え

		#region トリップリスト作成
		public static string GenerateTripList()
		{
			return GenerateTripList("");
		}
		public static string GenerateTripList(string def)
		{
			string[] trips = Interface.GetTripList();
			StringBuilder sb = new StringBuilder();

			sb.Append("<option value=\"\" selected></option>");
			foreach(string trip in trips)
			{
				if(trip == def)
					sb.Append("<option value=\"" + trip + "\" selected>" + HttpHtmlGenerator.ReplaceHtmlChar(trip) + "</option>");
				else
					sb.Append("<option value=\"" + trip + "\">" + HttpHtmlGenerator.ReplaceHtmlChar(trip) + "</option>");
			}

			return sb.ToString();
		}

		#endregion

		#region レス
		/*public static string ReplacePostID(WebPostInfo post,bool signOK,int num,int newResNum,WebHeaderInfo thread,string resTemplate,string thumPath)
		{
			string trip = "";
			if(!signOK)
			{
				trip = "◆<font color=\"red\">認証エラー</font>";
			}
			else if(post.Trip != null)
				trip = "◆" + TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip));

			string buffer = resTemplate.Replace("%SGUID%",TextUtility.GuidToResID(post.GUID))
				.Replace("%TRIP%",trip)
				.Replace("%NAME%",HttpHtmlGenerator.ReplaceHtmlChar(post.Name).Replace("◆",""))
				.Replace("%DATE%",post.Posted.ToString())
				.Replace("%NUM%",num.ToString())
				.Replace("%NUM2%",(num>newResNum ? "<b>" + num.ToString() + "</b>" :  num.ToString()))
				.Replace("%BODY%",HttpHtmlGenerator.CrLfToBR(HttpHtmlGenerator.CreateURLLink(GenerateAnchor(HttpHtmlGenerator.ReplaceHtmlChar(post.Body)),true)));

			if(post.Data != null)
			{
				string[] image_exts = new string[]{"jpg","jpeg","png","gif","ico"};
				bool isImagePost = Array.IndexOf(image_exts,Path.GetExtension(post.FileName).Replace(".","").ToLower()) >= 0;

				if(isImagePost)
				{
					buffer = buffer.Replace("%IMAGE%","<A href=\"" + post.GUID.ToString() + "/" + System.Web.HttpUtility.UrlEncode(post.FileName) + "\"><IMG align=\"left\" src=\"" + post.GUID.ToString() + "/" + System.Web.HttpUtility.UrlEncode(Path.GetFileNameWithoutExtension(post.FileName)) + ".thum" + Path.GetExtension(post.FileName) + "\"></A>").Replace("%FILE%","");
				}
				else
				{
					buffer = buffer.Replace("%FILE%","<BR><A href=\"" + post.GUID.ToString() + "/" + System.Web.HttpUtility.UrlEncode(post.FileName) + "\">" + post.FileName + " (" + TextUtility.SizeToString(post.Data.Length) + ")</A>").Replace("%IMAGE%","");
				}
			}
			else
				buffer = buffer.Replace("%IMAGE%","").Replace("%FILE%","");

			return buffer;
		}*/
		#endregion

		#region アンカー
		public static string GenerateAnchor(WebPostInfo[] posts,WebPostInfo post,string html)
		{
			return new AnchorReplacer(posts,post).Replace(html);
		}

		internal class AnchorReplacer
		{
			WebPostInfo[] _posts;
			WebPostInfo _anchor_post;
			public AnchorReplacer(WebPostInfo[] posts,WebPostInfo anchor)
			{
				_posts = posts;
				_anchor_post = anchor;
			}
			public string Replace(string baseHtml)
			{
				return Regex.Replace(baseHtml,@"(&gt;){2}(?<anc>((([a-zA-Z0-9\+\/\=]{4}-)|(([a-zA-Z0-9\+\/\=]{4},)*)))[a-zA-Z0-9\+\/\=]{4})",new MatchEvaluator(GenerateAnchorMatchEvaluator));
			}
			public string GenerateAnchorMatchEvaluator(Match m)
			{
				return "<A href=\"#" + m.Groups["anc"] + "\" onmouseover=\"javascript:showPopup(this);\" onmouseout=\"javascript:hidePopup(this)\";>&gt;&gt;" + m.Groups["anc"] + "</a><span onmouseover=\"javascript:showPopup(this);\" onmouseout=\"javascript:hidePopup(this)\"; class=\"popupblock\"></span>";
				//return "<A id=\"anc_" + index.ToString() + "\" href=\"#" + m.Groups["anc"].Value + "\" onmouseover=\"javascript:popup_open(" + m.Groups["anc"].Value + "');\" onmouseout=\"javascript:popup_close();\">&gt;&gt;" + m.Groups["anc"].Value + "</a>";

				/*WebPostInfo[] posts = BBSGenerator.RegetPostRange2(m.Groups["anc"].Value,_posts);
				StringBuilder sb = new StringBuilder();
				int index = Array.IndexOf(_posts,_anchor_post);
				foreach(WebPostInfo post in posts)
				{
					sb.Append((Array.IndexOf(_posts,post)+1).ToString() + ",");
				}
				//sb = sb.Replace("\r\n","").Replace("\r","").Replace("\n","").Replace("'","\0");
				if(sb.Length > 0) sb = sb.Remove(sb.Length-1,1);
				return "<A id=\"anc_" + index.ToString() + "\" href=\"#" + m.Groups["anc"].Value + "\" onmouseover=\"javascript:mover(" + index.ToString() + ",'" + sb.ToString() + "');\" onmouseout=\"javascript:mout(" + index.ToString() + ");\">&gt;&gt;" + m.Groups["anc"].Value + "</a>";*/
			}
		}

		#endregion

		#endregion

		#region サムネイル
		static string GetThumRelativePath(WebHeaderInfo header,WebPostInfo info)
		{
			string dir = Path.Combine(new DirectoryInfo(ThumbnailDirectory).Name,header.GUID.ToString());
			return Path.Combine(dir , info.GUID.ToString() + ".jpg");
		}
		static string GetThumPath(WebHeaderInfo header, WebPostInfo info)
		{
			string dir = Path.Combine(ThumbnailDirectory,header.GUID.ToString());
			Directory.CreateDirectory(dir);
			return Path.Combine(dir , info.GUID.ToString() + ".jpg");
		}
		static bool ExistsThum(WebHeaderInfo header, WebPostInfo info)
		{
			return File.Exists(GetThumPath(header,info));
		}
		public static void SaveThumbnail(WebPostInfo post,string path)
		{
			try
			{
				Bitmap bmp = new Bitmap(new MemoryStream(post.Data));

				float zoomW = HttpServerSetting.ThumbnailWidth / (float)bmp.Width;
				float zoomH = HttpServerSetting.ThumbnailHeight / (float)bmp.Height;
				float zoom = (float)Math.Min(zoomW,zoomH);
				if(zoom > 1.0F)
					zoom = 1.0F;

				Bitmap thum = new Bitmap((int)(bmp.Width*zoom),(int)(bmp.Height*zoom),System.Drawing.Imaging.PixelFormat.Format32bppArgb);
				Graphics g = Graphics.FromImage(thum);
				g.InterpolationMode = HttpServerSetting.ThumbnailInterpolationMode;
				g.DrawImage(bmp,0,0,thum.Width,thum.Height);
				g.Dispose();

				bmp.Dispose();

				System.Drawing.Imaging.EncoderParameters myEncoderParameters = new System.Drawing.Imaging.EncoderParameters(1);
				System.Drawing.Imaging.EncoderParameter myEncoderParameter = new System.Drawing.Imaging.EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 100L);
				myEncoderParameters.Param[0] = myEncoderParameter;
				System.Drawing.Imaging.ImageCodecInfo myCodec = GetEncoderInfo("image/jpeg");

				thum.Save(path,myCodec,myEncoderParameters);
				thum.Dispose();
			}
			catch{}
		}
		private static System.Drawing.Imaging.ImageCodecInfo GetEncoderInfo(string mineType)
		{
			System.Drawing.Imaging.ImageCodecInfo [] myEncoders = System.Drawing.Imaging.ImageCodecInfo.GetImageEncoders();
			foreach (System.Drawing.Imaging.ImageCodecInfo myEncoder in myEncoders)
			{
				if (myEncoder.MimeType == mineType)
					return myEncoder;
			}
			return null;
		}
		#endregion

		#region 範囲のポストデータ取得
		public static WebPostInfo[] RegetPostRange(string para,WebPostInfo[] posts_clone,bool sort)
		{
			if(sort)
				Array.Sort(posts_clone,new WebPostInfoComparer());

			WebPostInfo[] posts = RegetPostRange2(para,posts_clone);
			return posts;
		}
		public static WebPostInfo[] RegetPostRange2(string para,WebPostInfo[] posts_clone)
		{
			// 全部
			if(para == "")return posts_clone;

			// 最新Xレス
			if(para.Substring(0,1) == "l" && TextUtility.IsNumeric(para.Substring(1)))
			{
				int size = (int)Math.Min(posts_clone.Length,int.Parse(para.Substring(1)));
				WebPostInfo[] posts = new WebPostInfo[size];
				Array.Copy(posts_clone,posts_clone.Length-size,posts,0,size);
				return posts;
			}

			// 数値＆ID範囲
			string[] datas = para.Split('-');
			if(datas.Length > 1 && datas.Length < 3)
			{
				int start = 1;
				int end = posts_clone.Length;
				if(datas.Length == 1)
				{
					if(TextUtility.IsNumeric(datas[0]))
						start = end = int.Parse(datas[0]);
					else
						start = end = ToResIndex(posts_clone,datas[0]);
				}
				else if(datas.Length == 2)
				{
					if(TextUtility.IsNumeric(datas[0]) && (TextUtility.IsNumeric(datas[1]) || datas[1]==""))
					{
						start = int.Parse(datas[0]);
						if(datas[1] != "")
							end = int.Parse(datas[1]);
						else
							end = posts_clone.Length;
					}
					else
					{
						start = ToResIndex(posts_clone,datas[0]);
						if(datas[1] != "")
							end = ToResIndex(posts_clone,datas[1]);
						else
							end = posts_clone.Length;
					}
				}

				if(end < start)
				{
					int temp = end;
					end = start;
					start = temp;
				}

				if(end > posts_clone.Length)
					end = posts_clone.Length;

				if(start <= 0 || end <= 0 || end > posts_clone.Length)
					return new WebPostInfo[0];

				WebPostInfo[] posts = new WebPostInfo[end - start + 1];
				Array.Copy(posts_clone,start-1,posts,0,posts.Length);

				return posts;
			}

			// カンマ区切り
			datas = para.Split(',');
			if(datas.Length > 0)
			{
				ArrayList ps = new ArrayList();
				foreach(string d in datas)
				{
					try
					{
						if(TextUtility.IsNumeric(d))
							ps.Add(posts_clone[int.Parse(d)-1]);
						else
							ps.Add(posts_clone[ToResIndex(posts_clone,d)-1]);
					}
					catch{}
				}
				return (WebPostInfo[])ps.ToArray(typeof(WebPostInfo));
			}

			return posts_clone;
		}
		public static WebPostInfo[] GetPostRange(string para,WebPostInfo[] posts_clone)
		{
			// 空っぽのキャッシュ
			if(posts_clone.Length == 0)
				return new WebPostInfo[0];

			// 新着フラッグ
			//BBSThreadUserSetting st = bbs_tmgr.ReadThreadSetting(_thread.GUID);
			//_openedRes = st.OpenedRes;
			//st.OpenedRes = posts_clone[posts_clone.Length-1].GUID;
			//bbs_tmgr.WriteThreadSetting(st);

			// アンカー読み取り
			//_ancs = new ArrayList();
			//foreach(WebPostInfo post in posts_clone)
			//	_ancs.Add(TextUtility.GuidToResID(post.GUID));

			// 表示メニュー構築
			/*cmViewRangeMenu.MenuItems.Clear();
			cmViewRangeMenu.MenuItems.Add(new MenuItem("最新50",new EventHandler(ViewRangeMenu_Click)));
			cmViewRangeMenu.MenuItems.Add(new MenuItem("全部",new EventHandler(ViewRangeMenu_Click)));
			for(int i=0;i<posts_clone.Length;i+=100)
				cmViewRangeMenu.MenuItems.Add(new MenuItem((i+1) + "-" + ((i+100)>posts_clone.Length ? "" : (i+100).ToString()),new EventHandler(ViewRangeMenu_Click)));
			cmViewRangeMenu.MenuItems.Add(new MenuItem("-"));
			MenuItem mi = new MenuItem("ツリー表示");
			mi.Checked = _hutabaMode;mi.Click += new EventHandler(HutabaMode);
			cmViewRangeMenu.MenuItems.Add(mi);
			mi = new MenuItem("投稿時間で並び替え");
			mi.Checked = _sortByTime;mi.Click += new EventHandler(SortByTime);
			cmViewRangeMenu.MenuItems.Add(mi);*/

			return RegetPostRange(para,posts_clone,false);
		}
		#endregion

		#region IDtoレス番号
		static int ToResIndex(WebPostInfo[] posts, string id)
		{
			int i = 1;
			foreach(WebPostInfo info in posts)
			{
				if(TextUtility.GuidToResID(info.GUID) == id)
					return i;
				else
					i++;
			}
			return -1;
		}
		#endregion
	}
}
