﻿#region using namespaces
using System;
using System.Security.Cryptography;
using System.Reflection;
using System.Drawing;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;
#endregion

namespace RinGOch.HtmlGenerators
{
	public class BlogEngine
	{
		#region 説明
		/// <summary>
		/// ◆/GUID/ or /GUID/index.html
		/// 　メインページ
		/// ◆/GUID/about.html
		/// 　自己紹介ページ
		/// ◆/GUID/archives.html
		/// 　アーカイブのページ
		/// ◆/GUID/yyyy/mm/index.html
		/// 　月ごとのページ
		/// ◆/GUID\\/yyyy\\/MM\\/dd\\/index.html
		/// 　日ごとのページ
		/// ◆/GUID/comments.p2p
		/// 　コメントポストページ
		/// ◆/GUID/post.p2p
		/// </summary>
		#endregion

		#region POST処理
		public static byte[] Process(WebHeaderInfo info,StringDictionary param)
		{
			if(param["type"] != null && param["type"].ToLower() == "preview")
			{
				string mime;
				return Generate(info,param,out mime,"",new StringDictionary());
			}
			else if(param["type"] != null && param["type"].ToLower() == "post")
			{
				bool auth = HttpServer.HasAuthority(param);

				BlogData[] data;
				BlogCommentData[] comms;
				BlogTrackbackData[] trackbacks;
				BlogKeyData[] keys;
				BlogFileData[] files;
				BlogMgrData about,css,desc,menu,head,base_template;
				ReadBlogCache(info,out data,out comms,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out head,out base_template);

				if(!auth) auth = HasNewPostAuthority(param,info,keys);

				if(auth)
				{
					string tempFile = "";
					try
					{
						tempFile = CopyTempTrip(Encoding.ASCII.GetString(info.Trip),info.GUID,param,info,keys);
						if(param.ContainsKey("file"))
						{
							byte[] binary = Convert.FromBase64String(param["file"]);
							if(binary.Length > 0 && binary.Length <= 1024 * 1024)
							{
								if(Interface.PostToBbs(info.GUID,"","",TextUtility.XmlToTrip(Encoding.ASCII.GetString(info.Trip)),binary,"file\0" + Path.GetFileName(param["filename"])) == Guid.Empty)
									return ErrorHtml.Generate("投稿に失敗しました。連続投稿規制がかかっているかもしれません",null);
								else
									throw new HttpException(302,"Found","Location","index.html");
							}
							else
								return ErrorHtml.Generate("ファイルサイズは1バイト以上、1MB以下である必要があります",null);
						}
						else if(param["date"].ToLower() == "base" || param["date"].ToLower() == "head" || param["date"].ToLower() == "about" || param["date"].ToLower() == "css" || param["date"].ToLower() == "desc" || param["date"].ToLower() == "menu")
						{
							Interface.PostToBbs(info.GUID,"",param["text"],TextUtility.XmlToTrip(Encoding.ASCII.GetString(info.Trip)),null,param["date"]);
							throw new HttpException(302,"Found","Location","index.html");
						}
						else
						{
							DateTime date;
							try
							{
								date = DateTime.Parse(param["date"]);
							}
							catch
							{
								return ErrorHtml.Generate("日付が認識できません",null);
							}
							Interface.PostToBbs(info.GUID,"",param["text"],TextUtility.XmlToTrip(Encoding.ASCII.GetString(info.Trip)),null,string.Concat(date.ToString("yyyy\\/MM\\/dd"),"\0",param["title"]));

							if(param["trackback"] != null && param["trackback"] != "" && param["trackback"].Length > 32)
							{
								try
								{
									BlogData bd = new BlogData();
									bd.Body = param["text"];
									bd.Date = date;
									bd.Id = Guid.Empty;
									bd.Title = param["title"];
									bd.Posted = DateTime.Now;

									string trackbackURI = param["trackback"].TrimEnd(' ','　').ToLower();

									// プロトコルヘッダを削除
									if(trackbackURI.StartsWith("http://")) trackbackURI = trackbackURI.Substring(7);

									// IPアドレス：Portを削除
									trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

									// GUIDを取得
									Guid id = new Guid(trackbackURI.Substring(0,trackbackURI.IndexOf("/")));
									trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

									string[] sp = trackbackURI.Split('/');
									DateTime target_date = new DateTime(int.Parse(sp[0]),int.Parse(sp[1]),int.Parse(sp[2]));
							
									WebHeaderInfo targetInfo = WebCacheReader.ReadHeader(id);
									if(info == null) throw new Exception();

									StringDictionary dic = new StringDictionary();
									dic.Add("title",param["title"]);
									dic.Add("excerpt",GetClearText(info,bd));
									dic.Add("url",string.Concat("&local;",info.GUID.ToString(),"/",date.ToString("yyyy\\/MM\\/dd\\/"),BlogData.GetTitleHash(param["title"])));
									dic.Add("blog_name",info.Title);
									dic.Add("_path",string.Concat("/",targetInfo.GUID.ToString(),"/",trackbackURI));

									Process(targetInfo,dic);
								}
								catch
								{
									return ErrorHtml.Generate("TrackbackURIを認識できませんでした",null);
								}
							}

							throw new HttpException(302,"Found","Location","index.html");
						}
					}
					catch
					{
						throw;
					}
					finally
					{
						try
						{
							File.Delete(tempFile);
						}
						catch{}
					}
				}
				else
					throw HttpException.UnauthorizedException;
			}
			else if(param["type"] != null && param["type"].ToLower() == "comment")
			{
				DateTime date;
				try
				{
					date = DateTime.Parse(param["date"]);
				}
				catch
				{
					return ErrorHtml.Generate("日付が認識できません",null);
				}

				param["author"] = param["author"].Trim();
				if(param["author"] == "") param["author"] = "名無し";

				if(param["text"].Length > 2000) param["text"] = param["text"].Substring(0,1000);

				Interface.PostToBbs(info.GUID,param["author"],param["text"],param["trip"],null,param["date"] + "\0" + param["title"]);
				throw new HttpException(302,"Found","Location",date.ToString("yyyy\\/MM\\/dd\\/") + BlogData.GetTitleHash(param["title"]) + "#comments");
			}
			else
			{
				string[] sp = param["_path"].Split('/');
				if(sp.Length == 6)
				{
					try
					{
						DateTime date = new DateTime(int.Parse(sp[2]),int.Parse(sp[3]),int.Parse(sp[4]));

						string trackbackURI = param["_path"].TrimEnd(' ','　').ToLower();

						// プロトコルヘッダを削除
						if(trackbackURI.StartsWith("http://")) trackbackURI = trackbackURI.Substring(7);

						// IPアドレス：Portを削除
						trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

						// GUIDを取得
						Guid id = new Guid(trackbackURI.Substring(0,trackbackURI.IndexOf("/")));
						trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

						string[] sp2 = trackbackURI.Split('/');
						DateTime target_date = new DateTime(int.Parse(sp2[0]),int.Parse(sp2[1]),int.Parse(sp2[2]));

						BlogData[] target_data;BlogCommentData[] target_comms;BlogTrackbackData[] target_trackbacks;BlogKeyData[] target_keys;BlogFileData[] target_files;BlogMgrData target_about,target_css,target_desc,target_menu,target_head,target_base_template;
						ReadBlogCache(WebCacheReader.ReadHeader(info.GUID),out target_data,out target_comms,out target_trackbacks,out target_keys,out target_files,out target_about,out target_css,out target_desc,out target_menu,out target_head,out target_base_template);

						BlogData tb = ExtractBlog(target_data,target_date,sp[sp.Length-1]);
						if(tb == null) throw new Exception();
						
						if(param["excerpt"].Length > 256) param["excerpt"] = param["excerpt"].Substring(0,256);
						if(param["title"] != null) param["title"] = param["title"].Replace("\r","").Replace("\n","").Replace("\0","");
						if(param["url"] != null) param["url"] = param["url"].Replace("\r","").Replace("\n","").Replace("\0","");
						if(param["blog_name"] != null) param["blog_name"] = param["blog_name"].Replace("\r","").Replace("\n","").Replace("\0","");

						param.Add("entry_title",tb.Title);

						Interface.PostToBbs(
							info.GUID,
							string.Concat(param["blog_name"],"\0",param["title"]),
							param["excerpt"],"",null,
							string.Concat(date.ToString("yyyy\\/MM\\/dd"),"\0",param["url"] + "\0" + param["entry_title"]));

						return HttpServerSetting.HtmlEncoding.GetBytes("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<response>\r\n<error>0</error>\r\n</response>");
					}
					catch
					{
						return HttpServerSetting.HtmlEncoding.GetBytes("<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n<response>\r\n<error>1</error>\r\n<message>Invalid Trackback URL</message>\r\n</response>");
					}
				}
			}
			return ErrorHtml.Generate("サポートしていません",null);
		}
		static string CopyTempTrip(string publicKey,Guid id,StringDictionary param,WebHeaderInfo info,BlogKeyData[] keys)
		{
			publicKey = TextUtility.GetMiddleRangeString(publicKey,"<Modulus>","</Modulus>");
			string[] files = new string[0];
			if(Directory.Exists(Path.Combine(System.Environment.CurrentDirectory,"Settings\\ManagedThreads\\")))
				files = Directory.GetFiles(Path.Combine(System.Environment.CurrentDirectory,"Settings\\ManagedThreads\\"),"*.dat");
			string tempPath = Path.Combine(System.Environment.CurrentDirectory,"trip\\blogtemp_" + id.ToString() + ".xml");
			foreach(string path in files)
			{
				try
				{
					using(StreamReader reader = new StreamReader(path))
					{
						reader.ReadLine();
						string line = reader.ReadLine();
						string pk = TextUtility.GetMiddleRangeString(line,"<Modulus>","</Modulus>");
						if(pk == publicKey)
						{
							using(StreamWriter writer = new StreamWriter(tempPath))
							{
								writer.WriteLine(line);
								return tempPath;
							}
						}
					}
				}
				catch
				{
				}
			}

			if(param.ContainsKey("_authorization"))
			{
				string text = param["_authorization"].Trim();
				text = text.Substring(text.ToLower().IndexOf("basic") + 5);
				string userpass = System.Text.Encoding.Default.GetString(Convert.FromBase64String(text));
				string privateKey = SearchManageKey(info,userpass,keys);
				if(privateKey != "")
				{
					using(StringReader reader = new StringReader(privateKey))
					{
						reader.ReadLine();
						string line = reader.ReadLine();
						using(StreamWriter writer = new StreamWriter(tempPath))
						{
							writer.WriteLine(line);
							return tempPath;
						}
					}
				}
				else
					return "";
			}
			else
				return "";
		}
		public static void SendTrackbackPingToLocal(string trackbackURI,string title,string excerpt,string url,string blog_name,string entryTitle)
		{
			trackbackURI = trackbackURI.ToLower();

			// プロトコルヘッダを削除
			if(trackbackURI.StartsWith("http://")) trackbackURI = trackbackURI.Substring(7);

			// IPアドレス：Portを削除
			trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

			// GUIDを取得
			Guid id = new Guid(trackbackURI.Substring(0,trackbackURI.IndexOf("/")));
			trackbackURI = trackbackURI.Substring(trackbackURI.IndexOf("/")+1);

			WebHeaderInfo info = WebCacheReader.ReadHeader(id);
			if(info == null) return;

			string[] sp = trackbackURI.Split('/');
			DateTime date = new DateTime(int.Parse(sp[0]),int.Parse(sp[1]),int.Parse(sp[2]));

			StringDictionary dic = new StringDictionary();
			dic.Add("title",title);
			dic.Add("excerpt",excerpt);
			dic.Add("url",url);
			dic.Add("entry_title",entryTitle);
			dic.Add("blog_name",blog_name);
			dic.Add("_path",string.Concat("/",info.GUID.ToString(),"/",trackbackURI));

			Process(info,dic);
		}
		#endregion

		#region 分岐
		public static byte[] Generate(WebHeaderInfo info,StringDictionary param,out string mime,string trace_id,StringDictionary response)
		{
			mime = "text/html";
			try
			{
				string[] sp = param["_path"].Split('/');

				DateTime IfModifiedSince = DateTime.MinValue;

				BlogData[] data;
				BlogCommentData[] comms;
				BlogTrackbackData[] trackbacks;
				BlogKeyData[] keys;
				BlogFileData[] files;
				BlogMgrData about,css,desc,menu,head,base_template;
				ReadBlogCache(info,out data,out comms,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out head,out base_template);

				BlogEngine engine = new BlogEngine(info,data,comms,trackbacks,keys,files,about,css,desc,menu,head,base_template,param);

				try
				{
					if(param.ContainsKey("_If-Modified-Since"))
					{
						Debug.WriteLine(param["_If-Modified-Since"]);
						IfModifiedSince = DateTime.Parse(param["_If-Modified-Since"]);
					}
				}
				catch
				{
				}

				if(sp.Length == 6 && sp[5].ToLower() == "index.html")
				{
					try
					{
						DateTime date = new DateTime(int.Parse(sp[2]),int.Parse(sp[3]),int.Parse(sp[4]));
						engine.GenerateBaseTemplate(date);
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateDailyPage(date));
					}
					catch
					{
						return ErrorHtml.Generate("指定された日付は無効な形式です",null);
					}
				}
				if(sp.Length == 6 && IsTitleHash(sp[5]))
				{
					try
					{
						DateTime date = new DateTime(int.Parse(sp[2]),int.Parse(sp[3]),int.Parse(sp[4]));
						engine.GenerateBaseTemplate(date);
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateEntryPage(date,sp[5]));
					}
					catch
					{
						return ErrorHtml.Generate("指定された日付は無効な形式です",null);
					}
				}
				else if(sp.Length == 5 && sp[4].ToLower() == "index.html")
				{
					try
					{
						DateTime date = new DateTime(int.Parse(sp[2]),int.Parse(sp[3]),1);
						engine.GenerateBaseTemplate(date);
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateMonthlyPage(date));
					}
					catch
					{
						return ErrorHtml.Generate("指定された日付は無効な形式です",null);
					}
				}
				else if(param["_filename"].ToLower() == "post.p2p")
				{
					engine.GenerateBaseTemplate(DateTime.Now);
					if(param["type"] != null && param["type"].ToLower() == "preview")
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GeneratePreviewPage(param));
					else
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateNewPostPage(param));
				}
				else if(sp[sp.Length-2].ToLower() == "files")
				{
					foreach(BlogFileData file in files)
					{
						if(file.FileName.ToLower() == sp[sp.Length-1].ToLower())
						{
							if(file.Posted.Date == IfModifiedSince.Date && file.Posted.Hour == IfModifiedSince.Hour && file.Posted.Minute == IfModifiedSince.Minute && file.Posted.Second == IfModifiedSince.Second)
								throw new HttpException(304,"Not Modified");
							else
							{
								response.Add("Last-Modified",file.Posted.ToUniversalTime().ToString("ddd, dd MMM yyyy hh\\:mm\\:ss ",System.Globalization.DateTimeFormatInfo.InvariantInfo) + "GMT");
								mime = HttpServer.GetMimeType(file.FileName);
								return file.RawData;
							}
						}
					}

					return ErrorHtml.Generate("ファイルが見つかりません。以下の点をご確認ください\r\n\r\n１．キャッシュが届いていない可能性があります。しばらく時間をおいてからリロードしてみてください。\r\n２．URLが間違っている可能性があります。URLを再度ご確認ください",null);
				}
				else if(sp.Length == 3)
				{
					engine.GenerateBaseTemplate(DateTime.Now);
					if(sp[2].ToLower() == "about.html")
					{
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateAboutPage());
					}
					else if(sp[2].ToLower() == "trackbacks.html")
					{
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateAllTrackbacks());
					}
					else if(sp[2].ToLower() == "index.rdf")
					{
						mime = HttpServer.GetMimeType(sp[2]);
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateRssRdf());
					}
					else if(sp[2].ToLower() == "styles.css" || sp[2].ToLower() == "styles.txt")
					{
						mime = HttpServer.GetMimeType(sp[2]);
						if(css == null)
						{
							return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\styles.css"));
						}
						else
						{
							return HttpServerSetting.HtmlEncoding.GetBytes(HtmlTool.CancelDangerCSS(css.Body));
						}
					}
					else if(sp[2].ToLower() == "about.txt")
					{
						mime = HttpServer.GetMimeType(sp[2]);
						if(about == null)
							return new byte[0];
						else
							return HttpServerSetting.HtmlEncoding.GetBytes(about.Body);
					}
					else if(sp[2].ToLower() == "desc.txt")
					{
						mime = HttpServer.GetMimeType(sp[2]);
						if(desc == null)
							return new byte[0];
						else
							return HttpServerSetting.HtmlEncoding.GetBytes(desc.Body);
					}
					else if(sp[2].ToLower() == "menu.txt")
					{
						mime = HttpServer.GetMimeType(sp[2]);
						if(menu == null)
							return new byte[0];
						else
							return HttpServerSetting.HtmlEncoding.GetBytes(menu.Body);
					}
					else
						return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateMainPage());
				}

				engine.GenerateBaseTemplate(DateTime.Now);
				return HttpServerSetting.HtmlEncoding.GetBytes(engine.GenerateMainPage());
			}
			catch(HttpException)
			{
				throw;
			}
			catch(Exception exp)
			{
				return ErrorHtml.Generate("BlogEngine Internal Error",exp);
			}
		}
		static bool IsTitleHash(string titleHash)
		{
			char[] hashDisit = new char[]{'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
			for(int i=0;i<titleHash.Length;i++)
				 if(Array.IndexOf(hashDisit,titleHash[i]) < 0)
					 return false;
			return true;
		}
		#endregion

		#region キャッシュ読み込み
		static void ReadBlogCache(WebHeaderInfo info,WebPostInfo[] posts,out BlogData[] data,out BlogCommentData[] comm,out BlogTrackbackData[] trackbacks,out BlogKeyData[] keys,out BlogFileData[] files,out BlogMgrData about,out BlogMgrData css,out BlogMgrData desc,out BlogMgrData menu,out BlogMgrData header,out BlogMgrData base_template)
		{
			ArrayList a = new ArrayList();//BlogData
			ArrayList b = new ArrayList();//BlogCommentData
			ArrayList m = new ArrayList();//BlogMgrData
			ArrayList t = new ArrayList();//BlogTrackbackData
			ArrayList k = new ArrayList();//BlogKeyData
			ArrayList f = new ArrayList();//BlogFileData
			for(int i=0;i<posts.Length;i++)
			{
				WebPostInfo post = posts[i];
				object c = BlogDataDivider.Divide(info,post);
				if(c != null)
				{
					if(c.GetType().Equals(typeof(BlogData)))
						a.Add(c);
					else if(c.GetType().Equals(typeof(BlogCommentData)))
						b.Add(c);
					else if(c.GetType().Equals(typeof(BlogMgrData)))
						m.Add(c);
					else if(c.GetType().Equals(typeof(BlogTrackbackData)))
						t.Add(c);
					else if(c.GetType().Equals(typeof(BlogKeyData)))
						k.Add(c);
					else if(c.GetType().Equals(typeof(BlogFileData)))
						f.Add(c);
				}
			}

			// 同じ日付で同じタイトルのエントリがあったら古いほうを削除
			a.Sort(new BlogPostedDateComparer(-1));
			for(int i=0;i<a.Count-1;i++)
			{
				BlogData di = (BlogData)a[i];
				for(int q=i+1;q<a.Count;q++)
				{
					BlogData dq = (BlogData)a[q];
					if(dq.Date.Date == di.Date.Date && dq.Title == di.Title)
					{
						a.RemoveAt(q);
						q--;
					}
				}
			}
			a.Sort(new BlogDateComparer(-1));
			b.Sort(new BlogDateComparer(1));

			// 同じ日付のデータがあったら古いほうを削除
			m.Sort(new BlogPostedDateComparer(-1));
			for(int i=0;i<m.Count-1;i++)
			{
				BlogMgrData di = (BlogMgrData)m[i];
				for(int q=i+1;q<m.Count;q++)
				{
					BlogMgrData dq = (BlogMgrData)m[q];
					if(dq.Title == di.Title)
					{
						m.RemoveAt(q);
						q--;
					}
				}
			}

			// 同じURLのトラックバックがあったら削除
			t.Sort(new BlogPostedDateComparer(-1));
			for(int i=0;i<t.Count-1;i++)
			{
				BlogTrackbackData di = (BlogTrackbackData)t[i];
				for(int q=i+1;q<t.Count;q++)
				{
					BlogTrackbackData dq = (BlogTrackbackData)t[q];
					if(dq.Date.Date == di.Date.Date && dq.URL == di.URL)
					{
						t.RemoveAt(q);
						q--;
					}
				}
			}

			// 同じファイル名の添付があったら削除
			f.Sort(new BlogPostedDateComparer(-1));
			for(int i=0;i<f.Count-1;i++)
			{
				BlogFileData di = (BlogFileData)f[i];
				for(int q=i+1;q<f.Count;q++)
				{
					BlogFileData dq = (BlogFileData)f[q];
					if(dq.FileName.ToLower() == di.FileName.ToLower())
					{
						f.RemoveAt(q);
						q--;
					}
				}
			}

			about = css = desc = menu = header = base_template = null;
			foreach(BlogMgrData bmd in m)
			{
				switch(bmd.Title)
				{
					case "about":
						about = bmd;
						break;
					case "css":
						css = bmd;
						break;
					case "desc":
						desc = bmd;
						break;
					case "menu":
						menu = bmd;
						break;
					case "head":
						header = bmd;
						break;
					case "base":
						base_template = bmd;
						break;
				}
			}

			data = (BlogData[])a.ToArray(typeof(BlogData));
			comm = (BlogCommentData[])b.ToArray(typeof(BlogCommentData));
			trackbacks = (BlogTrackbackData[])t.ToArray(typeof(BlogTrackbackData));
			keys = (BlogKeyData[])k.ToArray(typeof(BlogKeyData));
			files = (BlogFileData[])f.ToArray(typeof(BlogFileData));
		}
		static void ReadBlogCache(WebHeaderInfo info,out BlogData[] data,out BlogCommentData[] comm,out BlogTrackbackData[] trackbacks,out BlogKeyData[] keys,out BlogFileData[] files,out BlogMgrData about,out BlogMgrData css,out BlogMgrData desc,out BlogMgrData menu,out BlogMgrData header,out BlogMgrData base_template)
		{
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			ReadBlogCache(info,posts,out data,out comm,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out header,out base_template);
		}
		#endregion

		#region メンバ変数
		string	_temp_base;
		string	_temp_post;
		string	_temp_post_head;
		string	_temp_comm;
		string	_temp_comm_head;
		string	_temp_form;
		string	_temp_new_post;
		string	_temp_trackback;
		string	_temp_trackback_head;
		string	_temp_navi;

		string	_local;

		WebHeaderInfo			_header;
		BlogData[]				_blogData;
		BlogTrackbackData[]	_blogTrackbacks;
		BlogKeyData[]			_blogKeys;
		BlogFileData[]			_blogFiles;
		BlogMgrData				_blogAbout;
		BlogMgrData				_blogCSS;
		BlogMgrData				_blogDesc;
		BlogMgrData				_blogMenu;
		BlogMgrData				_blogHeader;
		BlogCommentData[]		_blogComments;
		StringDictionary		_params;
		#endregion

		#region コンストラクタ
		BlogEngine(WebHeaderInfo info,BlogData[] blogs,BlogCommentData[] comms,BlogTrackbackData[] trackbacks,BlogKeyData[] keys,BlogFileData[] files,BlogMgrData about,BlogMgrData css,BlogMgrData desc,BlogMgrData menu,BlogMgrData head,BlogMgrData base_template,StringDictionary param)
		{
			bool useCustomBase = (base_template != null && base_template.Body != "");
			if(useCustomBase)
				_temp_base = HtmlTool.CancelDangerTag(base_template.Body);
			else
				_temp_base = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\base.html");

			_temp_base = ReplaceBaseTemplate(_temp_base,info);
			_temp_post = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\post.html"),info);
			_temp_post_head = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\posts.html"),info);
			_temp_comm = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\comment.html"),info);
			_temp_comm_head = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\comments.html"),info);
			_temp_trackback = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\trackback.html"),info);
			_temp_trackback_head = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\trackbacks.html"),info);
			_temp_form = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\form.html"),info);
			_temp_new_post = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\new_post.html"),info);
			_temp_navi = ReplaceBaseTemplate(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\navigator.html"),info);

			_header = info;
			_blogData = blogs;
			_blogComments = comms;
			_blogTrackbacks = trackbacks;
			_blogKeys = keys;
			_blogFiles = files;
			_params = param;
			_blogDesc = desc;
			_blogCSS = css;
			_blogAbout = about;
			_blogMenu = menu;
			_blogHeader = head;

			_local = HttpHtmlGenerator.CreateLocalAddress();

			if(desc == null)
				_temp_base = _temp_base.Replace("%description%","");
			else
				_temp_base = _temp_base.Replace("%description%",GenerateBlogBody(ConvertToBlogData(_blogDesc)));

			if(menu == null)
				_temp_base = _temp_base.Replace("&menu;","");
			else
				_temp_base = _temp_base.Replace("&menu;",GenerateBlogBody(ConvertToBlogData(menu)));
		}
		public void GenerateBaseTemplate(DateTime date)
		{
			_temp_base = _temp_base.Replace("&calendar;",GenerateCalendar(_params,date));
			_temp_base = _temp_base.Replace("&ver;",EngineVersion.BlogEngineVersion);
			_temp_base = _temp_base.Replace("&script;",HttpHtmlGenerator.ReplaceLocalAddress(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\script.txt")));
			_temp_base = GenerateArchiveLinks(_temp_base);
			_temp_base = GenerateRecentLinks(_temp_base);
			_temp_base = GenerateAllLinks(_temp_base);
			_temp_base = GenerateRecentCommentedLinks(_temp_base);
			_temp_base = HttpHtmlGenerator.ReplaceLocalAddress(_temp_base);
		}
		static string ReplaceBaseTemplate(string temp,WebHeaderInfo info)
		{
			temp = TemplateFieldReplacer.ReplaceCommonHeaderFields(temp,info);
			return HttpHtmlGenerator.ReplaceLocalAddress(temp,true);
		}
		#endregion

		#region メインページ生成
		string GenerateMainPage()
		{
			StringBuilder buffer = new StringBuilder(_temp_base);

			StringBuilder contents = new StringBuilder();

			if(_blogHeader != null)
			{
				contents.Append("<div class=\"header_entry\">");
				contents.Append(GenerateBlogBody(ConvertToBlogData(_blogHeader)));
				contents.Append("</div>");
			}

			if(_blogData.Length > 0)
			{
				int q = 0;
				DateTime[] days = new DateTime[4];
				days[0] = _blogData[0].Date;
				for(int i=1;i<_blogData.Length;i++)
				{
					if(_blogData[i].Date.Year != days[q].Year || _blogData[i].Date.Month != days[q].Month || _blogData[i].Date.Day != days[q].Day)
					{
						q++;
						days[q] = _blogData[i].Date.Date;
						if(q == days.Length-1) break;
					}
				}

				for(int i=0;i<=q;i++)
				{
					try
					{
						contents.Append(GenerateDailyBlog(days[i]));
					}
					catch{}
				}
			}

			return buffer.Replace("&contents;",contents.ToString()).ToString();
		}
		#endregion

		#region その日のページを作成
		string GenerateDailyBlog(DateTime date)
		{
			BlogData[] posts = ExtractBlog(date);
			if(posts.Length == 0)
			{
				throw new Exception();
			}

			StringBuilder contents = new StringBuilder();

			contents.Append(ReplaceBlogDate(_temp_post_head,date));

			foreach(BlogData data in posts)
			{
				contents.Append(GenerateBlog(data));
			}

			return contents.ToString();
		}
		string GenerateDailyPage(DateTime date)
		{
			StringBuilder buffer = new StringBuilder(_temp_base);
			try
			{
				return buffer.Replace("&contents;",string.Concat(GenerateDailyPageNavigator(date),GenerateDailyBlog(date))).ToString();
			}
			catch
			{
				return buffer.Replace("&contents;","指定された日付の記事は見つかりませんでした").ToString();
			}
		}
		string GenerateDailyPageNavigator(DateTime date)
		{
			string temp = _temp_navi;
			string temp_prev = "<a href=\"&local;%_guid%/%blog_date2%/index.html\">&laquo; %blog_date1%</a> | ";
			string temp_next = " | <a href=\"&local;%_guid%/%blog_date2%/index.html\">%blog_date1% &raquo;</a>";

			int index = -1;
			for(int i=0;i<_blogData.Length;i++)
				if(_blogData[i].Date.Date == date)
				{
					index = i;
					break;
				}
			DateTime prevDate = date.Date;
			DateTime nextDate = date.Date;
			if(index >= 0)
			{
				for(int i=index;i<_blogData.Length;i++)
					if(prevDate != _blogData[i].Date.Date)
					{
						prevDate = _blogData[i].Date;
						break;
					}
				for(int i=index;i>=0;i--)
					if(nextDate != _blogData[i].Date.Date)
					{
						nextDate = _blogData[i].Date;
						break;
					}
			}

			if(prevDate != date.Date)
				temp = temp.Replace("&prev;",ReplaceBaseTemplate(ReplaceBlogDate(temp_prev,prevDate),_header));
			else
				temp = temp.Replace("&prev;","");

			if(nextDate != date.Date)
				temp = temp.Replace("&next;",ReplaceBaseTemplate(ReplaceBlogDate(temp_next,nextDate),_header));
			else
				temp = temp.Replace("&next;","");

			return temp;
		}
		#endregion

		#region 記事のページを作成
		string GenerateEntryPage(DateTime date,string titleHash)
		{
			StringBuilder buffer = new StringBuilder(_temp_base);
			BlogData data = ExtractBlog(date,titleHash);
			if(data == null) return buffer.Replace("&contents;","指定された記事は見つかりませんでした").ToString();

			StringBuilder contents = new StringBuilder(GenerateEntryPageNavigator(data));

			contents.Append(ReplaceBlogDate(_temp_post_head,date));

			contents.Append(GenerateBlog(data));
			contents.Append(GenerateComments(date.Date,data.Title));
			contents.Append(GenerateTrackbacks(date.Date,data.Title));
			contents.Append(GenerateCommentForm(date.Date,data.Title));

			return buffer.Replace("&contents;",contents.ToString()).ToString();
		}
		string GenerateEntryPageNavigator(BlogData data)
		{
			string temp = _temp_navi;
			string temp_prev = "<a href=\"&local;%_guid%/%blog_entry_date2%/%blog_entry_title_hash%\">&laquo; %blog_entry_title%</a> | ";
			string temp_next = " | <a href=\"&local;%_guid%/%blog_entry_date2%/%blog_entry_title_hash%\">%blog_entry_title% &raquo;</a>";

			int index = Array.IndexOf(_blogData,data);
			if(index >= 0 && index < _blogData.Length - 1)
				temp = temp.Replace("&prev;",ReplaceBaseTemplate(ReplaceBlogData(temp_prev,_blogData[index+1]),_header));
			else
				temp = temp.Replace("&prev;","");

			if(index > 0)
				temp = temp.Replace("&next;",ReplaceBaseTemplate(ReplaceBlogData(temp_next,_blogData[index-1]),_header));
			else
				temp = temp.Replace("&next;","");

			return temp;
		}
		#endregion

		#region 新規投稿する権限があるかチェック。
		static bool HasNewPostAuthority(StringDictionary param,WebHeaderInfo info,BlogKeyData[] keys)
		{
			string userpass = "";
			if(param.ContainsKey("_authorization"))
			{
				string text = param["_authorization"].Trim();
				text = text.Substring(text.ToLower().IndexOf("basic") + 5);
				userpass = System.Text.Encoding.Default.GetString(Convert.FromBase64String(text));
			}

			return (ExistsManageKey(Encoding.ASCII.GetString(info.Trip)) || SearchManageKey(info,userpass,keys) != "");
		}
		#endregion

		#region その月のページを作成
		string GenerateMonthlyPage(DateTime date)
		{
			string temp = _temp_navi;
			string temp_prev = "<a href=\"&local;%_guid%/%blog_date4%/index.html\">&laquo; %blog_date3%</a> | ";
			string temp_next = " | <a href=\"&local;%_guid%/%blog_date4%/index.html\">%blog_date3% &raquo;</a>";

			DateTime[] list = GetArchivedMonth();
			int index = -1;
			for(int i=0;i<list.Length;i++)
				if(list[i].Year == date.Year && list[i].Month == date.Month)
				{
					index = i;
					break;
				}

			if(index >= 0 && index < list.Length - 1)
				temp = temp.Replace("&prev;",ReplaceBaseTemplate(ReplaceBlogDate(temp_prev,list[index+1]),_header));
			else
				temp = temp.Replace("&prev;","");

			if(index > 0)
				temp = temp.Replace("&next;",ReplaceBaseTemplate(ReplaceBlogDate(temp_next,list[index-1]),_header));
			else
				temp = temp.Replace("&next;","");


			StringBuilder buffer = new StringBuilder(_temp_base);
			StringBuilder contents = new StringBuilder();

			contents.Append(temp);

			int entries = 0;
			for(int i=DateTime.DaysInMonth(date.Year,date.Month);i>0;i--)
			{
				try
				{
					contents.Append(GenerateDailyBlog(new DateTime(date.Year,date.Month,i)));
					entries++;
				}
				catch{}
			}

			if(entries == 0)
				contents.Append("<p>この月に記事はありません</p>");

			return buffer.Replace("&contents;",contents.ToString()).ToString();
		}
		string ReplaceMonthlyDate(string temp,DateTime date)
		{
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_month1%",date.ToString("MMMM yyyy",System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_month2%",date.ToString("yyyy\\/MM"));
			return temp;
		}
		#endregion

		#region プレビューページ
		string GeneratePreviewPage(StringDictionary param)
		{
			StringBuilder buffer = new StringBuilder(_temp_base);

			StringBuilder contents = new StringBuilder();

			try
			{
				BlogData data = new BlogData();
				data.Body = param["text"];
				data.Date = DateTime.Parse(param["date"]);
				data.Id = Guid.Empty;
				data.Posted = DateTime.Now;
				data.Title = param["title"];
				contents.Append("<h2>Preview</h2>");
				contents.Append("<h3>");
				contents.Append(HttpUtility.HtmlEncode(data.Title));
				contents.Append("</h3><p>");
				contents.Append(GenerateBlogBody(data));
				contents.Append("</p>");
				contents.Append("<h3>Edit</h3>");
			}
			catch
			{
				contents.Append("<div class=\"preview_error\">プレビューを生成できませんでした</div>");
			}

			string temp = _temp_new_post;
			temp = temp.Replace("&data;",HttpUtility.HtmlEncode(param["text"]));
			temp = temp.Replace("&title;",HttpUtility.HtmlEncode(param["title"]));
			temp = temp.Replace("&date;",HttpUtility.HtmlEncode(param["date"]));
			temp = temp.Replace("&trackback;",HttpUtility.HtmlEncode(param["trackback"]));
			contents.Append(temp);

			return buffer.Replace("&contents;",contents.ToString()).ToString();
		}
		#endregion

		#region 新規発言ページ
		string GenerateNewPostPage(StringDictionary param)
		{
			StringBuilder buffer = new StringBuilder(_temp_base);

			StringBuilder contents = new StringBuilder();

			// 秘密鍵を持っているかチェック
			bool check = ExistsManageKey(Encoding.ASCII.GetString(_header.Trip));

			// 認証を要求
			if(!check)
			{
				if(param.ContainsKey("_authorization"))
				{
					string text = param["_authorization"].Trim();
					text = text.Substring(text.ToLower().IndexOf("basic") + 5);
					string userpass = System.Text.Encoding.Default.GetString(Convert.FromBase64String(text));
					if(SearchManageKey(_header,userpass,_blogKeys) != "")
						check = true;
				}
				else
					throw HttpException.UnauthorizedException;
			}

			if(check)
			{
				string temp = _temp_new_post;
				temp = temp.Replace("&trackback;","");
				bool flag = false;
				if(param["date"] != null && param["date"] != "")
				{
					try
					{
						DateTime d = DateTime.Parse(param["date"]);
						BlogData[] posts = ExtractBlog(d);
						foreach(BlogData data in posts)
						{
							if(data.Title == param["title"])
							{
								temp = temp.Replace("&data;",HttpUtility.HtmlEncode(data.Body));
								temp = temp.Replace("&title;",HttpUtility.HtmlEncode(data.Title));
								temp = temp.Replace("&date;",HttpUtility.HtmlEncode(data.Date.ToString("yyyy\\/MM\\/dd")));
								flag = true;
								break;
							}
						}
					}
					catch
					{
					}
				}
				if(!flag)
				{
					temp = temp.Replace("&data;","");
					temp = temp.Replace("&title;","");
					temp = temp.Replace("&date;","");
				}

				contents.Append(temp);

				return buffer.Replace("&contents;",contents.ToString()).ToString();
			}
			else
				throw HttpException.UnauthorizedException;
		}
		#endregion

		#region AboutMeページ
		string GenerateAboutPage()
		{
			StringBuilder buffer = new StringBuilder(_temp_base);
			StringBuilder contents = new StringBuilder();

			if(_blogAbout != null)
			{
				contents.Append(GenerateBlogBody(ConvertToBlogData(_blogAbout)));
			}

			return buffer.Replace("&contents;",contents.ToString()).ToString();
		}
		#endregion

		#region BlogMgrData→BlogData
		BlogData ConvertToBlogData(BlogMgrData d)
		{
			BlogData data = new BlogData();
			data.Body = d.Body;
			data.Date = d.Posted;
			data.Id = d.Id;
			data.Name = d.Name;
			data.Posted = d.Posted;
			data.Sign = d.Sign;
			data.Title = d.Title;
			data.Trip = d.Trip;
			return data;
		}
		#endregion

		#region すべてのトラックバック
		string GenerateAllTrackbacks()
		{
			StringBuilder buffer = new StringBuilder(_temp_base);
			StringBuilder html = new StringBuilder();

			html.Append("<h2 class=\"alltrackbacks\">すべてのトラックバック</h1>");
			DateTime last = DateTime.MaxValue;

			foreach(BlogTrackbackData data in _blogTrackbacks)
			{
				if(last.Date != data.Date.Date)
				{
					if(last != DateTime.MaxValue)
						html.Append("</p>");
					last = data.Date;
					html.Append("<p class=\"day_trackback\">");
					html.Append(last.ToString("MMMM dd,yyyy",System.Globalization.DateTimeFormatInfo.InvariantInfo));
				}

				html.Append(GenerateTrackback(data));
			}

			if(last != DateTime.MaxValue)
				html.Append("</p>");

			return buffer.Replace("&contents;",html.ToString()).ToString();
		}
		#endregion

		#region 管理鍵走査
		static bool ExistsManageKey(string publicKey)
		{
			string pkey;
			return ExistsManageKey(publicKey,out pkey);
		}
		static bool ExistsManageKey(string publicKey,out string privateKey)
		{
			privateKey = "";
			try
			{
				publicKey = TextUtility.GetMiddleRangeString(publicKey,"<Modulus>","</Modulus>");
				string[] files = Directory.GetFiles(Path.Combine(System.Environment.CurrentDirectory,"Settings\\ManagedThreads\\"),"*.dat");
				foreach(string path in files)
				{
					try
					{
						using(StreamReader reader = new StreamReader(path))
						{
							privateKey = reader.ReadToEnd();
							string pk = TextUtility.GetMiddleRangeString(privateKey,"<Modulus>","</Modulus>");
							if(pk == publicKey)
								return true;
						}
					}
					catch
					{
					}
				}
				return false;
			}
			catch
			{
				return false;
			}
		}
		#endregion

		#region RSS/RDF

		string GenerateRssRdf()
		{
			DateTime lastUpdate = DateTime.MinValue;
			foreach(BlogData d in _blogData)
				if(d.Posted > lastUpdate)
					lastUpdate = d.Posted;

			string baseTemp = HttpHtmlGenerator.ReplaceLocalAddress(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\index.rdf"),true);
			baseTemp = TemplateFieldReplacer.ReplaceCommonHeaderFields(baseTemp,_header);
			if(_blogDesc != null)
				baseTemp = TemplateFieldReplacer.ReplaceField(baseTemp,"%_desc%",GetClearText(_header,ConvertToBlogData(_blogDesc)));
			else
				baseTemp = TemplateFieldReplacer.ReplaceField(baseTemp,"%_desc%","");

			baseTemp = baseTemp.Replace("%_last_update%",lastUpdate.ToString("yyyy'-'MM'-'dd'T'HH':'mm':'sszzz"));

			string main = TextUtility.GetMiddleRangeString(baseTemp,"<!--main-->","<!--main-->").TrimStart('\r','\n');
			string seqLoop = TextUtility.GetMiddleRangeString(baseTemp,"<!--rdf:Seq:loop-->","<!--rdf:Seq:loop-->").TrimStart('\r','\n');
			string itemLoop = TextUtility.GetMiddleRangeString(baseTemp,"<!--item:loop-->","<!--item:loop-->").TrimStart('\r','\n');
			StringBuilder seqs = new StringBuilder();
			StringBuilder items = new StringBuilder();
			int len = (_blogData.Length >= 16 ? 16 : _blogData.Length);

			// rdf:Seq:Loop
			for(int i=0;i<len;i++)
			{
				seqs.Append(ReplaceBlogData(seqLoop,_blogData[i]));
			}

			// items
			for(int i=0;i<len;i++)
			{
				items.Append(
					ReplaceBlogData(itemLoop,_blogData[i])
					.Replace("%_update%",_blogData[i].Posted.ToString("yyyy'-'MM'-'dd'T'HH':'mm':'sszzz"))
					.Replace("&desc;",GetClearText(_header,_blogData[i]))
					);
			}

			return main.Replace("&rdf:Seq:loop;",seqs.ToString()).Replace("&items;",items.ToString());
		}
		

		public string GenerateBlogRssRdf()
		{
			StringBuilder temp = new StringBuilder();
			StringWriter writer = new StringWriter(temp);

			string desc = "";
			if(_blogData.Length > 0)
			{
				desc = GetClearText(_header,_blogData[0]);
			}

			writer.WriteLine(string.Concat("<item rdf:about=\"",_local,_header.GUID.ToString(),"/\">"));
			writer.WriteLine(string.Concat("<title>",HttpUtility.HtmlEncode(_header.Title),"</title> "));
			writer.WriteLine(string.Concat("<link>",_local,_header.GUID.ToString(),"/</link>"));
			writer.WriteLine(string.Concat("<dc:creator>",HttpUtility.HtmlEncode(_header.Category),"</dc:creator>"));
			writer.WriteLine(string.Concat("<dc:date>","","</dc:date>"));
			writer.WriteLine("</item>");
			writer.Close();
			return temp.ToString();
		}

		#endregion

		#region テキストデータを返す
		static string GetClearText(WebHeaderInfo info,BlogData data)
		{
			string text = GenerateBlogBody(info,data);
			text = GetClearTextFromHTML(text);
			if(text.Length > 256) text = text.Substring(0,253) + "...";
			return HttpUtility.HtmlEncode(text);
		}
		static string GetClearTextFromHTML(string html)
		{
			html = html.Replace("\r","").Replace("\n","");
			return Regex.Replace(html,@"\<.+?\>","");
		}
		#endregion

		#region 指定された日時の記事を取得
		BlogData[] ExtractBlog(DateTime date)
		{
			ArrayList list = new ArrayList();
			foreach(BlogData data in _blogData)
				if(data.Date.Date == date.Date)
					list.Add(data);
			return (BlogData[])list.ToArray(typeof(BlogData));
		}
		static BlogData ExtractBlog(BlogData[] datas,DateTime date,string titleHash)
		{
			foreach(BlogData data in datas)
				if(data.Date.Date == date.Date && data.TitleHash == titleHash)
					return data;
			return null;
		}
		BlogData ExtractBlog(DateTime date,string titleHash)
		{
			return ExtractBlog(_blogData,date,titleHash);
		}
		BlogData[] ExtractBlogMonthly(DateTime date)
		{
			ArrayList list = new ArrayList();
			foreach(BlogData data in _blogData)
				if(data.Date.Year == date.Year && data.Date.Month == date.Month)
					list.Add(data);
			return (BlogData[])list.ToArray(typeof(BlogData));
		}
		#endregion

		#region コメントを作成
		string GenerateCommentForm(DateTime date,string entryTitle)
		{
			string cf = ReplaceBlogDate(_temp_form,date);
			cf = TemplateFieldReplacer.ReplaceField(cf,"%blog_comm_form_title%",entryTitle);
			cf = cf.Replace("%triplist%",BBSGenerator.GenerateTripList());
			return cf;
		}
		string GenerateComments(DateTime date,string title)
		{
			BlogCommentData[] comms = ExtractComments(date,title);
			StringBuilder html = new StringBuilder(_temp_comm_head);

			foreach(BlogCommentData com in comms)
			{
				html.Append(GenerateComment(com));
			}

			return html.ToString();
		}
		string GenerateComment(BlogCommentData data)
		{
//			data.Comment = data.Comment.Replace("\r\n","\n").Replace("\r","\n").Replace("\n","<br />");
			data.Comment = data.Comment.Replace("\r\n","\n").Replace("\r","\n").Replace("\n","<br />");;

			string buffer = ReplaceBlogCommentData(_temp_comm,data);
			buffer = Regex.Replace(buffer,@"&lt;\s*br\s*/?\s*&gt;","<br />",RegexOptions.IgnoreCase);
			buffer = HttpHtmlGenerator.ReplaceLocalAddress(buffer);
			buffer = HttpHtmlGenerator.CreateURLLink(buffer);
//			buffer = buffer.Replace("\n","<br />");
//			buffer = Regex.Replace(buffer,@"&lt;\s*br\s*/?\s*&gt;","<br />",RegexOptions.IgnoreCase);
//			buffer = "てすと "+buffer;

			return buffer;
		}
		#endregion

		#region トラックバックＨＴＭＬを作成
		string GenerateTrackbacks(DateTime date,string title)
		{
			BlogTrackbackData[] tracks = ExtractTrackbacks(date,title);
			StringBuilder html = new StringBuilder(_temp_trackback_head.Replace("&trackback_url;",string.Concat("&local;",_header.GUID.ToString(),date.ToString("\\/yyyy\\/MM\\/dd\\/"),BlogData.GetTitleHash(title))));

			foreach(BlogTrackbackData track in tracks)
			{
				html.Append(GenerateTrackback(track));
			}

			return HttpHtmlGenerator.ReplaceLocalAddress(html.ToString());
		}
		string GenerateTrackback(BlogTrackbackData data)
		{
			string buffer = _temp_trackback;

			buffer = ReplaceBlogTrackbackData(buffer,data);

			return HttpHtmlGenerator.ReplaceLocalAddress(buffer);
		}
		#endregion

		#region 記事を作成
		string GenerateBlog(BlogData data)
		{
			string buffer = _temp_post;
			buffer = TemplateFieldReplacer.ReplaceField(buffer,"%blog_entry_body%",GenerateBlogBody(data));
			buffer = ReplaceBlogData(buffer,data);
			buffer = TemplateFieldReplacer.ReplaceField(buffer,"%blog_entry_comment_count%",ExtractComments(data.Date,data.Title).Length.ToString());
			buffer = TemplateFieldReplacer.ReplaceField(buffer,"%blog_entry_trackback_count%",ExtractTrackbacks(data.Date,data.Title).Length.ToString());

			return buffer;
		}
		string GenerateBlogBody(BlogData data)
		{
			return GenerateBlogBody(_header,data);
		}
		static string GenerateBlogBody(WebHeaderInfo info,BlogData data)
		{
			string body = data.Body;
			body = HttpHtmlGenerator.ReplaceLocalAddress(body,true);
			if(body.StartsWith("@html"))
			{
				body = body.Substring(5);
				body = HtmlTool.CancelDangerTag(body);
				return body;
			}
			else if(body.StartsWith("@wiki"))
			{
				WebPostInfo post = new WebPostInfo(data.Name,data.Body,data.Posted,data.Id,null,null,"",null);

				body = body.Substring(5);
				WikiEngine.ContentsTransformer transformer = new RinGOch.WikiEngine.ContentsTransformer(info,new WebPostInfo[0],post,"_blank");
				transformer.replacer.enableEmbedded = false;
				transformer.replacer.BlogMode = true;
				return transformer.Transform(body);
			}
			else
			{
				return HttpHtmlGenerator.CreateURLLink(HttpUtility.HtmlEncode(body).Replace("\r\n","\n").Replace("\r","\n").Replace("\n","<br/>"),true);
			}
		}
		#endregion

		#region 指定された日時のコメントを抜き出し
		static BlogCommentData[] ExtractComments(BlogCommentData[] comms,DateTime date,string title)
		{
			ArrayList list = new ArrayList();
			foreach(BlogCommentData comm in comms)
			{
				if(comm.Date.Date == date.Date && comm.EntryTitle == title)
					list.Add(comm);
			}
			return (BlogCommentData[])list.ToArray(typeof(BlogCommentData));
		}
		BlogCommentData[] ExtractComments(DateTime date,string title)
		{
			return ExtractComments(_blogComments,date,title);
		}
		#endregion

		#region 指定された日時のトラックバックを抜き出し
		BlogTrackbackData[] ExtractTrackbacks(DateTime date,string title)
		{
			ArrayList list = new ArrayList();
			foreach(BlogTrackbackData track in _blogTrackbacks)
			{
				if(track.Date.Date == date.Date && track.EntryTitle == title)
					list.Add(track);
			}
			return (BlogTrackbackData[])list.ToArray(typeof(BlogTrackbackData));
		}
		#endregion

		#region 最近の記事データを作成
		string GenerateRecentLinks(string template)
		{
			const string beginWord = "<!--BEGIN RECENT-->";
			const string endWord = "<!--END RECENT-->";

			int len = (_blogData.Length > 10 ? 10 : _blogData.Length);

			StringBuilder html = new StringBuilder();
			string loop = TextUtility.GetMiddleRangeString(template,beginWord,endWord);

			ArrayList temp = new ArrayList();
			temp.AddRange(_blogData);

			temp.Sort(new BlogPostedDateComparer(-1));

			for(int i=0;i<len;i++)
			{
				BlogData data = (BlogData)temp[i];
				string buf = ReplaceBlogData(loop,data);
				buf = HttpHtmlGenerator.ReplaceLocalAddress(buf);
				html.Append(buf);
			}

			temp.Clear();temp=null;

			return template.Replace(string.Concat(beginWord,loop,endWord),html.ToString());
		}
		#endregion

		#region 記事一覧を作成
		string GenerateAllLinks(string template)
		{
			return template;
			const string beginWord = "<!--BEGIN LIST-->";
			const string endWord = "<!--END LIST-->";

			int len = (_blogData.Length);

			StringBuilder html = new StringBuilder();
			string loop = TextUtility.GetMiddleRangeString(template,beginWord,endWord);

			ArrayList temp = new ArrayList();
			temp.AddRange(_blogData);

			temp.Sort(new BlogPostedDateComparer(-1));

			for(int i=0;i<len;i++)
			{
				BlogData data = (BlogData)temp[i];
				string buf = ReplaceBlogData(loop,data);
				buf = HttpHtmlGenerator.ReplaceLocalAddress(buf);
				html.Append(buf);
			}

			temp.Clear();temp=null;

			return template.Replace(string.Concat(beginWord,loop,endWord),html.ToString());
		}
		#endregion

		#region 最近コメントされた記事のデータを作成
		string GenerateRecentCommentedLinks(string template)
		{
			const string beginWord = "<!--BEGIN RECENT2-->";
			const string endWord = "<!--END RECENT2-->";

			StringBuilder html = new StringBuilder();
			string loop = TextUtility.GetMiddleRangeString(template,beginWord,endWord);

			ArrayList temp = new ArrayList();
			temp.AddRange(_blogComments);
			temp.Sort(new BlogPostedDateComparer(-1));
			int len = (_blogComments.Length > 10 ? 10 : _blogComments.Length);

			for(int i=0;i<len;i++)
			{
				BlogCommentData data = (BlogCommentData)temp[i];
				string buf = ReplaceBlogCommentData(loop,data);
				buf = HttpHtmlGenerator.ReplaceLocalAddress(buf);
				html.Append(buf);
			}

			temp=null;

			return template.Replace(string.Concat(beginWord,loop,endWord),html.ToString());
		}
		#endregion

		#region アーカイブデータ生成
		DateTime[] GetArchivedMonth()
		{
			ArrayList list = new ArrayList();

			DateTime start = DateTime.MaxValue;
			
			foreach(BlogData data in _blogData)
			{
				if(data.Date.Year != start.Year || data.Date.Month != start.Month)
				{
					start = data.Date;
					list.Add(start);
				}
			}

			return (DateTime[])list.ToArray(typeof(DateTime));
		}
		string GenerateArchiveLinks(string temp)
		{
			const string beginWord = "<!--BEGIN ARCHIVE-->";
			const string endWord = "<!--END ARCHIVE-->";

			DateTime[] list = GetArchivedMonth();

			StringBuilder html = new StringBuilder();
			string loop = TextUtility.GetMiddleRangeString(temp,beginWord,endWord);

			foreach(DateTime date in list)
			{
				string buf = ReplaceBlogDate(loop,date);
				buf = buf.Replace("%archive_uri%",string.Concat(_local,_header.GUID.ToString(),date.ToString("\\/yyyy\\/MM\\/"),"index.html"));
				html.Append(buf);
			}

			return temp.Replace(string.Concat(beginWord,loop,endWord),html.ToString());
		}
		#endregion

		#region カレンダー生成
		string GenerateCalendar(StringDictionary param,DateTime calDate)
		{
			StringBuilder buffer = new StringBuilder();
			StringWriter writer = new StringWriter(buffer);

			DateTime startDate = new DateTime(calDate.Year,calDate.Month,1);
			DateTime endDate = new DateTime(calDate.Year,calDate.Month,1).AddMonths(1).AddDays(-1);
			DateTime curDate = startDate.AddDays(-(int)startDate.DayOfWeek);

			writer.WriteLine("<table>");
			writer.WriteLine(string.Concat("<caption>",
				"<a class=\"prevMonth\" href=\"",
				_local,_header.GUID.ToString(),calDate.AddMonths(-1).ToString("\\/yyyy\\/MM\\/"),"index.html",
				"\">←</a><span class=\"cal_date\"",
				/*" onmouseout=\"javascript:calendar_onMouseOut(this)\" onmouseover=\"javascript:calendar_onMouseOver(this)\" title=\"&mentries;\"",*/
				">",
				calDate.ToString("MMMM yyyy",System.Globalization.DateTimeFormatInfo.InvariantInfo),
				"</span><a class=\"nextMonth\" href=\"",
				_local,_header.GUID.ToString(),calDate.AddMonths(1).ToString("\\/yyyy\\/MM\\/"),"index.html",
				"\">→</a>",
				"</caption>"));
			writer.WriteLine("<tr>");
			writer.WriteLine("<th abbr=\"Sunday\"><span class=\"calendar\">Sun</span></th>");
			writer.WriteLine("<th abbr=\"Monday\"><span class=\"calendar\">Mon</span></th>");
			writer.WriteLine("<th abbr=\"Tuesday\"><span class=\"calendar\">Tue</span></th>");
			writer.WriteLine("<th abbr=\"Wednesday\"><span class=\"calendar\">Wed</span></th>");
			writer.WriteLine("<th abbr=\"Thursday\"><span class=\"calendar\">Thu</span></th>");
			writer.WriteLine("<th abbr=\"Friday\"><span class=\"calendar\">Fri</span></th>");
			writer.WriteLine("<th abbr=\"Saturday\"><span class=\"calendar\">Sat</span></th>");
			writer.WriteLine("</tr>");

			StringBuilder mentries = new StringBuilder();
			while(curDate <= endDate)
			{
				writer.WriteLine("<tr>");
				for(int i=0;i<7;i++)
				{
					if(curDate < startDate || curDate > endDate)
					{
						writer.WriteLine("<td>&nbsp;</td>");
					}
					else
					{
						string holiday = IsHoliday(curDate);
						BlogData[] ds = ExtractBlog(curDate);
						if(ds != null && ds.Length > 0)
						{
							writer.Write("<td class=\"" + (holiday=="" ? string.Concat("week",i.ToString()) : string.Concat("holiday",i.ToString())) + "\"><a href=\"");
							writer.Write(_local);
							writer.Write(_header.GUID.ToString());
							writer.Write(curDate.ToString("\\/yyyy\\/MM\\/dd\\/"));
							writer.Write("index.html\" onmouseout=\"javascript:calendar_onMouseOut(this)\" onmouseover=\"javascript:calendar_onMouseOver(this)\" title=\"");

							mentries.Append(curDate.ToString("yyyy\\/MM\\/dd"));
							mentries.Append("&#13;&#10;");
							writer.Write(curDate.ToString(@"yyyy\/MM\/dd \(dddd\)"));
							if(holiday != "")
								writer.Write(" - " + holiday);
							writer.Write("&#13;&#10;");
							foreach(BlogData d in ds)
							{
								string temp = string.Concat(HttpUtility.HtmlEncode(string.Concat(
									d.Title,
									" [comments:",
									ExtractComments(d.Date,d.Title).Length.ToString(),
									" trackbacks:",
									ExtractTrackbacks(d.Date,d.Title).Length.ToString(),
									"]")
									),"&#13;&#10;").Replace("\"","'");
								writer.Write(temp);
								mentries.Append(TextUtility.ToQuotationText(temp,"&#9;"));
							}
							
							writer.Write("\">");
							writer.Write(curDate.Day.ToString());
							writer.WriteLine("</a></td>");
						}
						else
						{
							writer.Write("<td class=\"" + (holiday=="" ? string.Concat("week",i.ToString()) : string.Concat("holiday",i.ToString())) + "\" title=\"" + holiday + "\">");
							writer.Write(curDate.Day.ToString());
							writer.WriteLine("</td>");
						}
					}

					curDate = curDate.AddDays(1);
				}
				writer.WriteLine("</tr>");
			}

			writer.WriteLine("</table>");

			writer.Close();
			//buffer = buffer.Replace("&mentries;",mentries.ToString());
			return TextUtility.ToQuotationText(buffer.ToString(),"\t\t\t\t");
		}

		/// <summary>
		/// 祝日かどうかを判定します。空文字なら祝日ではありません。そのほかの場合は祝日名が入ります
		/// </summary>
		static string IsHoliday(DateTime date)
		{
			string holidayName = GetHolidayName(date);
			if(holidayName == "")
			{
				if(date.DayOfWeek == DayOfWeek.Monday)
				{
					if(date >= new DateTime(1973,4,12))
					{
						holidayName = GetHolidayName(date.AddDays(-1));
						if(holidayName != "")
							return holidayName + "の振替休日";
						else
							return "";
					}
					else
						return "";
				}
				else
					return "";
			}
			else
				return holidayName;
		}
		static string GetHolidayName(DateTime date)
		{
			if(date.Date < new DateTime(1948,7,20)) return "";

			switch(date.Month)
			{
				case 1:
					if(date.Day == 1)
						return "元日";
					else if( (date.Year >= 2000 && (((date.Day - 1) / 7)) == 1 && date.DayOfWeek == DayOfWeek.Monday) || (date.Year < 2000 && date.Day == 15) )
						return "成人の日";
					break;
				case 2:
					if(date.Year >= 1967 && date.Day == 11)
						return "建国記念の日";
					else if(date.Year == 1989 && date.Day == 24)
						return "昭和天皇の大喪の礼";
					break;
				case 3:
					if(IsVernalEquinoxDay(date))
						return "春分の日";
					break;
				case 4:
					if(date.Day == 29)
					{
						if(date.Year >= 1989)
							return "みどりの日";
						else
							return "天皇誕生日";
					}
					else if(date.Year == 1959 && date.Day == 10)
						return "皇太子明仁親王の結婚の儀";
					break;
				case 5:
					if(date.Day == 3)
						return "憲法記念日";
					else if(date.Day == 4)
					{
						if(date.DayOfWeek > DayOfWeek.Monday)
						{
							if(date.Year >= 1986)
								return "国民の休日";
						}
					}
					else if(date.Day == 5)
						return "子供の日";
					break;
				case 6:
					if(date.Year == 1993 && date.Day == 9)
						return "皇太子徳仁親王の結婚の儀";
					break;
				case 7:
					if(date.Year >= 2003)
					{
						if((((date.Day - 1) / 7)) == 2 && date.DayOfWeek == DayOfWeek.Monday)
							return "海の日";
					}
					else if(date.Year >= 1996)
					{
						if(date.Day == 20)
							return "海の日";
					}
					break;
				case 9:
					if(IsAutumnalEquinoxDay(date))
						return "秋分の日";
					else
					{
						if(date.Year >= 2003)
						{
							if((((date.Day - 1) / 7)) == 2 && date.DayOfWeek == DayOfWeek.Monday)
								return "敬老の日";
						else if(date.DayOfWeek == DayOfWeek.Tuesday && IsAutumnalEquinoxDay(date.AddDays(-1)))
								  return "国民の休日";
						}
						else if(date.Year >= 1966)
						{
							if(date.Day == 15)
								return "敬老の日";
						}
					}
					break;
				case 10:
					if(date.Year >= 2000)
					{
						if((((date.Day - 1) / 7)) == 1 && date.DayOfWeek == DayOfWeek.Monday)
							return "体育の日";
					}
					else if(date.Year >= 1966)
					{
						if(date.Day == 10)
							return "体育の日";
					}
					break;
				case 11:
					if(date.Day == 3)
						return "文化の日";
					else if(date.Day == 23)
						return "勤労感謝の日";
					else if(date.Year == 1990 && date.Day == 12)
						return "即位礼正殿の儀";
					break;
				case 12:
					if(date.Day == 23 && date.Year >= 1989)
						return "天皇誕生日";
					break;
			}
			return "";
		}
		static bool IsVernalEquinoxDay(DateTime date)
		{
			if(date.Month == 3)
			{
				if(date.Year <= 1947)
				{
					return false;			// 春分の日がないらしぃ
				}
				else if(date.Year <= 1979)
				{
					return (date.Day == (int)(20.8357 + (0.242194 * (date.Year - 1980)) - ((date.Year - 1983) / 4)));
				}
				else if(date.Year <= 2099)
				{
					return (date.Day == (int)(20.8431 + (0.242194 * (date.Year - 1980)) - (int)((date.Year - 1980) / 4)));
				}
				else if(date.Year == 2150)
				{
					return (date.Day == (int)(21.851 + (0.242194 * (date.Year - 1980)) - (int)((date.Year - 1980) / 4)));
				}
				else
					return false;			// 不明
			}
			else
				return false;
		}
		static bool IsAutumnalEquinoxDay(DateTime date)
		{
			if(date.Month == 9)
			{
				if(date.Year <= 1947)
				{
					return false;
				}
				else if(date.Year <= 1979)
				{
					return (date.Day == (int)(23.2588 + (0.242194 * (date.Year - 1980)) - (int)((date.Year - 1983) / 4)));
				}
				else if(date.Year <= 2099)
				{
					return (date.Day == (int)(23.2488 + (0.242194 * (date.Year - 1980)) - (int)((date.Year - 1980) / 4)));
				}
				else if(date.Year <= 2150)
				{
					return (date.Day == (int)(24.2488 + (0.242194 * (date.Year - 1980)) - (int)((date.Year - 1980) / 4)));
				}
				else
					return false;
			}
			else
				return false;
		}
		#endregion

		#region 指定された日時のブログがあるかチェック
		static bool Exists(DateTime date,BlogData[] data)
		{
			foreach(BlogData d in data)
			{
				if(date.Date == d.Date.Date)
				{
					return true;
				}
			}

			return false;
		}
		#endregion

		#region 登録鍵から秘密鍵検索
		static string SearchManageKey(WebHeaderInfo info,string userpass,BlogKeyData[] keys)
		{
			string publicKey = TextUtility.GetMiddleRangeString(Encoding.ASCII.GetString(info.Trip),"<Modulus>","</Modulus>");
					
			foreach(BlogKeyData key in keys)
			{
				try
				{
					string kk = Encoding.ASCII.GetString(key.Decode(userpass));
					string pk = TextUtility.GetMiddleRangeString(kk,"<Modulus>","</Modulus>");
					if(pk == publicKey)
					{
						return kk;
					}
				}
				catch
				{
				}
			}
			return "";
		}
		#endregion

		#region 管理

		#region 分岐
		public static byte[] GenerateManagePage(StringDictionary param,ref string mime)
		{
			if(HttpServer.HasAuthority(param))
			{
				WebHeaderInfo info;

				// スタイルシート
				if(param["page"] == "style")
				{
					mime = HttpServer.GetMimeType("styles.css");
					return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\styles.css"));
				}

				// idチェック＆ヘッダ読み込み
				try
				{
					Guid id = new Guid(param["id"]);
					info = WebCacheReader.ReadHeader(id);
					if(info == null) throw new Exception();
				}
				catch
				{
					return ErrorHtml.Generate("パラメータ(ID)が無効です。",null);
				}
			
				// キャッシュ読み込み
				BlogData[] data;
				BlogCommentData[] comms;
				BlogTrackbackData[] trackbacks;
				BlogKeyData[] keys;
				BlogFileData[] files;
				BlogMgrData about,css,desc,menu,head,base_template;
				ReadBlogCache(info,out data,out comms,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out head,out base_template);

				StringBuilder html = new StringBuilder(ReadBaseManageTemplate(info));
				string temp = "";

				switch(param["page"])
				{
					case "0":	// トップ
						temp = GenerateManageTop();
						break;
					case "1":	// Blog情報変更
						temp = GenerateManageInfo(info);
						break;
					case "2":	// エントリーの削除
						temp = GenerateManageEntryDeletePage(info,data);
						break;
					case "3":	// コメントの削除
						temp = GenerateManageCommentDeletePage(info,comms);
						break;
					case "4":	// 自動クリーンアップ
						temp = GenerateManageCleanup(info);
						break;
					case "5":	// 情報
						temp = GenerateManageInfo(info,data.Length,comms.Length,trackbacks.Length);
						break;
					case "6":
						temp = GenerateManageRegPass(info);
						break;
					case "7":
						temp = GenerateManageFileDeletePage(info,files);
						break;
					case "8":
						temp = GenerateManageDeleteSpecialPage(info);
						break;
					default:
						return ErrorHtml.Generate("パラメータ(page)が無効です",null);
				}

				html = html.Replace("&contents;",temp);
				return HttpServerSetting.HtmlEncoding.GetBytes(html.ToString());
			}
			else
				throw HttpException.UnauthorizedException;
		}
		#endregion

		#region ベーステンプレート読み込み
		static string ReadBaseManageTemplate(WebHeaderInfo info)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\base.html");
			temp = TemplateFieldReplacer.ReplaceCommonHeaderFields(temp,info);
			temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);

			return temp;
		}
		#endregion

		#region トップページ生成
		static string GenerateManageTop()
		{
			return HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\top.html");
		}
		#endregion

		#region 情報ページ生成
		static string GenerateManageInfo(WebHeaderInfo info,int entries,int comms,int tracks)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\info.html");

			temp = temp.Replace("&entries;",entries.ToString());
			temp = temp.Replace("&comments;",comms.ToString());
			temp = temp.Replace("&trackbacks;",tracks.ToString());

			temp = TemplateFieldReplacer.ReplaceCommonHeaderFields(temp,info);

			return temp;
		}
		#endregion

		#region 基本情報変更ページ
		static string GenerateManageInfo(WebHeaderInfo info)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\changeinfo.html");

			temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);
			temp = temp.Replace("&category;",TemplateFieldReplacer.GenerateCategoryList(info.Category));
			temp = temp.Replace("&guid;",info.GUID.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%title%",info.Title);

			return temp;
		}
		#endregion

		#region エントリ削除ページ
		static string GenerateManageEntryDeletePage(WebHeaderInfo info,BlogData[] blogs)
		{
			string temp = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_entry.html"),info),true);
			string entry_day = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_entry_day.html"),info),true);
			string entry_item = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_entry_item.html"),info),true);

			StringBuilder html = new StringBuilder();
			StringBuilder items = new StringBuilder();

			DateTime last = DateTime.MaxValue;

			foreach(BlogData data in blogs)
			{
				if(last.Year != data.Date.Year || last.Month != data.Date.Month || last.Day != data.Date.Day)
				{
					if(last != DateTime.MaxValue)
					{
						string day = ReplaceBlogDate(entry_day,last);
						day = day.Replace("&entry_items;",items.ToString());
						html.Append(day);
						items = new StringBuilder();
					}
					last = data.Date.Date;
				}

				items.Append(ReplaceBlogData(entry_item,data));
			}

			if(last != DateTime.MaxValue)
			{
				string day = ReplaceBlogDate(entry_day,last);
				day = day.Replace("&entry_items;",items.ToString());
				html.Append(day);
				items = new StringBuilder();
			}

			return temp.Replace("&entry_days;",html.ToString());
		}
		#endregion

		#region コメント削除ページ
		static string GenerateManageCommentDeletePage(WebHeaderInfo info,BlogCommentData[] comms)
		{
			string temp = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_comm.html"),info),true);
			string comm_day = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_comm_day.html"),info),true);
			string comm_item = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_comm_item.html"),info),true);

			StringBuilder html = new StringBuilder();
			StringBuilder items = new StringBuilder();

			Array.Sort(comms,new BlogDateComparer(-1));

			DateTime last = DateTime.MaxValue;

			foreach(BlogCommentData comm in comms)
			{
				if(last.Year != comm.Date.Year || last.Month != comm.Date.Month || last.Day != comm.Date.Day)
				{
					if(last != DateTime.MaxValue)
					{
						string day = ReplaceBlogDate(comm_day,last);
						day = day.Replace("&comm_items;",items.ToString());
						html.Append(day);
						items = new StringBuilder();
					}
					last = comm.Date.Date;
				}

				items.Append(ReplaceBlogCommentData(comm_item,comm));
			}

			if(last != DateTime.MaxValue)
			{
				string day = ReplaceBlogDate(comm_day,last);
				day = day.Replace("&comm_items;",items.ToString());
				html.Append(day);
				items = new StringBuilder();
			}

			return temp.Replace("&comm_days;",html.ToString());
		}
		#endregion

		#region ファイル削除ページ
		static string GenerateManageFileDeletePage(WebHeaderInfo info,BlogFileData[] files)
		{
			string temp = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_file.html"),info),true);
			string file_item = HttpHtmlGenerator.ReplaceLocalAddress(TemplateFieldReplacer.ReplaceCommonHeaderFields(HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_file_item.html"),info),true);

			StringBuilder items = new StringBuilder();

			DateTime last = DateTime.MaxValue;

			foreach(BlogFileData file in files)
			{
				items.Append(ReplaceBlogFileData(file_item,file));
			}

			return temp.Replace("&file_items;",items.ToString());
		}
		#endregion

		#region ファイル削除ページ
		static string GenerateManageDeleteSpecialPage(WebHeaderInfo info)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\del_special.html");

			temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);
			temp = temp.Replace("&guid;",info.GUID.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%title%",info.Title);

			return temp;
		}
		#endregion

		#region 自動クリーンアップ
		static string GenerateManageCleanup(WebHeaderInfo info)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\cleanup.html");

			temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);
			temp = temp.Replace("&guid;",info.GUID.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%title%",info.Title);

			return temp;
		}
		#endregion

		#region パス登録
		static string GenerateManageRegPass(WebHeaderInfo info)
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\blog\\manage\\reg.html");

			temp = HttpHtmlGenerator.ReplaceLocalAddress(temp);
			temp = temp.Replace("&guid;",info.GUID.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%title%",info.Title);

			return temp;
		}
		#endregion

		#region クリーンアップ
		static Guid[] CleanupPosts(WebPostInfo[] posts,BlogData[] data,BlogCommentData[] comms,BlogTrackbackData[] trackbacks,BlogKeyData[] keys,BlogFileData[] files,BlogMgrData about,BlogMgrData css,BlogMgrData desc,BlogMgrData menu,BlogMgrData head,BlogMgrData base_template)
		{
			ArrayList list = new ArrayList();list.AddRange(posts);

			foreach(BlogData d in data)
				RemovePost(list,d.Id);

			foreach(BlogCommentData d in comms)
				RemovePost(list,d.Id);

			foreach(BlogTrackbackData d in trackbacks)
				RemovePost(list,d.Id);

			foreach(BlogKeyData d in keys)
				RemovePost(list,d.Id);

			foreach(BlogFileData d in files)
				RemovePost(list,d.Id);

			if(about!=null) RemovePost(list,about.Id);
			if(css!=null)RemovePost(list,css.Id);
			if(desc!=null)RemovePost(list,desc.Id);
			if(menu!=null)RemovePost(list,menu.Id);
			if(head!=null)RemovePost(list,head.Id);
			if(base_template!=null)RemovePost(list,base_template.Id);

			ArrayList guids = new ArrayList();
			foreach(WebPostInfo post in list)
				guids.Add(post.GUID);
			return (Guid[])guids.ToArray(typeof(Guid));
		}
		#endregion

		#region 管理更新
		public static byte[] ProcessManage(StringDictionary param)
		{
			if(HttpServer.HasAuthority(param))
			{
				WebHeaderInfo info;
				try
				{
					Guid id = new Guid(param["id"]);
					info = WebCacheReader.ReadHeader(id);
					if(info == null) throw new Exception();
				}
				catch
				{
					return ErrorHtml.Generate("パラメータ：IDが不正です。",null);
				}

				// キャッシュ読み込み
				BlogData[] data;
				BlogCommentData[] comms;
				BlogTrackbackData[] trackbacks;
				BlogKeyData[] keys;
				BlogFileData[] files;
				BlogMgrData about,css,desc,menu,head,base_template;
				WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
				ReadBlogCache(info,posts,out data,out comms,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out head,out base_template);

				ArrayList delete_guids = new ArrayList(CleanupPosts(posts,data,comms,trackbacks,keys,files,about,css,desc,menu,head,base_template));

				switch(param["page"])
				{
					case "1":	// 基本情報
					{
						if(param["title"] == null) param["title"] = "";
						if(param["category"] == null) param["category"] = "";

						if(param["title"] == "" || param["category"] == "") return GeneratePostError("タイトルまたはカテゴリが空です",info);

						if(param["title"].Length > 30) param["title"] = param["title"].Substring(0,30);
						if(param["category"].Length > 20) param["category"] = param["category"].Substring(0,20);

						string result = Interface.ManagePage(info.GUID,param["title"],param["category"],new Guid[0]);
						if(result == "")
							return GeneratePostError("基本情報の更新に成功しました",info);
						else
							return GeneratePostError(result,info);
					}
					case "2":	// エントリの削除
					{
						ArrayList list = new ArrayList();list.AddRange(posts);

						// 日付+タイトルなアイテムを削除
						int entries = 0;
						foreach(DictionaryEntry entry in param)
						{
							if(((string)entry.Key).StartsWith("del_"))
							{
								try
								{
									DateTime dt = DateTime.Parse(((string)entry.Key).Substring(4,10));
									string title = (string)entry.Value;

									bool flag = false;

									foreach(BlogData d in data)
									{
										if(d.Date.Date == dt.Date && d.Title == title)
										{
											flag = true;
											for(int i=0;i<list.Count;i++)
												if(((WebPostInfo)list[i]).GUID.CompareTo(d.Id) == 0)
												{
													delete_guids.Add(((WebPostInfo)list[i]).GUID);
												}
										}
									}

									if(flag) entries++;
								}
								catch
								{
								}
							}
						}

						if(delete_guids.Count > 0)
						{
							string result = Interface.ManagePage(info.GUID,info.Title,info.Category,(Guid[])delete_guids.ToArray(typeof(Guid)));

							int deleted = delete_guids.Count;
							list.Clear();list = null;
						
							if(result == null || result == "")
								return GeneratePostError("管理更新完了。" + entries + "エントリー、" + deleted + "ポスト削除しました。",info);
							else
								return GeneratePostError(result,info);
						}
						else
							return GeneratePostError("削除する記事がありません",info);
					}
					case "3":	// コメントの削除
					{
						// 日付+タイトルなアイテムを削除
						foreach(DictionaryEntry entry in param)
						{
							if(((string)entry.Key).StartsWith("del_"))
							{
								try
								{
									Guid id = new Guid((string)entry.Value);
									delete_guids.Add(id);
								}
								catch
								{
								}
							}
						}

						if(delete_guids.Count > 0)
						{
							string result = Interface.ManagePage(info.GUID,info.Title,info.Category,(Guid[])delete_guids.ToArray(typeof(Guid)));

							int deleted = delete_guids.Count;

							if(result == null || result == "")
								return GeneratePostError("管理更新完了。" + deleted + "件のコメントを削除しました。",info);
							else
								return GeneratePostError(result,info);
						}
						else
							return GeneratePostError("削除するコメントがありません",info);
					}
					case "4":	// 自動クリーンアップ
					{
						Guid[] guids = (Guid[])delete_guids.ToArray(typeof(Guid));

						if(guids.Length > 0)
						{
							string result = Interface.ManagePage(info.GUID,info.Title,info.Category,guids);

							Debug.WriteLine("Step8 Clear");
							int deleted = guids.Length;
							Array.Clear(guids,0,guids.Length);guids = null;
						
							if(result == null || result == "")
								return GeneratePostError("クリーンアップ成功。" + deleted + "件削除しました。",info);
							else
								return GeneratePostError(result,info);
						}
						else
							return GeneratePostError("クリーンアップ対象データはありませんでした",info);
					}
					case "6":	// パス登録
					{
						if(param.ContainsKey("delete") && param["delete"] == "all")
						{
							Guid[] guids = new Guid[keys.Length];
							for(int i=0;i<guids.Length;i++)
								guids[i] = keys[i].Id;
							if(guids.Length > 0)
							{
								string ret = Interface.ManagePage(info.GUID,info.Title,info.Category,guids);
								if(ret == "")
									return GeneratePostError(guids.Length + "つの登録パスワードを削除しました。",info);
								else
									return GeneratePostError(ret,info);
							}
							else
								return GeneratePostError("削除する登録パスワードがありませんでした",info);
						}
						else
						{
							string privateKey = "";
							if(ExistsManageKey(Encoding.ASCII.GetString(info.Trip),out privateKey))
							{
								byte[] key = BlogKeyData.Encode(Encoding.ASCII.GetBytes(privateKey),param["user"]+":"+param["pass"]);

								string tempFile = CopyTempTrip(Encoding.ASCII.GetString(info.Trip),info.GUID,param,info,keys);
								Guid id = Interface.PostToBbs(info.GUID,"","",TextUtility.XmlToTrip(Encoding.ASCII.GetString(info.Trip)),key,"key");
								try
								{
									File.Delete(tempFile);
								}
								catch
								{
								}
								if(id != Guid.Empty)
								{
									return GeneratePostError("鍵登録に成功しました",info);
								}
								else
								{
									return GeneratePostError("鍵登録に失敗しました",info);
								}
							}
							else
								return GeneratePostError("管理鍵が見つかりませんでした",info);
						}
					}
					case "7":	// ファイルの削除
					{
						int entries = 0;

						// 同じファイル名のアイテムを削除
						foreach(DictionaryEntry entry in param)
						{
							if(((string)entry.Key).StartsWith("del_"))
							{
								string filename = ((string)entry.Value).ToLower();

								foreach(BlogFileData d in files)
								{
									if(d.FileName.ToLower() == filename)
									{
										delete_guids.Add(d.Id);
										entries++;
									}
								}
							}
						}

						if(delete_guids.Count > 0)
						{
							string result = Interface.ManagePage(info.GUID,info.Title,info.Category,(Guid[])delete_guids.ToArray(typeof(Guid)));

							int deleted = delete_guids.Count;

							if(result == null || result == "")
								return GeneratePostError("管理更新完了。" + entries + "個のファイル、" + deleted +"ポストを削除しました。",info);
							else
								return GeneratePostError(result,info);
						}
						else
							return GeneratePostError("削除するファイルがありません",info);
					}
					case "8":
					{
						switch(param["specialname"])
						{
							case "css":
								if(css != null) delete_guids.Add(css.Id);
								break;
							case "about":
								if(about != null) delete_guids.Add(about.Id);
								break;
							case "desc":
								if(desc != null) delete_guids.Add(desc.Id);
								break;
							case "menu":
								if(menu != null) delete_guids.Add(menu.Id);
								break;
							case "head":
								if(head != null) delete_guids.Add(head.Id);
								break;
							case "base":
								 if(base_template != null) delete_guids.Add(base_template.Id);
								break;
						}
						string result = Interface.ManagePage(info.GUID,info.Title,info.Category,(Guid[])delete_guids.ToArray(typeof(Guid)));

						int deleted = delete_guids.Count;

						if(result == null || result == "")
							return GeneratePostError("管理更新完了。計" + deleted +"ポストを削除しました。",info);
						else
							return GeneratePostError(result,info);
					}
					default:
						return GeneratePostError("エラー",info);
				}
			}
			else
				throw HttpException.UnauthorizedException;
		}
		static void RemovePost(ArrayList list,Guid id)
		{
			for(int i=0;i<list.Count;i++)
				if(((WebPostInfo)list[i]).GUID.CompareTo(id) == 0)
				{
					list.RemoveAt(i);
					break;
				}
		}
		static byte[] GeneratePostError(string desc,WebHeaderInfo info)
		{
			string temp = ReadBaseManageTemplate(info);
			temp = temp.Replace("&contents;",desc);
			return HttpServerSetting.HtmlEncoding.GetBytes(temp);
		}
		#endregion

		#endregion

		#region 置き換え
		const string DateFormat1 = "MMMM dd,yyyy";
		const string DateFormat2 = "yyyy\\/MM\\/dd";
		const string TimeFormat1 = "hh\\:mm tt";
		const string TimeFormat2 = "HH\\:mm";
		static string ReplaceBlogDate(string temp,DateTime date)
		{
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_date1%",date.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_date2%",date.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_date3%",date.ToString("MMMM yyyy",System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_date4%",date.ToString("yyyy\\/MM"));
			return temp;
		}
		static string ReplaceBlogData(string temp,BlogData data)
		{
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_body%",data.Body);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_date1%",data.Date.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_date2%",data.Date.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_guid%",data.Id.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_name%",data.Name);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_posted1%",data.Posted.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_posted2%",data.Posted.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_posted_time1%",data.Posted.ToString(TimeFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_posted_time2%",data.Posted.ToString(TimeFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_title%",data.Title);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_entry_title_hash%",data.TitleHash);
			return temp;
		}
		static string ReplaceBlogFileData(string temp,BlogFileData data)
		{
			if(data.RawData == null) data.RawData = new byte[0];
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_file_size%",TextUtility.SizeToString(data.RawData.Length));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_file_guid%",data.Id.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_file_posted1%",data.Posted.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_file_posted2%",data.Posted.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_file_filename%",data.FileName);
			return temp;
		}
		static string ReplaceBlogCommentData(string temp,BlogCommentData data)
		{
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_text%",data.Comment);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_date1%",data.Date.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_date2%",data.Date.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_guid%",data.Id.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_name%",data.Name);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_id%",BlogData.GetTitleHash(data.Id.ToByteArray()));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_title%",data.EntryTitle);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_title_hash%",BlogData.GetTitleHash(data.EntryTitle));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_posted1%",data.Posted.ToString(DateFormat1 + " " + TimeFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_posted2%",data.Posted.ToString(DateFormat2 + " " + TimeFormat2));
			if(data.Trip != null)
			{
				string trip = TextUtility.XmlToTrip(Encoding.ASCII.GetString(data.Trip));
				temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_trip%",trip);
				temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_tripname%",string.Concat(data.Name.Replace("◆",""),"◆",trip));
			}
			else
			{
				temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_trip%","");
				temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_comm_tripname%",data.Name.Replace("◆",""));
			}
			return temp;
		}
		static string ReplaceBlogTrackbackData(string temp,BlogTrackbackData data)
		{
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_blogname%",data.BlogName);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_date1%",data.Date.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_date2%",data.Date.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_guid%",data.Id.ToString());
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_quotation%",data.Quotation);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_posted1%",data.Posted.ToString(DateFormat1,System.Globalization.DateTimeFormatInfo.InvariantInfo));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_posted2%",data.Posted.ToString(DateFormat2));
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_title%",data.Title);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_entry%",data.EntryTitle);
			temp = TemplateFieldReplacer.ReplaceField(temp,"%blog_track_url%",data.URL);
			return temp;
		}
		#endregion

		#region 検索
		public static void Search(WebHeaderInfo info,Regex regex,string temp_loop,ref StringBuilder temp,string local,bool include_comm,ref int hitThreads,ref int totalRes,ref int hitRes)
		{
			BlogData[] entries;
			BlogCommentData[] all_comms;
			BlogTrackbackData[] trackbacks;
			BlogKeyData[] keys;
			BlogFileData[] files;
			BlogMgrData about,css,desc,menu,head,base_template;
			ReadBlogCache(info,out entries,out all_comms,out trackbacks,out keys,out files,out about,out css,out desc,out menu,out head,out base_template);
			bool incremented = false;

			foreach(BlogData data in entries)
			{
				totalRes++;
				string body = GetClearText(info,data);
				Match m = regex.Match(body);
				if(m.Success)
				{
					hitRes++;
					string loop = TemplateFieldReplacer.ReplaceField(temp_loop,"%_summary%","..." + HttpUtility.HtmlEncode(m.Groups["summary"].Value) + "...<br />");
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_date%",data.Posted.ToString("yyyy年MM月dd日 hh時mm分ss秒"));
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",info.Title + " 「" + data.Title + "」");
					loop = TemplateFieldReplacer.ReplaceField(loop,"%_url%",string.Concat(local,info.GUID.ToString(),data.Date.ToString("\\/yyyy\\/MM\\/dd\\/"),data.TitleHash));
					temp.Append(loop);
					if(!incremented)
					{
						incremented = true;
						hitThreads++;
					}
				}

				if(include_comm)
				{
					BlogCommentData[] comms = ExtractComments(all_comms,data.Date,data.Title);
					StringBuilder comm_temp = new StringBuilder();
					foreach(BlogCommentData comm in comms)
					{
						totalRes++;
						Match cm = regex.Match(comm.Comment);
						if(cm.Success)
						{
							hitRes++;
							string loop = TemplateFieldReplacer.ReplaceField(temp_loop,"%_summary%","..." + HttpUtility.HtmlEncode(cm.Groups["summary"].Value) + "...<br />");
							loop = TemplateFieldReplacer.ReplaceField(loop,"%_date%",comm.Posted.ToString("yyyy年MM月dd日 hh時mm分ss秒"));
							if(comm.Trip == null)
								loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",comm.Name);
							else
								loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",comm.Name + "◆" + TextUtility.XmlToTrip(Encoding.ASCII.GetString(comm.Trip)));
							loop = TemplateFieldReplacer.ReplaceField(loop,"%_url%",string.Concat(local,info.GUID.ToString(),data.Date.ToString("\\/yyyy\\/MM\\/dd\\/"),data.TitleHash,"#comments"));
							comm_temp.Append(loop);
						}
					}
					if(comm_temp.Length > 0)
					{
						if(!m.Success)
						{
							string loop = TemplateFieldReplacer.ReplaceField(temp_loop,"%_summary%","");
							loop = TemplateFieldReplacer.ReplaceField(loop,"%_date%",data.Posted.ToString("yyyy年MM月dd日 hh時mm分ss秒"));
							loop = TemplateFieldReplacer.ReplaceField(loop,"%_title%",info.Title + " 「" + data.Title + "」");
							loop = TemplateFieldReplacer.ReplaceField(loop,"%_url%",string.Concat(local,info.GUID.ToString(),data.Date.ToString("\\/yyyy\\/MM\\/dd\\/"),data.TitleHash));
							temp.Append(loop);

							if(!incremented)
							{
								incremented = true;
								hitThreads++;
							}
						}
						temp.Append("<blockquote>");
						temp.Append(comm_temp.ToString());
						temp.Append("</blockquote>");
					}
				}
			}
		}
		#endregion
	}

	#region Blogデータ構造体
	class BlogDataDivider
	{
		public static object Divide(WebHeaderInfo info,WebPostInfo post)
		{
			try
			{
				string[] fs = post.FileName.Split('\0');
				string[] ns = post.Name.Split('\0');

				if(post.Trip != null && ByteArrayComparer.Comparer(info.Trip,post.Trip) == 0)
				{
					switch(fs[0].ToLower())
					{
						case "head":
						case "about":
						case "css":
						case "desc":
						case "base":
						case "menu":
						{
							BlogMgrData data = new BlogMgrData();
							data.Body = post.Body;
							data.Id = post.GUID;
							data.Name = post.Name;
							data.Posted = post.Posted;
							data.Sign = post.Signature;
							data.Trip = post.Trip;
							data.Title = fs[0].ToLower();
							return data;
						}
						case "key":
						{
							BlogKeyData data = new BlogKeyData();
							data.CryptedData = post.Data;
							data.Id = post.GUID;
							data.Posted = post.Posted;
							return data;
						}
						case "file":
						{
							BlogFileData data = new BlogFileData();
							data.FileName = fs[1];
							data.Id = post.GUID;
							data.Posted = post.Posted;
							data.RawData = post.Data;
							data.Sign = post.Signature;
							data.Trip = post.Trip;
							return data;
						}
						default:
						{
							BlogData data = new BlogData();
							data.Body = post.Body;
							data.Id = post.GUID;
							data.Name = post.Name;
							data.Posted = post.Posted;
							data.Sign = post.Signature;
							data.Trip = post.Trip;
							data.Date = DateTime.Parse(fs[0]);
							data.Title = fs[1];
							return data;
						}
					}
				}
				else
				{
					if(ns.Length == 2 && fs.Length == 3)
					{
						BlogTrackbackData data = new BlogTrackbackData();
						data.Quotation = post.Body;
						data.Date = DateTime.Parse(fs[0]);
						data.Id = post.GUID;
						data.URL = fs[1];
						data.EntryTitle = fs[2];
						data.BlogName = ns[0];
						data.Title = ns[1];
						data.Posted = post.Posted;
						return data;
					}
					else if(ns.Length != 2 && fs.Length == 2)
					{
						BlogCommentData data = new BlogCommentData();
						data.Comment = post.Body;
						data.Date = DateTime.Parse(fs[0]);
						data.EntryTitle = fs[1];
						data.Id = post.GUID;
						data.Name = post.Name;
						data.Posted = post.Posted;
						data.Sign = post.Signature;
						data.Trip = post.Trip;
						return data;
					}
					else
						return null;
				}
			}
			catch
			{
				return null;
			}
		}
	}

	class BlogDateComparer : IComparer
	{
		int _v;
		public BlogDateComparer(int value1)
		{
			_v = value1;
		}

		#region IComparer メンバ

		public int Compare(object x, object y)
		{
			TimeSpan span,span2;
			if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogData)))
			{
				span = ((BlogData)x).Date.Date.Subtract(((BlogData)y).Date.Date);
				span2 = ((BlogData)x).Posted.Subtract(((BlogData)y).Posted);
			}
			else if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogCommentData)))
			{
				span = ((BlogCommentData)x).Date.Date.Subtract(((BlogCommentData)y).Date.Date);
				span2 = ((BlogCommentData)x).Posted.Subtract(((BlogCommentData)y).Posted);
			}
			else
				throw new ArgumentException();

			int diff = (int)(span2.TotalSeconds - span.TotalSeconds);
			int border = 60 * 60 * 24 * 7;
			if(diff > border) diff = border;
			if(diff < -border) diff = -border;
				
			return (int)(span.TotalDays * border + diff) * _v;
		}

		#endregion

	}
	class BlogPostedDateComparer : IComparer
	{
		int _v;
		public BlogPostedDateComparer(int value)
		{
			_v = value;
		}

		#region IComparer メンバ

		public int Compare(object x, object y)
		{
			if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogData)))
			{
				return _v*(int)Math.Sign(((TimeSpan)((BlogData)x).Posted.Subtract(((BlogData)y).Posted)).TotalSeconds);
			}
			else if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogMgrData)))
			{
				return _v*(int)Math.Sign(((TimeSpan)((BlogMgrData)x).Posted.Subtract(((BlogMgrData)y).Posted)).TotalSeconds);
			}
			else if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogCommentData)))
			{
				return _v*(int)Math.Sign(((TimeSpan)((BlogCommentData)x).Posted.Subtract(((BlogCommentData)y).Posted)).TotalSeconds);
			}
			else if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogTrackbackData)))
			{
				return _v*(int)Math.Sign(((TimeSpan)((BlogTrackbackData)x).Posted.Subtract(((BlogTrackbackData)y).Posted)).TotalSeconds);
			}
			else if(x.GetType() == y.GetType() && (y.GetType() == typeof(BlogFileData)))
			{
				return _v*(int)Math.Sign(((TimeSpan)((BlogFileData)x).Posted.Subtract(((BlogFileData)y).Posted)).TotalSeconds);
			}
			else
				throw new ArgumentException();
		}

		#endregion

	}


	class BlogData
	{
		public string		Title;
		public string		Name;
		public string		Body;
		public DateTime	Posted;
		public DateTime	Date;
		public Guid			Id;

		public byte[]		Trip;
		public byte[]		Sign;

		string _hash = "";
		public static string GetTitleHash(byte[] title)
		{
			ICSharpCode.SharpZipLib.Checksums.Crc32 crc = new ICSharpCode.SharpZipLib.Checksums.Crc32();
			crc.Update(title);
			return crc.Value.ToString("x");
		}
		public static string GetTitleHash(string title)
		{
			return GetTitleHash(System.Text.Encoding.UTF8.GetBytes(title));
		}
		public string TitleHash
		{
			get
			{
				if(_hash == "")
				{
					_hash = GetTitleHash(this.Title);
				}

				return _hash;
			}
		}
	}

	class BlogMgrData
	{
		public string		Title;
		public string		Name;
		public string		Body;
		public DateTime	Posted;
		public Guid			Id;

		public byte[]		Trip;
		public byte[]		Sign;
	}

	class BlogCommentData
	{
		public string		EntryTitle;
		public string		Name;
		public string		Comment;
		public DateTime	Posted;
		public DateTime	Date;
		public Guid			Id;

		public byte[]		Trip;
		public byte[]		Sign;
	}

	class BlogTrackbackData
	{
		public string		EntryTitle;
		public string		Quotation;
		public string		URL;
		public string		BlogName;
		public string		Title;
		public DateTime	Date;
		public DateTime	Posted;
		public Guid			Id;
	}

	class BlogFileData
	{
		public string		FileName;
		public byte[]		RawData;
		public DateTime	Posted;
		public Guid			Id;
		public byte[]		Trip;
		public byte[]		Sign;
	}

   class BlogKeyData
	{
		public byte[]		CryptedData;
		public DateTime	Posted;
		public Guid			Id;

		public static byte[] Encode(byte[] mgrKey,string userpass)
		{
			byte[] key;
			byte[] ret;
			using(MD5CryptoServiceProvider hash = new MD5CryptoServiceProvider())
			{
				hash.Initialize();
				key = hash.ComputeHash(Encoding.UTF8.GetBytes(userpass));
				hash.Clear();
			}

			using(RijndaelManaged aes = new RijndaelManaged())
			{
				using(MemoryStream ms = new MemoryStream())
				{
					using(CryptoStream stream = new CryptoStream(ms,aes.CreateEncryptor(key,new byte[aes.IV.Length]),CryptoStreamMode.Write))
					{
						stream.Write(mgrKey,0,mgrKey.Length);
						stream.Flush();
						stream.Close();
					}
					ms.Close();
					ret = ms.ToArray();
				}
			}
			return ret;
		}

		public byte[] Decode(string userpass)
		{
			byte[] key;
			using(MD5CryptoServiceProvider hash = new MD5CryptoServiceProvider())
			{
				hash.Initialize();
				key = hash.ComputeHash(Encoding.UTF8.GetBytes(userpass));
				hash.Clear();
			}

			byte[] mgrKey;

			using(RijndaelManaged aes = new RijndaelManaged())
			{
				using(MemoryStream ms = new MemoryStream(CryptedData))
				{
					int readed = 0;
					byte[] buffer = new byte[1024 * 10];
					using(CryptoStream stream = new CryptoStream(ms,aes.CreateDecryptor(key,new byte[aes.IV.Length]),CryptoStreamMode.Read))
					{
						while(true)
						{
							int ret = stream.Read(buffer,readed,buffer.Length-readed);
							if(ret == 0)break;
							readed += ret;
						}
					}
					mgrKey = new byte[readed];
					Array.Copy(buffer,0,mgrKey,0,readed);
					ms.Close();
				}
			}
			return mgrKey;
		}
	}
	#endregion
}
