﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.Text;

namespace RinGOch.HtmlGenerators
{
	public class ChatGenerator
	{
		public static string[] PublicRooms = new string[]
		{
			"学校説明会",
			"校庭のベンチ",
			"Global PDF",
			"多目的室",
			"放送室",
			"食堂",
			"体育館の裏",
			"屋上",
			"図書室",
			"音楽室",
			"美術室",
			"演劇部",
			"工作教室",
			"職員室",
			"校庭",
			"電算室",
			"自習室",
			"人生相談室",
			"視聴覚室",
			"更衣室"
		};

		public static byte[] Generate(StringDictionary parameters)
		{
			try
			{
				string temp = "";
				if(parameters["hash"] != null && parameters["hash"] != "")
					return ReturnIconBinary(parameters);
				else if(parameters["room"] == null || parameters["room"] == "")
					temp = GenerateRoomList();
				else
					temp = GenerateRoom(parameters);

				return System.Text.Encoding.Default.GetBytes(temp);
			}
			catch(HttpException)
			{
				throw;
			}
			catch(Exception ex)
			{
				return ErrorHtml.Generate("chat.p2pを処理中に例外が発生しました",ex);
			}
		}

		static byte[] ReturnIconBinary(StringDictionary parameters)
		{
			byte[] hash = Convert.FromBase64String(parameters["hash"]);
			return Interface.ChatGetIconBinary(hash);
		}

		static string GenerateRoomList()
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\chat.html");
			string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CHAT ROOM LIST LOOP-->","<!--END CHAT ROOM LIST LOOP-->");
			StringBuilder html = new StringBuilder();

			// HTMLヘッダを追加
			html.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN HEADER-->","<!--END HEADER-->"));

			// ルームヘッダを追加
			html.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CHAT ROOM LIST HEADER-->","<!--END CHAT ROOM LIST HEADER-->"));

			foreach(string room in PublicRooms)
			{
				string msg = loop;
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_room%",room,System.Text.Encoding.Default);
				html.Append(msg);
			}

			// ルームフッタを追加
			html.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CHAT ROOM LIST FOOTER-->","<!--END CHAT ROOM LIST FOOTER-->"));

			return html.ToString();
		}

		static string GenerateIconList()
		{
			string[] files = Interface.ChatGetIcons();
			Array.Sort(files);
			StringBuilder html = new StringBuilder();
			foreach(string file in files)
				html.Append("<option value=\"" + HttpUtility.HtmlEncode(file) + "\">" + HttpUtility.HtmlEncode(System.IO.Path.GetFileNameWithoutExtension(file)) + "</option>");
			return html.ToString();
		}

		static string GenerateRoom(StringDictionary param)
		{
			int num = (TextUtility.IsNumeric(param["type"])?int.Parse(param["type"]):1);
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\chat" + num.ToString() + ".html");
			string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CAHT LOOP-->","<!--END CAHT LOOP-->");
			StringBuilder html = new StringBuilder();

			int count = 10;
			if(param["count"]!=null && TextUtility.IsNumeric(param["count"].TrimEnd(' ','\0')))
				count = int.Parse(param["count"].TrimEnd(' ','\0'));

			// HTMLヘッダを追加
			html.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN HEADER-->","<!--END HEADER-->"));

			// ルームヘッダを追加
			html.Append(
				TemplateFieldReplacer.ReplaceField(
					TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CHAT HEADER-->","<!--END CHAT HEADER-->"),
					"%_chat_room%",param["room"],System.Text.Encoding.Default).Replace("%_chat_count%",count.ToString())
					.Replace("%_chat_type%",param["type"])
					.Replace("%_chat_sort%",param["sort"])
					.Replace("%_chat_iconlist%",GenerateIconList())
				);

			string[] posts = Interface.ChatReadMessage(param["room"],count);

			if(posts == null)
				return ErrorHtml.GenerateString("指定されたチャットルームは存在しません",null);

			if( !(param["sort"]!=null && param["sort"].ToLower() == "ascending") )
			{
				Array.Reverse(posts);
			}

			foreach(string post in posts)
			{
				string[] items = post.Split('\t');
				string msg = loop;
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_room%",param["room"],System.Text.Encoding.Default);
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_name%",items[2],System.Text.Encoding.Default);
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_body%",
					HttpHtmlGenerator.CreateURLLink(
					HttpUtility.HtmlEncode(
					items[3]
					)
					,true),System.Text.Encoding.Default
					);
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_id%",items[1],System.Text.Encoding.Default);
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_time%",new DateTime(long.Parse(items[0])).ToString("HH\\:mm\\:ss"),System.Text.Encoding.Default);
				msg = TemplateFieldReplacer.ReplaceField(msg,"%_chat_icon%",items[4],System.Text.Encoding.Default);
				html.Append(msg);
			}

			// ルームフッタを追加
			html.Append(
				TemplateFieldReplacer.ReplaceField(
				TextUtility.GetMiddleRangeString(temp,"<!--BEGIN CHAT FOOTER-->","<!--END CHAT FOOTER-->"),
				"%_chat_room%",param["room"],System.Text.Encoding.Default).Replace("%_chat_count%",param["count"])
				.Replace("%_chat_type%",param["type"])
				.Replace("%_chat_sort%",param["sort"])
				.Replace("%_chat_iconlist%",GenerateIconList())
				);

			return html.ToString();
		}
	}
}
