﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Specialized;

namespace RinGOch.HtmlGenerators
{
	public class ManageGenerator
	{
		public static byte[] Generate(StringDictionary param)
		{
			if(!HttpServer.HasAuthority(param))
			{
				throw HttpException.UnauthorizedException;
				//return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\gateway\\unsupported.html"));
			}
			else
			{
				try
				{
					Guid id = new Guid(param["id"]);
					WebHeaderInfo info = WebCacheReader.ReadHeader(id);
					if(info != null)
					{
						switch(info.Type)
						{
							case WebType.FileBoard:
							case WebType.ImageBoard:
							case WebType.TextOnlyBoard:
							{
								try
								{
									return GenerateBbsManagePage(info,param);
								}
								catch(Exception ex)
								{
									return ErrorHtml.Generate("BBS管理ページのHTMLを生成中に例外が発生しました",ex);
								}
							}
							case WebType.Blog:
							{
								throw new HttpException(302,"Found","Location","blogmgr.p2p?page=0&id=" + id.ToString());
							}
							case WebType.Wiki:
							{
								try
								{
									if(param[""].StartsWith("manage_wiki.p2p"))
										return WikiManagerList(info,param);
									else if(param[""].StartsWith("manage.p2p"))
										return WikiManagerIndex(info,param);
									else
										return GenerateChoicePage();
								}
								catch(Exception ex)
								{
									return ErrorHtml.Generate("Wiki管理ページのHTMLを生成中に例外が発生しました",ex);
								}
							}
							default:
								return GenerateChoicePage();
						}
					}
					else
						return ErrorHtml.Generate("指定されたページのキャッシュを読み取れません。\r\nページ管理を行うにはキャッシュを保持している必要があります",null);
				}
				catch(HttpException)
				{
					throw;
				}
				catch(Exception)
				{				
					return GenerateChoicePage();
				}
			}
		}

		#region 管理するスレの選択
		static byte[] GenerateChoicePage()
		{
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\manage.html");
			StringBuilder buffer = new StringBuilder();
			Guid[] pages = Interface.GetManagePages();

			buffer.Append(temp.Substring(0,temp.IndexOf("<!--BEGIN PROGRAMMABLE-->")));

			if(pages.Length > 0)
			{
				string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
				foreach(Guid page in pages)
				{
					WebHeaderInfo info = WebCacheReader.ReadHeader(page);
					if(info != null)
					{
						buffer.Append(TemplateFieldReplacer.ReplaceCommonHeaderFields(loop,info));
					}
				}
			}
			else
			{
				buffer.Append(TextUtility.GetMiddleRangeString(temp,"<!--BEGIN NO MANAGEMENT-->","<!--END NO MANAGEMENT-->"));
			}

			buffer.Append(temp.Substring(temp.IndexOf("<!--END PROGRAMMABLE-->")+"<!--END PROGRAMMABLE-->".Length));
			return HttpServerSetting.HtmlEncoding.GetBytes(buffer.ToString());
		}
		#endregion

		#region BBS管理表示
		static byte[] GenerateBbsManagePage(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			string header = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\bbs_header.html");	
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\bbs_res.html");	
			string res_template_text = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN TEXT-->","<!--END TEXT-->");
			string res_template_image = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN IMAGE-->","<!--END IMAGE-->");
			string res_template_file = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN FILE-->","<!--END FILE-->");
			string res_template_abon = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN ABON-->","<!--END ABON-->");
			string footer = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\bbs_footer.html");

			info = WebCacheReader.ReadHeader(info.GUID);
			if(info == null)
			{
				return HttpServerSetting.HtmlEncoding.GetBytes("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html;charset=UTF-8\"></head><body>キャッシュが見つかりません。BBSを管理するには手元にキャッシュがある必要があります</body></html>");
			}

			// ヘッダ＆フッタ置き換え
			header = header.Replace("%_category_selected_list%",TemplateFieldReplacer.GenerateCategoryList(info.Category));
			header = TemplateFieldReplacer.ReplaceBbsFields(header,info,null,false,"",0,"","");
			footer = TemplateFieldReplacer.ReplaceBbsFields(footer,info,null,false,"",0,"","");

			// ヘッダ追加
			html.Append(header);

			// ポスト読み込み
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);

			int index = 0;
			foreach(WebPostInfo post in posts)
			{
				html.Append(TemplateFieldReplacer.ReplaceBbsFields(
					BBSGenerator.ChoicePostTemplate(post,false,res_template_text,res_template_image,res_template_file,res_template_text,res_template_image,res_template_file),
					info,post,true,BBSGenerator.GetThumbnailUrl(info,post),++index,"",""));
			}

			// フッタ追加
			html.Append(footer);

			// リターン
			return HttpServerSetting.HtmlEncoding.GetBytes(html.ToString());
		}
		#endregion

		#region Wikiの管理表示
		public static byte[] WikiManagerIndex(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\wiki_index.html")
				.Replace("%GUID%",info.GUID.ToString());

			return HttpServerSetting.HtmlEncoding.GetBytes(temp);
		}
		public static byte[] WikiManagerList(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\manage\\wiki_list.html");
			string pageTemp = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN PAGE LOOP-->","<!--END PAGE LOOP-->");

			// ヘッダを書き込み
			string header = temp.Substring(0,temp.IndexOf("<!--BEGIN PAGE LOOP-->"));
			header = TemplateFieldReplacer.ReplaceCommonHeaderFields(header,info)
				.Replace("%_category_selected_list%",TemplateFieldReplacer.GenerateCategoryList(info.Category));
			html.Append(header);

			// ポストを投稿順に並べ替え
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			Array.Sort(posts,new WebPostInfoComparer());
			Array.Reverse(posts);

			// ページを取得
			ArrayList names = new ArrayList();
			foreach(WebPostInfo post in posts)
				if(post.FileName!=null && !names.Contains(post.FileName))
					names.Add(post.FileName);

			// ページループ
			foreach(string page in names)
			{
				string loop = TextUtility.GetMiddleRangeString(pageTemp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
				StringBuilder temp2 = new StringBuilder();
				temp2.Append(pageTemp.Substring(0,pageTemp.IndexOf("<!--BEGIN LOOP-->")));
				foreach(WebPostInfo post in posts)
					if(post.FileName.Equals(page))
					{
						temp2.Append(TemplateFieldReplacer.ReplaceCommonPostFields(TemplateFieldReplacer.ReplaceCommonHeaderFields(loop,info),post));
					}
				temp2 = temp2.Replace("%WIKINAME%",(page==""?"FrontPage":page)).Replace("%ENC_WIKINAME%",HttpUtility.UrlEncode(page));
				temp2.Append(pageTemp.Substring(pageTemp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length));
				html.Append(temp2.ToString());
			}

			html.Append(temp.Substring(temp.IndexOf("<!--END PAGE LOOP-->")+"<!--END PAGE LOOP-->".Length));
			return HttpServerSetting.HtmlEncoding.GetBytes(html.ToString());
		}
		#endregion
	}
}
