﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;
using RinGOch.WikiEngine;

namespace RinGOch.HtmlGenerators
{
	public class WikiGenerator
	{
		#region 分岐
		public static byte[] Generate(WebHeaderInfo info,StringDictionary parameters,out string mime,StringDictionary response)
		{
			try
			{
				try
				{
					string fname = Path.GetDirectoryName((string)parameters[""]);
					Guid id = new Guid(fname);
					return ReadAttachFile(info,parameters,id,out mime,response);
				}
				catch(HttpException){throw;}
				catch{}

				if(parameters[""] == null || parameters[""]=="")
					parameters[""] = "wiki.p2p";

				if((parameters.ContainsKey("name") && parameters["name"] == "") || (parameters[""].ToLower() == "wiki.p2p" && !parameters.ContainsKey("name")))
					parameters["name"] = "FrontPage";

				mime = HttpServer.GetMimeType(parameters[""].ToLower());

				switch(parameters[""].ToLower())
				{
					case "edit.p2p":
						mime = "text/html";
						return HttpServerSetting.HtmlEncoding.GetBytes(GenerateWikiEditPage(info,parameters));
					case "list.p2p":
						mime = "text/html";
						return HttpServerSetting.HtmlEncoding.GetBytes(GenerateWikiList(info,parameters));
					case "history.p2p":
						mime = "text/html";
						return HttpServerSetting.HtmlEncoding.GetBytes(GenerateWikiHistory(info,parameters));
					case "preview.p2p":
						mime = "text/html";
						return HttpServerSetting.HtmlEncoding.GetBytes(GenerateWikiPreviewPage(info,parameters));
					case "default.css":
						mime = "text/css";
						return HttpServerSetting.HtmlEncoding.GetBytes(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\default.css"));
					case "attach.p2p":
						mime = "text/html";
						if(parameters.ContainsKey("name"))
							return HttpServerSetting.HtmlEncoding.GetBytes(GenerateAttachPage(info,parameters));
						else
							return HttpServerSetting.HtmlEncoding.GetBytes(GenerateAttachFileList(info,parameters));
					case "":
					case "wiki.p2p":
					default:
						return HttpServerSetting.HtmlEncoding.GetBytes(GenerateWikiPage(info,parameters));
				}
			}
			catch(HttpException)
			{
				throw;
			}
			catch(Exception exp)
			{
				mime = "text/html";
				return ErrorHtml.Generate("Wikiページの生成中に例外が発生しました",exp);
			}
		}

		#endregion

		#region 最新のWikiNamePostを取得
		/// <summary>
		/// 最新・指定のWikiNamePostを取得
		/// </summary>
		/// <param name="id">指定した記事を取得。Emptyの場合は最新の記事</param>
		/// <returns></returns>
		public static WebPostInfo GetNewestWikiName(WebPostInfo[] posts,WebHeaderInfo info,string wikiname,Guid id)
		{
			if(wikiname == "") wikiname="FrontPage";
			WebPostInfo sel_post = null;
			bool freezed = false;

			Array.Sort(posts,new WebPostInfoComparer());

			foreach(WebPostInfo post in posts)
				if(post.WikiName == wikiname)
				{
					if(post.GUID.Equals(id))
					{
						sel_post = post;
						break;
					}

					bool flag = !freezed;

					if(post.Trip != null && ByteArrayComparer.Comparer(post.Trip,info.Trip)==0)
					{
						if(post.IsEffectiveSignature())
						{
							freezed = true;
							flag = true;
						}
					}

					if(flag)
					{
						if(sel_post==null)
							sel_post = post;
						else if(sel_post.Posted < post.Posted)
							sel_post = post;
					}
				}
			return sel_post;
		}
		#endregion

		#region Wikiページ
		static string GenerateWikiPage(WebHeaderInfo info,StringDictionary param)
		{
			DateTime start = DateTime.Now;
			string header = ReadPageHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadPageHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));

			Guid id = Guid.Empty;
			string wikiName = param["name"];
			if(wikiName==null)wikiName="";
			try
			{
				id = new Guid(param["id"]);
			}
			catch{id=Guid.Empty;}
			
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			WebPostInfo sel_post = GetNewestWikiName(posts,info,wikiName,id);
			bool checkSign = true;

			if(sel_post != null)
			{
				if(sel_post.Data == null)
				{
					header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts,ref checkSign);
					string body = Engine.Transform(sel_post.Body,info,posts,sel_post);
					footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts,ref checkSign).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));

					return header + body + footer;
				}
				else
				{
					header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts,ref checkSign);
					string body = Engine.Transform(
						TemplateFieldReplacer.ReplaceWikiFields(
						TemplateFieldReplacer.ReplaceField(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_attach_info.html")
						,"%_wiki_attach_filename%",sel_post.FileName.Substring(sel_post.FileName.IndexOf(":\0")+2))
						,info,sel_post,posts,ref checkSign)
						,info,posts,sel_post);
					footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts,ref checkSign).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
					return header + body + footer;
				}
			}
			else
			{
				sel_post = new WebPostInfo("","",DateTime.Now,Guid.Empty,null,null,wikiName,null);
				header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts,ref checkSign);
				string body = Engine.Transform(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_noname.html"),info,posts,sel_post);
				footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts,ref checkSign).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
				return header + body + footer;
			}
		}
		#endregion

		#region 一覧
		static string GenerateWikiList(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			DateTime start = DateTime.Now;
			string header = ReadListHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadListHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_list.html");
			string loop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			ArrayList names = new ArrayList();

			header = TemplateFieldReplacer.ReplaceWikiFields(header,info,null,posts);//.Replace("%_wikiname%",info.Title + "::ページの一覧").Replace("%_wiki_hierarchical_link%",info.Title + "::ページの一覧");
			html.Append(TemplateFieldReplacer.ReplaceCommonHeaderFields(temp.Substring(0,temp.IndexOf("<!--BEGIN LOOP-->")),info));

			foreach(WebPostInfo post in posts)
			{
				if(post.WikiName!=null && post.WikiName!="" && !names.Contains(post.WikiName) && !post.WikiName.StartsWith("\0"))
				{
					names.Add(post.WikiName);
				}
			}

			names.Sort();

			foreach(string name in names)
			{
				html.Append(TemplateFieldReplacer.ReplaceWikiFields(loop,info,GetNewestWikiName(posts,info,name,Guid.Empty),posts));
			}

			html.Append(TemplateFieldReplacer.ReplaceCommonHeaderFields(temp.Substring(temp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length),info));
			footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,null,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));

			return header + html.ToString() + footer;
		}
		#endregion

		#region 編集ページ
		static string GenerateWikiEditPage(WebHeaderInfo info,StringDictionary param)
		{
			DateTime start = DateTime.Now;
			string header = ReadEditHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadEditHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_edit.html");
			string wikiName = param["name"];
			if(wikiName==null)wikiName="";
			Guid id = Guid.Empty;
			try
			{
				id = new Guid(param["id"]);
			}
			catch{id=Guid.Empty;}
			
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			WebPostInfo sel_post = GetNewestWikiName(posts,info,wikiName,id);

			string trips = "";
			if(sel_post != null && sel_post.Trip != null && ByteArrayComparer.Comparer(sel_post.Trip,info.Trip) == 0)
			{
				if(KeyTools.ExistsManageKey(Encoding.ASCII.GetString(info.Trip)))
				{
					trips = GenerateTripList(info,TextUtility.XmlToTrip(Encoding.ASCII.GetString(sel_post.Trip)));
				}
				else
				{
					header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts);//.Replace("</title>","を編集</title>").Replace("</h1>","を編集</h1>");
					footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
					return string.Concat(header,sel_post.WikiName,"はロック中のため編集できません",footer);
				}
			}

			if(trips == "" ) trips = GenerateTripList(info);
			temp = temp.Replace("%_bbs_triplist%",trips);

			if(sel_post != null)
			{
				temp = TemplateFieldReplacer.ReplaceWikiFields(temp,info,sel_post,posts);
			}
			else
			{
				sel_post = new WebPostInfo("","",DateTime.Now,Guid.NewGuid(),null,null,wikiName,null);
				temp = TemplateFieldReplacer.ReplaceWikiFields(temp,info,sel_post,posts);
			}

			if(sel_post.FileName != "" && sel_post.FileName.StartsWith("\0"))
			{
				throw new Exception("このWikiページは特殊データのため編集することは出来ません。");
			}

			header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts);//.Replace("</title>","を編集</title>").Replace("</h1>","を編集</h1>");
			footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
			return header + temp + footer;
		}
		#endregion

		#region プレビューページ
		public static string GenerateWikiPreviewPage(WebHeaderInfo info,StringDictionary param)
		{
			DateTime start = DateTime.Now;
			string header = ReadEditHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadEditHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_preview.html");
			string wikiName = param["name"];
			if(wikiName==null)wikiName="";
			Guid id = Guid.Empty;
			try
			{
				id = new Guid(param["id"]);
			}
			catch{id=Guid.Empty;}
			
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			WebPostInfo sel_post = GetNewestWikiName(posts,info,wikiName,id);

			temp = TemplateFieldReplacer.ReplaceField(temp,"%_editor%",param["editor"]);
			temp = temp.Replace("%_bbs_triplist%",GenerateTripList(info,param["trip"]));

			if(sel_post != null)
			{
				sel_post = new WebPostInfo(param["editor"],param["body"],DateTime.Now,Guid.NewGuid(),null,null,wikiName,null);
				temp = TemplateFieldReplacer.ReplaceWikiFields(temp,info,sel_post,posts);
			}
			else
			{
				sel_post = new WebPostInfo(param["editor"],param["body"],DateTime.Now,Guid.NewGuid(),null,null,wikiName,null);
				temp = TemplateFieldReplacer.ReplaceWikiFields(temp,info,sel_post,posts);
			}

			temp = temp.Replace("%_wiki_preview%",WikiEngine.Engine.Transform(param["body"],info,posts,sel_post));

			header = TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts);
			footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
			return header + temp + footer;
		}
		#endregion

		#region 履歴ページ
		static string GenerateWikiHistory(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			DateTime start = DateTime.Now;
			string header = ReadHistoryHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadHistoryHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_history.html");
			string pageTemp = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN PAGE LOOP-->","<!--END PAGE LOOP-->");

			// ヘッダを書き込み
			html.Append(TemplateFieldReplacer.ReplaceCommonHeaderFields(temp.Substring(0,temp.IndexOf("<!--BEGIN PAGE LOOP-->")).Replace("%TITLE%",info.Title),info));

			// ポストを投稿順に並べ替え
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			Array.Sort(posts,new WebPostInfoComparer());
			Array.Reverse(posts);

			// ページを取得
			ArrayList names = new ArrayList();
			foreach(WebPostInfo post in posts)
				if(post.WikiName!=null && !names.Contains(post.WikiName) && !post.WikiName.StartsWith("\0"))
					names.Add(post.WikiName);

			// ページループ
			foreach(string page in names)
			{
				string loop = TextUtility.GetMiddleRangeString(pageTemp,"<!--BEGIN LOOP-->","<!--END LOOP-->");
				string temp2 = pageTemp.Substring(0,pageTemp.IndexOf("<!--BEGIN LOOP-->"));
				foreach(WebPostInfo post in posts)
					if(post.WikiName.Equals(page))
					{
						post.Name = (post.Name == ""?"":"　( " + post.Name + " )");
						temp2 += TemplateFieldReplacer.ReplaceCommonPostFields(loop,post);
					}
				temp2 = temp2.Replace("%_wikiname%",(page==""?"FrontPage":page));
				temp2 += pageTemp.Substring(pageTemp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length);
				html.Append(temp2);
			}

			html.Append(TemplateFieldReplacer.ReplaceCommonHeaderFields(temp.Substring(temp.IndexOf("<!--END PAGE LOOP-->")+"<!--END PAGE LOOP-->".Length),info));

			header = TemplateFieldReplacer.ReplaceWikiFields(header,info,null,posts);//.Replace("%_wikiname%",info.Title + "の更新履歴").Replace("%_wiki_hierarchical_link%",info.Title + "の更新履歴");
			footer = TemplateFieldReplacer.ReplaceWikiFields(footer,info,null,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00"));
			return header + html.ToString() + footer;
		}
		#endregion

		#region 添付ページ
		static string GenerateAttachPage(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			DateTime start = DateTime.Now;
			string header = ReadAttachHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadAttachHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_attach.html");
			string fileLoop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN LOOP-->","<!--END LOOP-->");

			// ページを読み込み
			Guid id = Guid.Empty;
			string wikiName = param["name"];
			if(wikiName==null)wikiName="";
			try
			{
				id = new Guid(param["id"]);
			}
			catch{id=Guid.Empty;}
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			WebPostInfo sel_post = GetNewestWikiName(posts,info,wikiName,id);

			if(sel_post.FileName != null && sel_post.FileName.StartsWith("\0"))
				throw new Exception("指定されたWikiNameは特殊ページをあらわしているので、添付関連操作は実行できません");

			// ヘッダを書き込み
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(header,info,sel_post,posts));
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(temp.Substring(0,temp.IndexOf("<!--BEGIN LOOP-->")),info,sel_post,posts).Replace("%_bbs_triplist%",GenerateTripList(info)));

			// 添付ファイル一覧を作成
			string bt = "\0" + sel_post.WikiName + ":\0";
			ArrayList attachList = new ArrayList();
			foreach(WebPostInfo post in posts)
				if(post.Data != null && post.FileName.StartsWith(bt))
					attachList.Add(post);

			// ポストを投稿順に並べ替え
			attachList.Sort(new WebPostInfoComparer());
			attachList.Reverse();

			foreach(WebPostInfo post in attachList)
			{
				html.Append(
					TemplateFieldReplacer.ReplaceField(
					TemplateFieldReplacer.ReplaceWikiFields(fileLoop,info,post,posts,true)
					,"%_wiki_attach_filename%",post.FileName.Substring(bt.Length))
					);
			}

			// フッタを書き込み～
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(temp.Substring(temp.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length),info,sel_post,posts).Replace("%_bbs_triplist%",GenerateTripList(info)));
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(footer,info,sel_post,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00")));

			return html.ToString();
		}
		#endregion

		#region 添付ファイルリスト
		static string GenerateAttachFileList(WebHeaderInfo info,StringDictionary param)
		{
			StringBuilder html = new StringBuilder();
			DateTime start = DateTime.Now;
			string header = ReadAttachFileListHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_header.html"));
			string footer = ReadAttachFileListHeader(HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_footer.html"));
			string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_attach_list.html");
			string wikiLoopHeader = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN WIKINAME LOOP-->","<!--BEGIN ATTACH LOOP-->");
			string wikiLoopFooter = TextUtility.GetMiddleRangeString(temp,"<!--END ATTACH LOOP-->","<!--END WIKINAME LOOP-->");
			string attachLoop = TextUtility.GetMiddleRangeString(temp,"<!--BEGIN ATTACH LOOP-->","<!--END ATTACH LOOP-->");

			// ページを読み込み
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);

			// ヘッダを書き込み
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(header,info,null,posts));
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(temp.Substring(0,temp.IndexOf("<!--BEGIN WIKINAME LOOP-->")),info,null,posts));

			// WikiName一覧を作成
			ArrayList wikinames = new ArrayList();
			ArrayList wikiAttachNames = new ArrayList();
			foreach(WebPostInfo post in posts)
			{
				if(post.Data != null && post.FileName.StartsWith("\0"))
				{
					int pos = post.FileName.IndexOf(":\0");
					if(pos > 0)
					{
						string name = post.FileName.Substring(1,pos-1);
						if(!wikinames.Contains(name))
						{
							wikinames.Add(name);
							wikiAttachNames.Add("\0" + name + ":\0");
						}
					}
				}
			}

			// 添付ファイル一覧を作成
			ArrayList attachList = new ArrayList();
			foreach(WebPostInfo post in posts)
			{
				foreach(string bt in wikiAttachNames)
				{
					if(post.Data != null && post.FileName.StartsWith(bt))
					{
						attachList.Add(post);
						break;
					}
				}
			}

			// ポストを投稿順に並べ替え
			attachList.Sort(new WebPostInfoComparer());
			attachList.Reverse();

			for(int i=0;i<wikinames.Count;i++)
			{
				string name = (string)wikinames[i];
				string bt = (string)wikiAttachNames[i];
				html.Append(TemplateFieldReplacer.ReplaceField(wikiLoopHeader,"%_wikiname%",name));
				foreach(WebPostInfo post in attachList)
				{
					if(post.FileName.StartsWith(bt))
					{
						html.Append(
							TemplateFieldReplacer.ReplaceField(
							TemplateFieldReplacer.ReplaceWikiFields(attachLoop,info,post,posts,true)
							,"%_wiki_attach_filename%",post.FileName.Substring(bt.Length))
							);
					}
				}
				html.Append(TemplateFieldReplacer.ReplaceField(wikiLoopFooter,"%_wikiname%",name));
			}

			// フッタを書き込み～
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(temp.Substring(temp.IndexOf("<!--END WIKINAME LOOP-->")+"<!--END WIKINAME LOOP-->".Length),info,null,posts));
			html.Append(TemplateFieldReplacer.ReplaceWikiFields(footer,info,null,posts).Replace("&ver;",EngineVersion.WikiEngineVersion).Replace("&time;",DateTime.Now.Subtract(start).TotalSeconds.ToString("0.00")));

			return html.ToString();
		}
		#endregion

		#region 添付ファイルダウンロード
		static byte[] ReadAttachFile(WebHeaderInfo info,StringDictionary param,Guid id,out string mime,StringDictionary response)
		{
			DateTime IfModifiedSince = DateTime.MinValue;
			try
			{
				if(param.ContainsKey("_If-Modified-Since"))
					IfModifiedSince = DateTime.Parse(param["_If-Modified-Since"]);
			}
			catch
			{
			}

			mime = "*/*";
			try
			{
				WebPostInfo[] posts = WebCacheReader.ReadAllPostusingGuid(info.GUID,new Guid[]{id});
				if(posts != null && posts.Length > 0)
				{
					if(posts[0].Trip == null || (posts[0].Trip != null && posts[0].IsEffectiveSignature()))
					{
						if(posts[0].Posted.Date == IfModifiedSince.Date && posts[0].Posted.Hour == IfModifiedSince.Hour && posts[0].Posted.Minute == IfModifiedSince.Minute && posts[0].Posted.Second == IfModifiedSince.Second)
						{
							throw new HttpException(304,"Not Modified");
						}
						else
						{
							response.Add("Last-Modified",posts[0].Posted.ToUniversalTime().ToString("ddd, dd MMM yyyy HH\\:mm\\:ss ",System.Globalization.DateTimeFormatInfo.InvariantInfo) + "GMT");
							mime = HttpServer.GetMimeType(posts[0].FileName.Replace("\0","/"));
							return posts[0].Data;
						}
					}
					else
					{
						mime = "text/html";
						return ErrorHtml.Generate("指定されたファイルのトリップが不正です。改ざんされているおそれがあります",null);
					}
				}
				else
					return null;
			}
			catch(HttpException)
			{
				throw;
			}
			catch
			{
				return null;
			}
		}
		#endregion

		#region Wikiページ以外は非表示にするヘッダを削除する
		static string ReadHistoryHeader(string temp)
		{
			return RemoveCommentHeader(
				RemovePageOnlyHeader(
				RemoveEditOnlyHeader(
				RemoveListOnlyHeader(
				RemoveAttachOnlyHeader(
				RemoveAttachFileListOnlyHeader(
				temp
				)))))
				,"HISTORYONLY");
		}
		static string ReadListHeader(string temp)
		{
			return RemoveCommentHeader(
				RemoveHistoryOnlyHeader(
				RemoveEditOnlyHeader(
				RemovePageOnlyHeader(
				RemoveAttachOnlyHeader(
				RemoveAttachFileListOnlyHeader(
				temp
				)))))
				,"LISTONLY");
		}
		static string ReadEditHeader(string temp)
		{
			return RemoveCommentHeader(
				RemoveHistoryOnlyHeader(
				RemovePageOnlyHeader(
				RemoveListOnlyHeader(
				RemoveAttachOnlyHeader(
				RemoveAttachFileListOnlyHeader(
				temp
				)))))
				,"EDITONLY");
		}
		static string ReadPageHeader(string temp)
		{
			return RemoveCommentHeader(
				RemoveHistoryOnlyHeader(
				RemoveEditOnlyHeader(
				RemoveListOnlyHeader(
				RemoveAttachOnlyHeader(
				RemoveAttachFileListOnlyHeader(
				temp
				)))))
				,"PAGEONLY");
		}
		static string ReadAttachHeader(string temp)
		{
			return RemoveCommentHeader(
				RemoveHistoryOnlyHeader(
				RemoveEditOnlyHeader(
				RemoveListOnlyHeader(
				RemovePageOnlyHeader(
				RemoveAttachFileListOnlyHeader(
				temp
				)))))
				,"ATTACHONLY");
		}
		static string ReadAttachFileListHeader(string temp)
		{
			return RemoveCommentHeader(
				RemoveHistoryOnlyHeader(
				RemoveEditOnlyHeader(
				RemoveListOnlyHeader(
				RemovePageOnlyHeader(
				RemoveAttachOnlyHeader(
				temp
				)))))
				,"ATTACHLISTONLY");
		}
		static string RemoveEditOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"EDITONLY");
		}
		static string RemoveHistoryOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"HISTORYONLY");
		}
		static string RemoveListOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"LISTONLY");
		}
		static string RemovePageOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"PAGEONLY");
		}
		static string RemoveAttachOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"ATTACHONLY");
		}
		static string RemoveAttachFileListOnlyHeader(string temp)
		{
			return RemoveOnlyHeader(temp,"ATTACHLISTONLY");
		}
		static string RemoveCommentHeader(string temp,string id)
		{
			return temp.Replace("<!--BEGIN " + id + "-->","").Replace("<!--END " + id + "-->","");
		}
		static string RemoveOnlyHeader(string temp,string id)
		{
			string _begin = "<!--BEGIN " + id + "-->";
			string _end = "<!--END " + id + "-->";

			try
			{
				while(true)
				{
					string ret = _begin + TextUtility.GetMiddleRangeString(temp,_begin,_end) + _end;
					temp = temp.Replace(ret,"");
				}
			}
			catch{}

			return temp;
		}
		#endregion

		#region トリップ
		static string GenerateTripList(WebHeaderInfo info)
		{
			return GenerateTripList(info,"");
		}
		static string GenerateTripList(WebHeaderInfo info,string selected)
		{
			string trips = BBSGenerator.GenerateTripList(selected);
			if(KeyTools.ExistsManageKey(Encoding.ASCII.GetString(info.Trip)))
			{
				string trip = TextUtility.XmlToTrip(Encoding.ASCII.GetString(info.Trip));
				trips += "<option" + (selected==trip ? " selected" : "") + " value=\"" + HttpUtility.HtmlEncode(trip) + "\">" + HttpUtility.HtmlEncode(trip) + "@凍結用管理鍵</option>";
			}
			return trips;
		}
		#endregion
	}
}
