﻿using System;
using System.Text;
using System.IO;

namespace RinGOch.HtmlGenerators
{
	public class XmlGenerator
	{
		public static string GenerateThreadListXML()
		{
			StringBuilder xml = new StringBuilder();
			StringWriter writer = new StringWriter(xml);

			object[] threads = Interface.GetWebHeaders();

			writer.WriteLine("<threads>");

			foreach(WebHeaderInfo info in threads)
			{
				string readFlag = "";
				if(WebCacheReader.IsCached(info.GUID))
				{
					if(WebPageUpdateFlag.GetFlag(info.GUID,false))
						readFlag = "updated";
					else
						readFlag = "read";
				}
				else
				{
					if(DateTime.Now.Subtract(info.Update).TotalHours <= 1)
						readFlag = "updated-nonread";
					else
						readFlag = "nonread";
				}

				writer.WriteLine("<thread>");
				writer.WriteLine("\t<category>" + HttpUtility.HtmlEncode(info.Category) + "</category>");
				writer.WriteLine("\t<managed>" + HttpUtility.HtmlEncode(info.Created.ToString()) + "</managed>");
				writer.WriteLine("\t<guid>" + HttpUtility.HtmlEncode(info.GUID.ToString()) + "</guid>");
				writer.WriteLine("\t<hash>" + HttpUtility.HtmlEncode(TextUtility.ByteArrayToString(info.Hash)) + "</hash>");
				writer.WriteLine("\t<size>" + info.Size.ToString() + "</size>");
				writer.WriteLine("\t<posts>" + info.Posts.ToString() + "</posts>");
				writer.WriteLine("\t<title>" + HttpUtility.HtmlEncode(info.Title) + "</title>");
				writer.WriteLine("\t<update>" + HttpUtility.HtmlEncode(info.Update.ToString()) + "</update>");
				writer.WriteLine("\t<type>" + HttpUtility.HtmlEncode(WebTypeToString.ToString(info.Type)) + "</type>");
				writer.WriteLine("\t<state>" + readFlag + "</state>");
				writer.WriteLine("</thread>");
			}

			writer.WriteLine("</threads>");

			writer.Close();
			return xml.ToString();
		}

		public static string GenerateBbsXML(WebHeaderInfo info)
		{
			WebPostInfo[] posts = WebCacheReader.ReadAllPost(info.GUID);
			StringBuilder xml = new StringBuilder();

			StringWriter writer = new StringWriter(xml);

			writer.WriteLine("<thread>");
			writer.WriteLine("<info>");
			writer.WriteLine("\t<title>" + HttpUtility.HtmlEncode(info.Title) + "</title>");
			writer.WriteLine("\t<category>" + HttpUtility.HtmlEncode(info.Category) + "</category>");
			writer.WriteLine("\t<type>" + HttpUtility.HtmlEncode(WebTypeToString.ToString(info.Type)) + "</type>");
			writer.WriteLine("\t<update>" + HttpUtility.HtmlEncode(info.Update.ToString()) + "</update>");
			writer.WriteLine("\t<guid>" + HttpUtility.HtmlEncode(info.GUID.ToString()) + "</guid>");
			writer.WriteLine("\t<hash>" + HttpUtility.HtmlEncode(TextUtility.ByteArrayToString(info.Hash)) + "</hash>");
			writer.WriteLine("\t<managed>" + HttpUtility.HtmlEncode(info.Created.ToString()) + "</managed>");
			writer.WriteLine("\t<size>" + info.Size.ToString() + "</size>");
			writer.WriteLine("\t<posts>" + info.Posts.ToString() + "</posts>");
			writer.WriteLine("</info>");

			foreach(WebPostInfo post in posts)
			{
				writer.WriteLine("<post>");
				writer.WriteLine("\t<body>" + HttpUtility.HtmlEncode(post.Body) + "</body>");
				writer.WriteLine("\t<filename>" + HttpUtility.HtmlEncode(post.FileName) + "</filename>");
				writer.WriteLine("\t<guid>" + HttpUtility.HtmlEncode(post.GUID.ToString()) + "</guid>");
				writer.WriteLine("\t<name>" + HttpUtility.HtmlEncode(post.Name) + "</name>");
				writer.WriteLine("\t<posted>" + HttpUtility.HtmlEncode(post.Posted.ToString()) + "</posted>");
				if(post.Trip != null)
				{
					writer.WriteLine("\t<trip>" + HttpUtility.HtmlEncode(TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(post.Trip))) + "</trip>");
				}
				writer.WriteLine("\t<id>" + HttpUtility.HtmlEncode(TextUtility.GuidToResID(post.GUID)) + "</id>");
				writer.WriteLine("</post>");
			}

			writer.WriteLine("</thread>");

			writer.Close();
			return xml.ToString();
		}
	}
}
