﻿using System;
using System.Diagnostics;
using System.IO;
using System.Collections;
using System.Collections.Specialized;
using System.Text;
using System.Text.RegularExpressions;

namespace RinGOch
{
	public class HttpHtmlGenerator
	{
		public static byte[] GenerateMenu(StringDictionary parameters,ref string mime,StringDictionary response)
		{
			return HtmlGenerators.MenuGenerator.Generate(parameters,ref mime);
		}

		public static byte[] GeneratePage(Guid id,StringDictionary parameters,out string mime,string trace_id,StringDictionary response)
		{
			WebHeaderInfo info = WebCacheReader.ReadHeader(id);
			Interface.CheckUpdatePage.BeginInvoke(id,null,null);
			
			if(info != null)
			{
				WebPageUpdateFlag.SetFlag(id,false);
				if(info != null)
				{
					switch(info.Type)
					{
						case WebType.FileBoard:
						case WebType.ImageBoard:
						case WebType.TextOnlyBoard:
							if(parameters["_filename"] == "abon.p2p" && parameters.ContainsKey("guids"))
							{
								mime = "text/html";
								return PostProcessors.BbsProcessor.ProcessResAbon(info,parameters);
							}
							else
								return HtmlGenerators.BBSGenerator.Generate(info,parameters,out mime,trace_id);
						case WebType.Wiki:
							return HtmlGenerators.WikiGenerator.Generate(info,parameters,out mime,response);
						case WebType.Blog:
							return HtmlGenerators.BlogEngine.Generate(info,parameters,out mime,trace_id,response);
						default:
						case WebType.Page:
							mime = "";
							return null;
					}
				}
				else
				{
					mime = "";
					return null;
				}
			}
			else
			{
				Interface.CheckUpdatePage(id);
				mime = "text/html";
				return HttpServerSetting.HtmlEncoding.GetBytes(ReadTemplateFile("template\\gateway\\nocache.html"));
			}
		}

		#region ファイルを読み込む
		public static string ReadTemplateFile(string relative_path)
		{
			try
			{
				relative_path = Path.Combine(System.Environment.CurrentDirectory,relative_path);
				using(System.IO.FileStream fs = new System.IO.FileStream(relative_path,System.IO.FileMode.Open,FileAccess.Read,FileShare.Read))
				{
					byte[] buffer = new byte[fs.Length];
					fs.Read(buffer,0,buffer.Length);

					return System.Text.Encoding.UTF8.GetString(buffer);
				}
			}
			catch
			{
				return "";
			}
		}
		#endregion

		#region スタイルを読み込む
		public static string ReadStyleSheet(string baseDir)
		{
			string _baseStyle = ReadTemplateFile(Path.Combine(baseDir,"base.css"));
			string _userStyle = ReadTemplateFile(Path.Combine(baseDir,"template\\userStyle.css"));
			return "<style>\r\n" + _baseStyle + "\r\n" + _userStyle + "\r\n</style>";
		}
		#endregion

		#region HTML制御文字
		public static string ReplaceHtmlChar(string text)
		{
			return text.Replace("<","&lt;").Replace(">","&gt;");
		}
		#endregion

		#region CrLfToBR
		public static string CrLfToBR(string text)
		{
			return text.Replace("\r\n","\n").Replace("\r","\n").Replace("\n","<BR>");
		}
		#endregion

		#region URL
		public static string CreateLocalAddress()
		{
			string local = HttpServerSetting.BindAddress;
			if(local=="")
			{
				if(HttpServerSetting.AcceptRemote)
				{
					local = "http://" + NetworkAddress.LocalAddress.ToString() + ":" + HttpServerSetting.ServerPort.ToString() + "/";
				}
				else
					local = "http://127.0.0.1:" + HttpServerSetting.ServerPort.ToString() + "/";
			}
			else
			{
				if(!local.EndsWith("/"))
					local += "/";
			}
			return local;
		}
		public static string ReplaceLocalAddress(string text,bool skipEscapeLocal)
		{
			string local = CreateLocalAddress();
			text = Regex.Replace(text,
				@"(&" + (skipEscapeLocal ? "" : "|&amp;") + @")local;(?<addr>[a-zA-Z0-9/_#%&+*:;~?@{}|[\]\-=,.]*)",
				local + "${addr}",
				RegexOptions.IgnoreCase | RegexOptions.Singleline);
			text = text.Replace(local + "/",local);
			return text;
		}
		public static string ReplaceLocalAddress(string text)
		{
			return ReplaceLocalAddress(text,false);
		}
		public static string CreateURLLink(string text)
		{
			return CreateURLLink(text,false);
		}
		public static string CreateURLLink(string text,bool replaceLocal)
		{
			if(replaceLocal)
				text = ReplaceLocalAddress(text);
			text = Regex.Replace(text,
				@"(?<link>(h?ttps?://)(?!ringoch.p2p)(?<addr>[a-zA-Z0-9/_#%&+*:;~?@{}|[\]\-=,.]+))", 
				"<a target=\"_blank\" href=\"http://${addr}\">${link}</a>", 
				RegexOptions.IgnoreCase | RegexOptions.Singleline);
			return P2PLinkReplacer.Replace(text);
		}
		#endregion
	}
}
