﻿using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Diagnostics;

namespace RinGOch
{
	public class HttpServer
	{
		#region メンバ変数
		static Socket	_listener;
		static int		_working = 0;
		#endregion

		#region プロパティ
		public static int ActiveConnections
		{
			get { return _working; }
		}
		#endregion

		#region 開始処理
		public static void StartServer()
		{
			// 設定読み込み
			WebCertificationCache.Initialize();

			/*Debug.Listeners.Remove("default"); // Debug.Listeners.Clear();
			try
			{
				File.Delete("C:\\Debug.txt");
			}
			catch{}
			Debug.Listeners.Add( new TextWriterTraceListener("C:\\Debug.txt"));
			Debug.AutoFlush = true;
			Debug.WriteLine(Interface.IgnorePage.Method.DeclaringType.FullName);
			*/

			WatchThread.Start();

			try
			{
				_listener = new Socket(AddressFamily.InterNetwork,SocketType.Stream,ProtocolType.Tcp);
				if(HttpServerSetting.AcceptRemote)
					_listener.Bind(new IPEndPoint(IPAddress.Parse("0.0.0.0"),HttpServerSetting.ServerPort));
				else
					_listener.Bind(new IPEndPoint(IPAddress.Parse("127.0.0.1"),HttpServerSetting.ServerPort));

				_listener.Listen(4096);
				_listener.BeginAccept(new AsyncCallback(AcceptCallback),null);
			}
			catch(Exception ex)
			{
				System.Windows.Forms.MessageBox.Show("バインドに失敗しました。ポートがすでに使用されている可能性があります\r\n" + ex.ToString());
			}
		}
		#endregion

		#region 終了処理
		public static void StopServer()
		{
			// 設定保存
			WebCertificationCache.SaveData();

			_listener.Close();

			WatchThread.Stop();
			/*if(_listener != null)
			{
				_listener.Stop();
				_listener = null;
			}

			if(_listenThread != null)
			{
				try
				{
					_listenThread.Abort();
				}
				catch{}
				_listenThread = null;
			}*/
		}
		#endregion

		#region AcceptCallback
		static void AcceptCallback(IAsyncResult ar)
		{
			Socket sock = null;
			if(_listener == null)return;

			try
			{
				// 受諾
				sock = _listener.EndAccept(ar);
			}
			catch
			{
			}

			try
			{
				// 開始
				_listener.BeginAccept(new AsyncCallback(AcceptCallback),null);
			}
			catch
			{
			}

			if(sock == null) return;

			try
			{
				// コネクション数制限
				/*while(_working > HttpServerSetting.NumberOfMaximumConnections)
					Thread.Sleep(1);*/

				System.Threading.Interlocked.Increment(ref _working);

				// 一応、null、接続済みかどうかチェック
				if(sock!=null && sock.Connected)
				{
					if(false)
					{
						ProcessClass pc = new ProcessClass(sock);
						Thread invokeThread = new Thread(new ThreadStart(pc.Invoke));
						invokeThread.Start();
						if(!invokeThread.Join(new TimeSpan(0,10,0)))	// 10分がタイムアウト
						{					
							// スレッド中断処理
							for(int i=0;i<100;i++)
							{
								try
								{
									invokeThread.Abort();
								}
								catch{}
								if(invokeThread.Join(1)) break;
							}
						}
						invokeThread = null;
						pc = null;
					}
					else
						HttpServer_Process(sock);
				}
				else
				{
					sock.Close();
				}
			}
			catch
			{
			}
			finally
			{
				System.Threading.Interlocked.Decrement(ref _working);
			}
		}
		#endregion

		#region プロセス実行スレッド
		internal class ProcessClass
		{
			Socket _sock;
			public ProcessClass(Socket sock)
			{
				_sock = sock;
			}

			public void Invoke()
			{
				HttpServer_Process(_sock);
			}
		}
		#endregion

		#region 処理
		static void HttpServer_Process(object osock)
		{
			string id = Guid.NewGuid().ToString().Substring(0,6);
			StringDictionary parameters = null;
			using(Socket sock = (Socket)osock)
			{
				try
				{
					StringDictionary resParam = new StringDictionary();
					byte[] buffer = new byte[1024 * 1024 * 3];

					Debug.WriteLine(DateTime.Now.ToString() + "\tSocket Accept. Begin Receive HTTP Header",id);

					// HTTPリクエスト受信
					int readed = ReceiveHttpHeaders(sock,buffer);
					string httpBuffer = System.Text.Encoding.ASCII.GetString(buffer,0,readed);

					Debug.WriteLine(DateTime.Now.ToString() + "\tAnalyze HTTP Header",id);

					// HTTPヘッダを解析
					AnalyzeHttpHeader(httpBuffer,out parameters);

					// クライアント情報追加
					parameters.Add("_ip",((IPEndPoint)sock.RemoteEndPoint).Address.ToString());

					// 常に認証が必要
					if(HttpServerSetting.AlwaysAuthorize && !HasAuthority(parameters))
					{
						throw HttpException.UnauthorizedException;
					}

					Debug.WriteLine(DateTime.Now.ToString() + "\tStart Main Process",id);
		
					// メソッド別に処理。
					byte[] data;
					string msg = "404 Not found";
					string mime = "";
					if(parameters["_method"] == "GET")
						data = ProcessGetMethod(sock,parameters,out msg,out mime,id,resParam);
					else if(parameters["_method"] == "POST")
						data = ProcessPostMethod(sock,parameters,out msg,out mime);
					else if(parameters["_method"] == "HEAD")
					{
						data = ProcessGetMethod(sock,parameters,out msg,out mime,id,resParam);
					}
					else
					{
						// HEAD等、GET、POST以外のメソッドには非対応・・・
						mime = "*/*";
						msg = "405 Method not allowed";
						data = null;
					}

					if(data == null)
						data = data = new byte[0];

					if(mime.StartsWith("text/") && HttpServerSetting.UseGzipDeflate && data.Length > 0)
					{
						if(parameters.ContainsKey("_accept-encoding") && parameters["_accept-encoding"].ToLower().IndexOf("gzip")>=0)
						{
							Debug.WriteLine("圧縮開始：" + data.Length,id);
							data = Compress(data);
							resParam.Add("Content-Encoding","gzip");
							Debug.WriteLine("圧縮完了：" + data.Length,id);
						}
					}

					// HTTPレスポンスヘッダを送信
					int rsize = SendHeader(parameters["_version"],data.Length,mime,resParam,msg,sock);

					// コンテンツがあれば送信(HEADメソッド以外)
					if(data.Length > 0 && parameters["_method"] != "HEAD")
						rsize += SendToBrowser(data,sock);

					if(HttpServerSetting.LoggingMode)
						HttpServerLogger.AppendLog(sock,parameters,msg,rsize);
				}
				catch(HttpException he)
				{
					try
					{
						int exsize = SendHeader("HTTP/1.0",0,"",he.ResponseFields,he.ErrorNumber.ToString() + " " + he.ErrorMessage,sock);

						if(HttpServerSetting.LoggingMode)
						{
							if(parameters == null) parameters = new StringDictionary();
							HttpServerLogger.AppendLog(sock,parameters,he.ErrorNumber,exsize);
						}
					}
					catch{}
				}
				catch(Exception ex)
				{
					try
					{
						// キャッチしていない例外が起きたら内部サーバエラーを返す
						byte[] data = HtmlGenerators.ErrorHtml.Generate("Internal Server Error",ex);
						int exsize = SendHeader("HTTP/1.0",data.Length,"text/html",null,"500 Internal Error",sock);
						exsize += SendToBrowser(data,sock);

						if(HttpServerSetting.LoggingMode)
						{
							if(parameters == null) parameters = new StringDictionary();
							HttpServerLogger.AppendLog(sock,parameters,500,exsize,ex);
						}
					}
					catch{}
				}
				try
				{
					sock.Shutdown(SocketShutdown.Both);
					sock.Close();
				}
				catch{}
			}
			Debug.WriteLine(DateTime.Now.ToString() + "\tFinished Process",id);
		}
		#endregion

		#region 圧縮処理
		public static byte[] Compress(byte[] data)
		{
			using(MemoryStream ms = new MemoryStream())
			{
				//using(ICSharpCode.SharpZipLib.Zip.Compression.Streams.DeflaterOutputStream strm = new ICSharpCode.SharpZipLib.Zip.Compression.Streams.DeflaterOutputStream(ms))
				ICSharpCode.SharpZipLib.GZip.GZipOutputStream strm = new ICSharpCode.SharpZipLib.GZip.GZipOutputStream(ms);
				strm.Write(data,0,data.Length);
				strm.Close();
				ms.Close();
				return ms.ToArray();
			}
		}
		#endregion

		#region HTTP処理

		#region HTTPヘッダ受信
		static int ReceiveHttpHeaders(Socket sock,byte[] buffer)
		{
			int readed = 0;
			while(true)
			{
				int read = sock.Receive(buffer,readed,1,SocketFlags.None);
				if(read == 0)break;
				readed += read;
								
				if(readed>4 && (buffer[readed-1]==10 && buffer[readed-2]==13 && buffer[readed-3]==10 && buffer[readed-4]==13))
					break;
			}
			return readed;
		}
		#endregion

		#region HTTPヘッダ解析
		static void AnalyzeHttpHeader(string request, out StringDictionary parameter)
		{
			// コレクション初期化
			parameter = new StringDictionary();

			// 行に分割
			string[] lines = request.Split('\n');
			if(lines.Length > 0) parameter.Add("_firstline",lines[0]);
			
			// 最初の一行はメソッド＆バージョン
			string method = lines[0].Substring(0,lines[0].IndexOf(" "));
			string path = lines[0].Substring(method.Length+1,lines[0].LastIndexOf(" ") - method.Length - 1);
			string ver = lines[0].Substring(lines[0].LastIndexOf(" ")+1).TrimEnd('\r');
			parameter.Add("_method",method);
			parameter.Add("_path",path);
			parameter.Add("_version",ver);

			// 各パラメータをアンダースコアを先頭につけてコレクションに追加
			for(int i=1;i<lines.Length;i++)
			{
				string[] items = lines[i].Trim().Split(new char[]{':'},2);
				if(items.Length == 2)
				{
					parameter.Add("_" + items[0].Trim(),items[1].Trim());
				}
			}

			// リクエストファイルを取得
			path = path.Replace("\\","/");

			// GETパラメータを分析
			if(path.IndexOf("?") >= 0)
			{
				// 文字コード判別
				TxtEnc te = new TxtEnc();te.CodeBreak=false;te.MaxRead=0;
				byte[] tb = System.Web.HttpUtility.UrlDecodeToBytes(lines[0]);
				te.SetByteArray(ref tb);

				System.Text.Encoding encoding = System.Text.Encoding.GetEncoding(te.Codec);
				string[] getParams = path.Substring(path.IndexOf("?")+1).Split('&');
				path = path.Substring(0,path.IndexOf("?"));

				if(path.ToLower().EndsWith("chat.p2p"))
					encoding = System.Text.Encoding.GetEncoding("shift-jis");

				foreach(string p in getParams)
				{
					string[] ps = p.Split('=');
					if(ps.Length == 2)
						parameter.Add(HttpUtility.UrlDecode(ps[0],encoding),HttpUtility.UrlDecode(ps[1],encoding));
				}
			}

			// リクエストファイル名を取得
			int iStartPos = path.LastIndexOf("/") + 1;
			string sRequestedFile = path.Substring(iStartPos);

			// ディレクトリ名を取得
			string sDirName = "";
			if(path.IndexOf("/") < path.LastIndexOf("/"))
				sDirName = path.Substring(path.IndexOf("/"), path.LastIndexOf("/")-path.IndexOf("/")+1);
			else
				sDirName = "/";
			parameter.Add("_directory",TxtEnc.AutoConvert(HttpUtility.UrlDecodeToBytes(sDirName,System.Text.Encoding.ASCII)));
			parameter.Add("_filename",TxtEnc.AutoConvert(HttpUtility.UrlDecodeToBytes(sRequestedFile,System.Text.Encoding.ASCII)));
			parameter["_path"] = TxtEnc.AutoConvert(HttpUtility.UrlDecodeToBytes(parameter["_path"],System.Text.Encoding.ASCII));
		}

		#endregion

		#region HTTPヘッダ作成
		static int SendHeader(string sHttpVersion, int iTotBytes,string sMIMEHeader,StringDictionary otherParam, string sStatusCode, Socket mySocket)
		{

			String sBuffer = "";

			if(sStatusCode != null)
				sStatusCode = sStatusCode.Trim();

			sBuffer = sBuffer + sHttpVersion + " " + sStatusCode + "\r\n";
			if(sStatusCode.IndexOf("\n") < 0)
			{
				sBuffer = sBuffer + "Accept-Ranges: bytes\r\n";
				sBuffer = sBuffer + "Content-Type: " + sMIMEHeader + "\r\n";
				sBuffer = sBuffer + "Content-Length: " + iTotBytes + "\r\n";
			}
			if(otherParam != null)
			{
				foreach(DictionaryEntry entry in otherParam)
				{
					sBuffer += entry.Key + ": " + entry.Value + "\r\n";
				}
			}

			sBuffer = sBuffer.TrimEnd('\r','\n') + "\r\n\r\n";
			
			Byte[] bSendData = System.Text.Encoding.ASCII.GetBytes(sBuffer); 

			return SendToBrowser( bSendData, mySocket);
		}
		#endregion

		#region HTTPコンテンツデータ送信
		static int SendToBrowser(String sData, Socket mySocket)
		{
			return SendToBrowser (System.Text.Encoding.ASCII.GetBytes(sData), mySocket);
		}
		static int SendToBrowser(Byte[] bSendData, Socket mySocket)
		{
			try
			{
				return mySocket.Send(bSendData, bSendData.Length,0);
			}
			catch 
			{
				return 0;			
			}
		}
		#endregion

		#region HTTP/MIMEを取得
		public static string GetMimeType(string sRequestedFile)
		{
			StreamReader sr;
			String sLine = "";
			String sFileExt = "";
				
			// Convert to lowercase
			sRequestedFile = sRequestedFile.ToLower();
			
			int iStartPos = sRequestedFile.LastIndexOf(".");
			if(iStartPos < 0)
				return "*/*";

			sFileExt = sRequestedFile.Substring(iStartPos).Replace(".","");
			
			try
			{
				//Open the Vdirs.dat to find out the list virtual directories
				using(sr = new StreamReader(System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("RinGOch.mime.txt"),System.Text.Encoding.GetEncoding("shift-jis")))
				{

					while ((sLine = sr.ReadLine()) != null)
					{

						sLine.Trim();

						if (sLine.Length > 0)
						{
							//find the separator
							iStartPos = sLine.IndexOf(";");
						
							// Convert to lower case
							sLine = sLine.ToLower();
						
							string sMimeExt = sLine.Substring(0,iStartPos);
							string sMimeType = sLine.Substring(iStartPos + 1);
						
							string[] exts = sMimeType.ToLower().Split(',');
							if (Array.IndexOf(exts,sFileExt) >= 0)
								return sMimeExt;
						}
					}
				}
			}
			catch
			{
			}
			return "*/*";
		}
		#endregion

		#endregion

		#region GET
		static byte[] ProcessGetMethod(Socket sock,StringDictionary parameters,out string msg,out string mime,string trace_id,StringDictionary response)
		{
			mime = "";msg="";

			// アドレスからGUIDを取得
			Guid id = ExtractGuidFromAddress(parameters);

			if(id.Equals(Guid.Empty))
			{
				// アドレスにGUIDが含まれていないので、ゲートウェイメニューを表示
				try
				{
					parameters.Add("",(parameters["_filename"]==""?"index.html":parameters["_filename"]));
					msg = "200 OK";
					mime = GetMimeType(parameters[""]);
					if(parameters["_filename"].Replace("/","").ToLower() == "chat.p2p" && parameters["hash"]!=null)
						mime = GetMimeType("TEST.png");
					return HttpHtmlGenerator.GenerateMenu(parameters,ref mime,response);
				}
				catch(HttpException)
				{
					throw;
				}
				catch(Exception)
				{
					throw new HttpException(400,"Bad Request");
				}
			}
			else
			{
				string path = "";
				try
				{
					path =System.IO.Path.Combine(parameters["_directory"].Substring(parameters["_directory"].IndexOf("/",1)+1),parameters["_filename"]);
				}
				catch
				{
					throw new HttpException(400,"Bad Request");
				}

				path = HttpUtility.UrlDecode(path);
				parameters.Add("",path);
				msg = "200 OK";

				// HTML生成
				try
				{
					return HttpHtmlGenerator.GeneratePage(id,parameters,out mime,trace_id,response);
				}
				catch(HttpException)
				{
					throw;
				}
				catch(Exception ex)
				{
					msg = "500 Internal Error";
					return HtmlGenerators.ErrorHtml.Generate("GETメソッドの処理中に例外が発生しました",ex);
				}
			}
		}
		#endregion

		#region POST
		static byte[] ProcessPostMethod(Socket sock, StringDictionary parameters,out string msg,out string mime)
		{
			msg = "";
			mime = "text/html";

			// データを受信
			ReceivePostData(sock,parameters);
			
			Guid id = ExtractGuidFromAddress(parameters);
			
			if(id.Equals(Guid.Empty))
			{
				try
				{
					parameters.Add("",(parameters["_filename"]==""?"index.html":parameters["_filename"]));
					mime = GetMimeType(parameters[""]);
					msg = " 200 OK";
					return HttpPostDataProcessor.Process(parameters);
				}
				catch(HttpException)
				{
					throw;
				}
				catch(Exception ex)
				{
					msg = "500 Internal Error";
					return HtmlGenerators.ErrorHtml.Generate("POSTデータの処理中に例外が発生しました",ex);
				}
			}
			else
			{
				string path;
				try
				{
					path =System.IO.Path.Combine(parameters["_directory"].Substring(parameters["_directory"].IndexOf("/",1)+1),parameters["_filename"]);
				}
				catch(Exception ex)
				{
					msg = "400 Bad request";
					return HtmlGenerators.ErrorHtml.Generate("無効なPOSTパラメータ",ex);
				}

				try
				{
					msg = " 200 OK";
					return HttpPostDataProcessor.Process(id,parameters);
				}
				catch(HttpException)
				{
					throw;
				}
				catch(Exception ex)
				{
					msg = "500 Internal Error";
					return HtmlGenerators.ErrorHtml.Generate("POSTデータの処理中に例外が発生しました",ex);
				}
			}
		}
		#endregion

		#region Postデータ受信
		static void ReceivePostData(Socket sock,StringDictionary parameter)
		{
			byte[] buffer = new byte[1024 * 1024 * 3];
			int readed = 0;
			int totalSize = 0;
			try
			{
				totalSize = (parameter["_content-length"]==null?0:int.Parse(parameter["_content-length"].Trim()));
			}
			catch{}
			while(true)
			{
				int read = sock.Receive(buffer,readed,buffer.Length-readed,SocketFlags.None);
				if(read == 0)break;
				readed += read;
								
				if(readed >= totalSize)
					break;
			}

			if(parameter["_content-type"].StartsWith("multipart/form-data"))
			{
				LoadProc(buffer,parameter);
			}
			else
			{
				// 文字コード判別
				TxtEnc te = new TxtEnc();
				te.MaxRead = 0;te.CodeBreak=false;
				byte[] tb = System.Web.HttpUtility.UrlDecodeToBytes(buffer,0,totalSize);
				te.SetByteArray(ref tb);
				System.Text.Encoding encoding = System.Text.Encoding.GetEncoding(te.Codec);

				string[] getParams = System.Text.Encoding.ASCII.GetString(buffer,0,totalSize).Split('&');
				foreach(string p in getParams)
				{
					string[] ps = p.Split(new char[]{'='},2);
					if(ps.Length == 2)
					{
						parameter.Add(HttpUtility.UrlDecode(ps[0],encoding),HttpUtility.UrlDecode(ps[1],encoding));
					}
				}
			}
		}

		static void LoadProc(byte[] data,StringDictionary parameter)
		{
			byte[] content = System.Text.Encoding.ASCII.GetBytes("Content-Disposition");

			// \r までをセパレートとして記憶
			int spos = Array.IndexOf(data,(byte)13);
			byte[] separator = new byte[spos];
			Array.Copy(data,0,separator,0,separator.Length);

			int begin = spos+2;

			// 全スキャン
			for(int i=begin;i<data.Length;i++)
			{
				// Content-Dispositionをチェック
				if(ArrayEqual(data,i,content,0,content.Length))
				{
					int start = i;
					int end = i;

					int headerStart = i;
					int headerEnd = MoveNextLine(data,i) - 2;

					// データの開始位置までシーク
					while(i < data.Length)
					{
						if(ArrayEqual(data,i-2,new byte[]{13,10,13,10},0,4))
						{
							i += 2;
							start = i;
							break;
						}
						else
							i = MoveNextLine(data,i);
					}

					// データの開始位置までシーク
					while(i < data.Length)
					{
						if(ArrayEqual(data,i,separator,0,separator.Length))
						{
							end = i - 2;
							break;
						}
						else
							i = MoveNextLine(data,i);
					}

					System.Text.Encoding encoding;
					/*if(parameter["_filename"] != null && parameter["_filename"].ToLower().StartsWith("chat.p2p"))
						encoding = System.Text.Encoding.Default;
					else*/
					
					encoding = HttpServerSetting.HtmlEncoding;
					string header = encoding.GetString(data,headerStart,headerEnd-headerStart);
					int pp = header.IndexOf("name=\"")+6;
					string formName = header.Substring(pp,header.IndexOf("\"",pp)-pp);

					int pp2 = header.IndexOf("filename=\"");
					string fileName = "";
					if(pp2 >= 0)
					{
						pp2 += "filename=\"".Length;
						fileName = header.Substring(pp2,header.IndexOf("\"",pp2)-pp2);
					}

					// データを読み取ります
					if(fileName != "")
					{
						parameter.Add(formName,Convert.ToBase64String(data,start,end-start));
						parameter.Add("filename",fileName);
					}
					else
						parameter.Add(formName,encoding.GetString(data,start,end-start));
				}
			}
		}
		
		static int MoveNextLine(byte[] data,int pos)
		{
			for(int i = pos ; i < data.Length ; i++)
				if(data[i] == 13 && data[i+1] == 10)
					return i+2;
			return data.Length;
		}
		static bool ArrayEqual(byte[] a,int ia,byte[] b,int ib,int len)
		{
			for(int i=0;i<len;i++)
				if(a[ia+i] != b[ib+i])
					return false;
			return true;
		}
		#endregion

		#region その他-未整理
		
		static Guid ExtractGuidFromAddress(StringDictionary parameters)
		{
			try
			{
				if(parameters["_directory"] != null && parameters["_directory"] != "/")
					return new Guid(parameters["_directory"].Substring(1,parameters["_directory"].IndexOf("/",1)-1));
				else if(parameters["_directory"] == "/")
					return new Guid(parameters["_filename"]);
				else
					return Guid.Empty;
			}
			catch
			{
				return Guid.Empty;
			}
		}

		#endregion

		#region 認証
		public static bool HasAuthority(StringDictionary param)
		{
			if(IsNeedAuthority(param))
			{
				if(param.ContainsKey("_authorization"))
				{
					string text = param["_authorization"].Trim();
					text = text.Substring(text.ToLower().IndexOf("basic") + 5);
					string userpass = System.Text.Encoding.Default.GetString(Convert.FromBase64String(text));
					if(HttpServerSetting.BasicPassword == ":" || HttpServerSetting.BasicPassword == "")
					{
						return false;
					}
					else
					{
						if(userpass.Trim() == HttpServerSetting.BasicPassword)
							return true;
						else
							return false;
					}
				}
				else
					return false;
			}
			else
				return true;
		}
		static bool IsNeedAuthority(StringDictionary param)
		{
			if(HttpServerSetting.AlwaysAuthorize) return true;
			if(HttpServerSetting.AuthorizeLocalAccess) return true;
			if(HttpServerSetting.PublicMode)
			{
				if(param["_ip"]=="127.0.0.1" || param["_ip"]==NetworkAddress.LocalAddress.ToString())
					return false;
				else
					return true;
			}
			else
				return false;
		}
		#endregion
	}
}
