﻿using System;
using System.Collections.Specialized;
using System.Text.RegularExpressions;

namespace RinGOch.PostProcessors
{
	/// <summary>
	/// WikiProcessor の概要の説明です。
	/// </summary>
	public class WikiProcessor
	{
		public static byte[] Process(WebHeaderInfo info,StringDictionary param)
		{
			if(param["_filename"].ToLower().StartsWith("attach.p2p"))
			{
				return AttachProcess(info,param);
			}
			else
			{
				if(param["name"] != null)
					return PageProcess(info,param);
				else if(param["id"] != null)
					return CommentProcess(info,param);
				else
					return null;
			}
		}

		static byte[] AttachProcess(WebHeaderInfo info,StringDictionary param)
		{
			string name = param["name"];
			if(name==null || name=="") name = "FrontPage";

			try
			{
				string fname = "";
				fname = System.IO.Path.GetFileName(param["filename"]);
				if(fname == null || fname=="")
					fname = "unknown";
				byte[] binary = Convert.FromBase64String(param["binary"]);

				if(binary==null || binary.Length==0) throw new Exception();

				Guid posted = Interface.PostToBbs(info.GUID,param["sender"],"",param["trip"],binary,"\0" + name + ":\0" + fname);
				Console.WriteLine(posted);
			}
			catch(Exception ex)
			{
				Console.WriteLine(ex.ToString());
			}

			throw new HttpException(302,"Found","Location","attach.p2p?name=" + System.Web.HttpUtility.UrlEncode(name));
		}

		static byte[] PageProcess(WebHeaderInfo info,StringDictionary param)
		{
			if(param["write"] != null)
			{
				string body = param["body"];

				string tripPath = "";
				if(param["trip"].Equals(TextUtility.XmlToTrip(System.Text.Encoding.ASCII.GetString(info.Trip))))
				{
					tripPath = KeyTools.CopyTempTrip(System.Text.Encoding.ASCII.GetString(info.Trip),info);
				}

				Interface.PostToBbs(info.GUID,param["editor"],body,param["trip"],null,param["name"]);

				try
				{
					if(tripPath != "") System.IO.File.Delete(tripPath);
				}
				catch{}
				throw new HttpException(302,"Found","Location","wiki.p2p?name=" + System.Web.HttpUtility.UrlEncode(param["name"]));
			}
			else
			{
				// プレビューなのさ
				return HttpServerSetting.HtmlEncoding.GetBytes(HtmlGenerators.WikiGenerator.GenerateWikiPreviewPage(info,param));
			}
		}

		static byte[] CommentProcess(WebHeaderInfo info,StringDictionary param)
		{
			string id = "\0" + param["id"];
			string body = (param["comment"]==null?"":param["comment"]);
			string name = (param["sender"]==null?"":param["sender"]);
			if(body.Length > 1500)body = body.Substring(0,1500);
			if(name.Length > 20)name = name.Substring(0,20);

			string short_id = Regex.Replace(Convert.ToBase64String(Guid.NewGuid().ToByteArray()),@"(/|\=|#)","").Substring(0,6);
			body = body.Replace("&short-id;",short_id);

			Interface.PostToBbs(info.GUID,name,body,null,null,id);
			throw new HttpException(302,"Found","Location","wiki.p2p?name=" + System.Web.HttpUtility.UrlEncode(param["wikiname"]));
		}
	}
}
