﻿using System;
using System.Text.RegularExpressions;
using System.Collections;
using System.Security.Cryptography;

namespace RinGOch
{
	public class TextUtility
	{
		public static string XmlToTrip(string xml)
		{
			// ハッシュ計算
			using(HashAlgorithm ha = new SHA1Managed())
			{
				byte[] key = Convert.FromBase64String(GetMiddleRangeString(xml,"<Modulus>","</Modulus>"));
				byte[] hash = ha.ComputeHash(key);
				ha.Clear();

				byte[] temp = new byte[hash.Length/2];
				for(int i=0;i<hash.Length;i+=2)
					temp[i/2] = (byte)(hash[i] ^ hash[i+1]);

				return Convert.ToBase64String(temp).Replace("=","");
			}
		}

		static char[] ResIdCharTable = {
											'a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z',
											'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
											'0','1','2','3','4','5','6','7','8','9'
										};
		static int ResIdCharTableLength = ResIdCharTable.Length;

		public static string GuidToResID(Guid id)
		{
			byte[] data = id.ToByteArray();
			int val = data[0] + (data[1]<<8) + (data[2]<<16);
		    string buffer = "";
			for(int i=0;i<4;i++)
			{
				buffer+=ResIdCharTable[val%ResIdCharTableLength].ToString();
				val/=ResIdCharTableLength;
			}
			return buffer;
			//return Convert.ToBase64String(id.ToByteArray(),0,3).Replace("=","").PadLeft(4,'0');
		}

		public static string ToQuotationText(string text)
		{
			return ToQuotationText(text,"> ");
		}
		public static string ToQuotationText(string text,string qword)
		{
			string[] lines = text.Split('\n');
			for(int i=0;i<lines.Length;i++)
				lines[i] = qword + lines[i];
			return string.Join("\n",lines);
		}

		public static string SizeToString(long size)
		{
			if(size < 1024)
				return size + " B";
			else if(size < 1024 * 1024)
				return (size/1024.0).ToString("0.00") + " KB";
			else
				return (size/1024.0/1024.0).ToString("0.00") + " MB";
		}
		public static string SizeToString(ulong size)
		{
			if(size < 1024)
				return size + " B";
			else if(size < 1024 * 1024)
				return (size/1024.0).ToString("0.00") + " KB";
			else if(size < 1024 * 1024 * 1024)
				return (size/1024.0/1024.0).ToString("0.00") + " MB";
			else
				return (size/1024.0/1024.0/1024.0).ToString("0.00") + " GB";
		}

		public static string ByteArrayToString(byte[] ary)
		{
			System.Text.StringBuilder s = new System.Text.StringBuilder();
			foreach(byte b in ary)
				s.Append(b.ToString("x2"));
			return s.ToString();
		}

		public static bool IsNumeric(string data)
		{
			try
			{
				int.Parse(data);
				return true;
			}
			catch
			{
				return false;
			}
		}

		public static string DateTimeToString(DateTime dt)
		{
			return dt.Year.ToString("0000") + "/" + dt.Month.ToString("00") + "/" + dt.Day.ToString("00") + " " +
				dt.Hour.ToString("00") + ":" + dt.Minute.ToString("00") + ":" + dt.Second.ToString("00");
		}

		public static string GetMiddleRangeString(string text,string begin,string end)
		{
			try
			{
				int pos1 = text.ToLower().IndexOf(begin.ToLower());
				if(pos1 < 0)throw new Exception("開始文字列が見つかりません");
				pos1 += begin.Length;

				int pos2 = text.ToLower().IndexOf(end.ToLower(),pos1);
				if(pos2 < 0)throw new Exception("終了文字列が見つかりません");

				return text.Substring(pos1,pos2 - pos1);
			}
			catch
			{
				throw;
			}
		}

		public static string[] ReadTextFile(string path,bool toLower)
		{
			try
			{
				string[] lines;
				using(System.IO.FileStream fs = new System.IO.FileStream(path,System.IO.FileMode.Open,System.IO.FileAccess.Read))
				{
					byte[] buf = new byte[fs.Length];
					fs.Read(buf,0,buf.Length);
					fs.Close();
					lines = System.Text.Encoding.Default.GetString(buf).Split('\n');
				}

				System.Collections.ArrayList ary = new System.Collections.ArrayList();
				foreach(string line in lines)
				{
					if(line.TrimEnd('\r').Length > 0)
					{
						if(toLower)
							ary.Add(line.TrimEnd('\r').ToLower());
						else
							ary.Add(line.TrimEnd('\r'));
					}
				}

				return (string[])ary.ToArray(typeof(string));
			}
			catch
			{
				return new string[0];
			}
		}

		public static void AppendText(string path,string text)
		{
			try
			{
				if(!System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(path)))
					System.IO.Directory.CreateDirectory(System.IO.Path.GetDirectoryName(path));

				using(System.IO.FileStream fs = new System.IO.FileStream(path,System.IO.FileMode.Append,System.IO.FileAccess.Write,System.IO.FileShare.ReadWrite))
				{
					byte[] buffer = System.Text.Encoding.Default.GetBytes(text);
					fs.Write(buffer,0,buffer.Length);
					fs.Close();
				}
			}
			catch{}
		}

		public static void WriteText(string path,string text)
		{
			try
			{
				if(!System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(path)))
					System.IO.Directory.CreateDirectory(System.IO.Path.GetDirectoryName(path));

				using(System.IO.FileStream fs = new System.IO.FileStream(path,System.IO.FileMode.Create,System.IO.FileAccess.Write,System.IO.FileShare.ReadWrite))
				{
					byte[] buffer = System.Text.Encoding.Default.GetBytes(text);
					fs.Write(buffer,0,buffer.Length);
					fs.Close();
				}
			}
			catch{}
		}

		public static bool TextContains(string text,string[] words)
		{
			string temp = text.ToLower();
			foreach(string word in words)
			{
				if(temp.IndexOf(word) >= 0)
				{
					return true;
				}
			}
			return false;
		}

		public static string[] ExtractAddress(string text)
		{
			ArrayList ary = new ArrayList();

			MatchCollection p2pweb = Regex.Matches(text,@"(?<link>((bbs|page|image)://)(?<addr>[a-zA-Z0-9/_#%&+*:;~?@{}|[\]\-=,.]+))");	
			foreach(Match m in p2pweb)
				ary.Add(m.Value);

			MatchCollection http = Regex.Matches(text,@"(?<link>(h?ttps?://)(?<addr>[a-zA-Z0-9/_#%&+*:;~?@{}|[\]\-=,.]+))");	
			foreach(Match m in http)
			{
				if(m.Value.ToLower().StartsWith("ttp"))
					ary.Add("h" + m.Value);
				else
					ary.Add(m.Value);
			}

			return (string[])ary.ToArray(typeof(string));
		}
	}
}
