﻿using System;
using System.Text;
using System.IO;
using System.Collections;

namespace RinGOch
{
	/// <summary>
	/// トリップチェックキャッシュです。
	/// 保存するようにすれば、再起動後も高速トリップチェックできますが・・・ｗ
	/// </summary>
	public class WebCertificationCache
	{
		static Hashtable _table = new Hashtable();

		public static void Initialize()
		{
			_table = new Hashtable();
			
			try
			{
				string path = Path.Combine(Environment.CurrentDirectory,"Settings\\webcachedata.dat");
				if(File.Exists(path))
				{
					using(FileStream fs = new FileStream(path,FileMode.Open))
					{
						string buffer = Encoding.UTF8.GetString(new BinaryReader(fs).ReadBytes((int)fs.Length));
						using(StringReader reader = new StringReader(buffer))
						{
							string line = null;
							while ((line = reader.ReadLine()) != null)
							{
								Guid pageid = new Guid(line);
								WebCertificationCacheData data = new WebCertificationCacheData();
								while(true)
								{
									if((line = reader.ReadLine()) == null || (line == "\0")) break;
									data.Managed = new DateTime(long.Parse(line));
									if((line = reader.ReadLine()) == null || (line == "\0")) break;
									data.EndPost = new Guid(line);
									if((line = reader.ReadLine()) == null || (line == "\0")) break;
									data.Errors = toGuidArray(line);

									while(!((line = reader.ReadLine()) == null || (line == "\0")))
									{}

									break;
								}
								_table.Add(pageid,data);
							}
						}
					}
				}
			}
			catch{}
		}

		public static void SaveData()
		{
			try
			{
				StringBuilder buffer = new StringBuilder();
				foreach(DictionaryEntry entry in _table)
				{
					WebCertificationCacheData data = (WebCertificationCacheData)entry.Value;
					buffer.Append(
						((Guid)entry.Key).ToString() + "\r\n" +
						data.Managed.Ticks.ToString() + "\r\n" +
						data.EndPost.ToString() + "\r\n" +
						toString(data.Errors) + "\r\n" +
						"\0\r\n"
						);
				}

				using(FileStream fs = new FileStream(Path.Combine(Environment.CurrentDirectory,"Settings\\webcachedata.dat"),FileMode.Create))
				{
					byte[] temp = Encoding.UTF8.GetBytes(buffer.ToString());
					fs.Write(temp,0,temp.Length);
					fs.Close();
				}
			}
			catch{}
		}
		static Guid[] toGuidArray(string data)
		{
			ArrayList list = new ArrayList();
			string[] items = data.Split(',');
			foreach(string item in items)
			{
				try
				{
					list.Add(new Guid(item));
				}
				catch{}
			}
			return (Guid[])list.ToArray(typeof(Guid));
		}
		static string toString(Guid[] list)
		{
			StringBuilder buffer = new StringBuilder();
			foreach(Guid id in list)
			{
				buffer.Append(id.ToString() + ",");
			}
			if(buffer.Length > 0)
				buffer.Remove(buffer.Length-1,1);
			return buffer.ToString();
		}

		/// <summary>
		/// 最後に署名チェックしたときの最後のPOST-GUIDを返します。
		/// </summary>
		public static Guid GetEndGuid(Guid pageID,DateTime mgrDate)
		{
			if(_table.ContainsKey(pageID))
			{
				WebCertificationCacheData  data = (WebCertificationCacheData)_table[pageID];
				if(data.Managed.Equals(mgrDate))
					return data.EndPost;
				else
					return Guid.Empty;
			}
			else
				return Guid.Empty;
		}

		public static void SetEndGuid(Guid pageID,DateTime mgrDate,Guid endPost,Guid[] errors)
		{
			_table.Remove(pageID);
			_table.Add(pageID,new WebCertificationCacheData(mgrDate,endPost,errors));
		}

		public static Guid[] GetSignErrorList(Guid pageID)
		{
			if(_table.ContainsKey(pageID))
			{
				return ((WebCertificationCacheData)_table[pageID]).Errors;
			}
			else
				return new Guid[0];
		}
	}
	class WebCertificationCacheData
	{
		public DateTime Managed;
		public Guid EndPost;
		public Guid[] Errors;
		public WebCertificationCacheData()
		{

		}
		public WebCertificationCacheData(DateTime m,Guid p,Guid[] e)
		{
			Managed = m;
			EndPost = p;
			Errors = e;
		}
	}
}
