﻿using System;
using System.IO;

namespace RinGOch
{
	public class BinaryCoder
	{
		public static void Serialize(Stream strm,IBinaryCodable data)
		{
			strm.Write(BitConverter.GetBytes(GetBinaryID(data)),0,4);
			data.Serialize(strm);
		}
		public static object Deserialize(Stream strm)
		{
			if(strm == null) throw new NullReferenceException("Streamがnullです");
			byte[] buf = new byte[4];
			if(strm.Read(buf,0,4) != 4) throw new Exception("StreamがEOFに到達しました");
			int index = BitConverter.ToInt32(buf,0);
			if(index < RegTypes.Length && index >= 0)
			{
				Type t = RegTypes[index];
				return t.GetConstructor(new Type[]{typeof(Stream)}).Invoke(new object[]{strm});
			}
			else
				throw new Exception("indexの値が範囲外です。キャッシュが破損しているおそれがあります");
		}

		static Type[] RegTypes = new Type[]{
											   typeof(object), /* Alpha2との互換保持のため */
											   typeof(object), /* Alpha2との互換保持のため */
											   typeof(object), /* Alpha2との互換保持のため */
											   typeof(object), /* Alpha2との互換保持のため */
											   typeof(object), /* Alpha2との互換保持のため */
											   typeof(WebHeaderInfo),
											   typeof(WebPostInfo)
										   };

		public static int GetBinaryID(IBinaryCodable data)
		{
			return Array.IndexOf(RegTypes,data.GetType());
		}

		public static void WriteString(Stream strm,string text)
		{
			BinaryWriter writer = new BinaryWriter(strm);
			if(text != null)
			{
				byte[] buffer = System.Text.Encoding.UTF8.GetBytes(text);
				writer.Write(buffer.Length);
				writer.Write(buffer);
			}
			else
				writer.Write((int)0);
		}

		public static string ReadString(Stream strm)
		{
			BinaryReader reader = new BinaryReader(strm);
			int size = reader.ReadInt32();
			if(size > 0)
			{
				return System.Text.Encoding.UTF8.GetString(reader.ReadBytes(size));
			}
			else
			{
				return "";
			}
		}

		public static void WriteByteArray(Stream strm,byte[] ary)
		{
			BinaryWriter writer = new BinaryWriter(strm);
			if(ary != null)
			{
				writer.Write(ary.Length);
				writer.Write(ary);
			}
			else
				writer.Write(0);
		}

		public static byte[] ReadByteArray(Stream strm,bool nullIfEmpty)
		{
			BinaryReader reader = new BinaryReader(strm);
			int size = reader.ReadInt32();
			if(size > 0)
			{
				return reader.ReadBytes(size);
			}
			else
			{
				if(nullIfEmpty)
					return null;
				else
					return new byte[0];
			}
		}
		public static IBinaryCodable[] ReadCodableArray(Stream strm)
		{
			int len = new BinaryReader(strm).ReadInt32();
			IBinaryCodable[] ary = new IBinaryCodable[len];
			for(int i=0;i<len;i++)
				ary[i] = (IBinaryCodable)Deserialize(strm);
			return ary;
		}
		public static void WriteCodableArray(Stream strm,IBinaryCodable[] array)
		{
			BinaryWriter writer = new BinaryWriter(strm,System.Text.Encoding.UTF8);
			if(array == null)
			{
				writer.Write(0);
			}
			else
			{
				writer.Write(array.Length);
				foreach(IBinaryCodable bin in array)
					Serialize(strm,bin);
			}
		}
	}
}
