using System;
using System.Security.Cryptography;

namespace RinGOch
{
	public class WebHeaderInfo : IBinaryCodable
	{
		#region oϐ
		Guid		_guid;
		string		_title;
		byte[]		_hash;
		DateTime	_created;
		DateTime	_update;
		int			_posts;
		int			_size;
		string		_cate;
		WebType		_type;
		byte[]		_trip;
		byte[]		_sign;

		byte[]		_mgr_trip;
		byte[]		_mgr_sign;
		#endregion

		#region RXgN^
		public WebHeaderInfo(string title,string cate,Guid guid,int posts,int size,DateTime updated,DateTime created,int type,byte[] trip,byte[] sign,byte[] hash)
		{
			_title = title;
			_cate = cate;
			_guid = guid;
			_posts = posts;
			_size = size;
			_update = updated;
			_created = created;
			_type = (WebType)type;
			_trip = trip;
			_sign = sign;
			_hash = hash;
			_mgr_trip = null;
			_mgr_sign = null;	
		}
		#endregion

		#region 
		byte[] PublickeyToGuid(byte[] publickey)
		{
			using(MD5CryptoServiceProvider md5 = new MD5CryptoServiceProvider())
			{
				md5.Initialize();
				byte[] data = md5.ComputeHash(publickey);
				md5.Clear();
				return data;
			}
		}

		/// <summary>
		/// `FbN
		/// </summary>
		public bool IsEffectiveSignature()
		{
			try
			{
				if(_trip == null)return false;

				using(RSACryptoServiceProvider rsa = new RSACryptoServiceProvider())
				{
					rsa.FromXmlString(System.Text.Encoding.ASCII.GetString(_trip));
					bool check =  (ByteArrayComparer.Comparer(_guid.ToByteArray(),PublickeyToGuid(rsa.ExportParameters(false).Modulus))==0);
					if(check)
						check = rsa.VerifyHash(CalcDataHash(),CryptoConfig.MapNameToOID("SHA1"),_sign);
					rsa.Clear();
					return check;
				}
			}
			catch
			{
				return false;
			}
		}
		#endregion

		#region nbV`FbN
		public byte[] CalcDataHash()
		{
			using(HashAlgorithm ha = new SHA1Managed())
			{
				ha.Initialize();
				byte[] temp = new byte[1024*5];

				// ^Cg
				ha.TransformBlock(System.Text.Encoding.UTF8.GetBytes(_title),0,System.Text.Encoding.UTF8.GetByteCount(_title),temp,0);

				// JeS
				ha.TransformBlock(System.Text.Encoding.UTF8.GetBytes(_cate),0,System.Text.Encoding.UTF8.GetByteCount(_cate),temp,0);

				// t
				byte[] buffer = BitConverter.GetBytes(_created.Ticks);
				ha.TransformFinalBlock(buffer,0,buffer.Length);

				// I
				byte[] hash = ha.Hash;
				ha.Clear();
				return hash;
			}
		}
		#endregion

		#region vpeB
		public bool IsNullHash
		{
			get
			{
				if(_hash == null)return true;

				byte[] nullHash = new byte[_hash.Length];
				return (ByteArrayComparer.Comparer(_hash,nullHash)==0);
			}
		}
		public int Size
		{
			get { return _size; }
			set { _size = value;}
		}
		public WebType Type
		{
			get { return _type; }
		}
		public string Title
		{
			get { return _title; }
			set { _title = value;}
		}
		public Guid GUID
		{
			get { return _guid; }
		}
		public byte[] Hash
		{
			get { return _hash; }
			set { _hash = value;}
		}
		public DateTime Created
		{
			get { return _created; }
			set { _created = value;}
		}
		public int Posts
		{
			get { return _posts; }
			set { _posts = value;}
		}
		public DateTime Update
		{
			get { return _update; }
			set { _update = value;}
		}
		public string Category
		{
			get { return _cate; }
			set { _cate = value;}
		}
		public byte[] ManagerTrip
		{
			get { return _mgr_trip; }
		}
		public byte[] ManagerSignature
		{
			get { return _mgr_sign; }
		}
		public byte[] Trip
		{
			get { return _trip; }
			set { _trip = value;}
		}
		public byte[] Signature
		{
			get { return _sign; }
			set { _sign = value;}
		}
		#endregion

		#region r
		public override bool Equals(object obj)
		{
			if(obj.GetType().Equals(typeof(WebHeaderInfo)))
				return ((WebHeaderInfo)obj).GUID.Equals(this.GUID);
			if(obj.GetType().Equals(typeof(Guid)))
				return ((Guid)obj).Equals(this.GUID);
			/*if(obj.GetType().Equals(typeof(SeenInfoItem)))
				return ((SeenInfoItem)obj).ThreadInfo.GUID.Equals(this.GUID);
			if(obj.GetType().Equals(typeof(ThreadSideInfo)))
				return ((ThreadSideInfo)obj).ThreadInfo.GUID.Equals(this.GUID);*/
			return base.Equals(obj);				
		}

		public override int GetHashCode()
		{
			return base.GetHashCode ();
		}
		#endregion

		#region IBinaryCodable
		public void Serialize(System.IO.Stream strm)
		{
			System.IO.BinaryWriter writer = new System.IO.BinaryWriter(strm,System.Text.Encoding.UTF8);

			writer.Write(_guid.ToByteArray());
			writer.Write(_title);

			if(_hash != null)
			{
				writer.Write(_hash.Length);
				writer.Write(_hash);
			}
			else
				writer.Write(0);

			writer.Write(_created.Ticks);
			writer.Write(_update.Ticks);
			writer.Write(_posts);
			writer.Write(_size);
			writer.Write(_cate);
			writer.Write((int)_type);
			writer.Write(_trip.Length);
			writer.Write(_trip);
			writer.Write(_sign.Length);
			writer.Write(_sign);

			if(_mgr_trip != null)
			{
				writer.Write(_mgr_trip.Length);
				writer.Write(_mgr_trip);
			}
			else
				writer.Write(0);

			if(_mgr_sign != null)
			{
				writer.Write(_mgr_sign.Length);
				writer.Write(_mgr_sign);
			}
			else
				writer.Write(0);
		}

		public WebHeaderInfo(System.IO.Stream strm)
		{
			System.IO.BinaryReader reader = new System.IO.BinaryReader(strm,System.Text.Encoding.UTF8);

			_guid = new Guid(reader.ReadBytes(16));
			_title = reader.ReadString();

			int size = reader.ReadInt32();
			if(size > 0)
				_hash = reader.ReadBytes(size);
			else
				_hash = null;

			_created = new DateTime(reader.ReadInt64());
			_update = new DateTime(reader.ReadInt64());
			_posts = reader.ReadInt32();
			_size = reader.ReadInt32();
			_cate = reader.ReadString();
			_type = (WebType)reader.ReadInt32();

			_trip = reader.ReadBytes(reader.ReadInt32());
			_sign = reader.ReadBytes(reader.ReadInt32());
			
			size = reader.ReadInt32();
			if(size > 0)
				_mgr_trip = reader.ReadBytes(size);
			else
				_mgr_trip = null;

			size = reader.ReadInt32();
			if(size > 0)
				_mgr_sign = reader.ReadBytes(size);
			else
				_mgr_sign = null;
		}
		#endregion

	}
}
