using System;
using System.Security.Cryptography;

namespace RinGOch
{
	public class WebPostInfo : IBinaryCodable
	{
		#region oϐ
		Guid		_guid;
		string		_name;
		string		_body;

		DateTime	_posted;

		byte[]		_trip;
		byte[]		_sign;

		string		_fileName;
		byte[]		_data;
		#endregion

		#region RXgN^
		public WebPostInfo(string name,string body,DateTime posted,Guid guid,byte[] trip,byte[] sign,string file,byte[] data)
		{
			_name = name;
			_body = body;
			_posted = posted;
			_guid = guid;

			_trip = trip;
			_sign = sign;

			_fileName = file;
			_data = data;
		}
		#endregion

		#region vZ
		/// <summary>
		/// Lǂ`FbNI
		/// </summary>
		/// <returns>LȂtrueAȂfalse</returns>
		public bool IsEffectiveSignature()
		{
			try
			{
				if(_trip == null)return true;

				using(RSACryptoServiceProvider rsa = new RSACryptoServiceProvider())
				{
					rsa.FromXmlString(System.Text.Encoding.ASCII.GetString(_trip));
					bool check = rsa.VerifyHash(CalcDataHash(),CryptoConfig.MapNameToOID("SHA1"),_sign);
					rsa.Clear();
					return check;
				}
			}
			catch
			{
				return false;
			}
		}
		#endregion

		#region nbVvZ
		public byte[] CalcDataHash()
		{
			using(HashAlgorithm ha = new SHA1Managed())
			{
				ha.Initialize();
				byte[] temp = new byte[1024 * 1024 * 5];

				// OnbV`FbN
				if(_name != "")
					ha.TransformBlock(System.Text.Encoding.UTF8.GetBytes(_name),0,System.Text.Encoding.UTF8.GetByteCount(_name),temp,0);

				// {
				if(_body != "")
					ha.TransformBlock(System.Text.Encoding.UTF8.GetBytes(_body),0,System.Text.Encoding.UTF8.GetByteCount(_body),temp,0);

				// t
				byte[] buffer = BitConverter.GetBytes(_posted.Ticks);
				ha.TransformBlock(buffer,0,buffer.Length,temp,0);

				// Ytf[^
				if(_data != null)
					ha.TransformBlock(_data,0,_data.Length,temp,0);

				// Ytt@C
				if(_fileName != "")
					ha.TransformBlock(System.Text.Encoding.UTF8.GetBytes(_fileName),0,System.Text.Encoding.UTF8.GetByteCount(_fileName),temp,0);

				// ŌGUID
				ha.TransformFinalBlock(_guid.ToByteArray(),0,_guid.ToByteArray().Length);

				// I
				byte[] hash = ha.Hash;
				ha.Clear();
				return hash;
			}
		}
		#endregion

		#region vpeB
		public int TotalSize
		{
			get
			{
				int size = 0;
				if(Name != null)
					size += System.Text.Encoding.UTF8.GetByteCount(Name);
				if(Body != null)
					size += System.Text.Encoding.UTF8.GetByteCount(Body);
				if(FileName != null)
					size += System.Text.Encoding.UTF8.GetByteCount(FileName);
				if(Signature != null)
					size += Signature.Length;
				if(Data != null)
					size += Data.Length;
				if(Trip != null)
					size += Trip.Length;
				return size + 16 + 8;
			}
		}
		public string Name
		{
			get { return _name; }
			set { _name = value;}
		}
		public string Body
		{
			get { return _body; }
			set { _body = value;}
		}
		public Guid GUID
		{
			get { return _guid; }
			set { _guid = value;}
		}
		public DateTime Posted
		{
			get { return _posted; }
			set { _posted = value;}
		}
		public byte[] Signature
		{
			get { return _sign; }
			set { _sign = value;}
		}
		public byte[] Trip
		{
			get { return _trip; }
			set { _trip = value;}
		}
		public byte[] Data
		{
			get{ return _data;}
			set{ _data = value; }
		}
		public string FileName
		{
			get { return _fileName; }
			set { _fileName = value;}
		}
		public string WikiName
		{
			get 
			{
				if(FileName == null || FileName == "")
				{
					return "FrontPage";
				}
				else
					return FileName;
			}
		}
		#endregion

		#region r
		public override bool Equals(object obj)
		{
			return this.GUID.Equals(((WebPostInfo)obj).GUID);
		}

		public override int GetHashCode()
		{
			return base.GetHashCode ();
		}
		#endregion

		#region IBinaryCodable o

		public void Serialize(System.IO.Stream strm)
		{
			System.IO.BinaryWriter writer = new System.IO.BinaryWriter(strm,System.Text.Encoding.UTF8);

			writer.Write(_guid.ToByteArray());
			writer.Write(_name);
			writer.Write(_body);
			writer.Write(_posted.Ticks);
			BinaryCoder.WriteByteArray(strm,_trip);
			BinaryCoder.WriteByteArray(strm,_sign);
			writer.Write(_fileName);
			BinaryCoder.WriteByteArray(strm,_data);
		}

		public WebPostInfo(System.IO.Stream strm)
		{
			System.IO.BinaryReader reader = new System.IO.BinaryReader(strm,System.Text.Encoding.UTF8);

			_guid = new Guid(reader.ReadBytes(16));
			_name = reader.ReadString();
			_body = reader.ReadString();
			_posted = new DateTime(reader.ReadInt64());
			_trip = BinaryCoder.ReadByteArray(strm,true);
			_sign = BinaryCoder.ReadByteArray(strm,true);
			_fileName = reader.ReadString();
			_data = BinaryCoder.ReadByteArray(strm,true);
		}

		#endregion
	}
}
