using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using RinGOch.WikiEngine.LineData;

namespace RinGOch.WikiEngine
{
	/// <summary>
	/// InLineReplacer ̊Tv̐łB
	/// </summary>
	/// <remarks>
	/// CCuNXłB
	/// </remarks>
	public class InlineReplacer
	{
		/// <summary>
		/// ϊCC̎ނw肵܂B
		/// </summary>
		/// <remarks>
		/// ϊCCǂ̎ނ̍sɑĂ邩w肵܂B
		/// </remarks>
		public enum ReplaceType 
		{ 
			/// <summary>
			/// ϊCCy[W̖{sɑĂ鎖w肵܂B
			/// </summary>
			Contents = 0, 
			/// <summary>
			/// ϊCCy[W̍sɑĂ鎖w肵܂B 
			/// </summary>
			Headings = 1 
		}

		/// <summary>
		/// uPbgl[̐K\łB
		/// </summary>
		/// <remarks>
		/// pubNXR[v̒萔łB
		/// </remarks>
		public const string BRACKET_NAME = @"(?=\[\[[^\[])\[\[([^\]]+?)(?![^\]]\]\])\]\]";

		/// <summary>
		/// WIKIl[̐K\łB
		/// </summary>
		/// <remarks>
		/// pubNXR[v̒萔łB
		/// </remarks>
		//public const string WIKI_NAME = "([A-Z][a-z]+([A-Z][a-z]+)+)";

		/// <summary>
		/// IWiʂ̐K\łB
		/// </summary>
		/// <remarks>
		/// pubNXR[v̒萔łB
		/// </remarks>
		public const string EMBEDDED_NAME = @"(?=\[\[[^\[])\[\[(#[^\]]+?)(?![^\]]\]\])\]\]";//"\[*\[\[(#[^\]]+?)\]\]\]*";

		private const string ESCAPED_WORD = "@escapedWord@";
		private const string URL = @"(https?|ftp|news)(:\/\/[-_.!~*'()a-zA-Z0-9;\/?:\@&=+\$,%#]+)";
		private const string MAIL_ADDRESS = @"[a-zA-Z0-9_\-\.]+@[a-zA-Z0-9_\-]+(\.[a-zA-Z0-9_\-]+)+";
		private const string IMAGE = @"^(https?|ftp|news).*?(?<image>\.gif|\.png|\.jpeg|\.jpg)?$";

		private string linkTarget;
		private Queue escapedWords = new Queue();
		private WebPostInfo[] wikiPosts;
		private WebPostInfo   wikiDisplaying;
		private WebHeaderInfo wikiHeader;
		private string _localAdrs = "";

		public bool BlogMode = false;
		public bool	enableEmbedded = true;
		public bool enableStyleAttribute = true;

		public bool enableComment = true;
		public bool enableInclude = true;

		public InlineReplacer(string linkTarget,WebHeaderInfo header,WebPostInfo[] posts,WebPostInfo disp)
		{
			this.linkTarget = linkTarget;
			this.wikiDisplaying = disp;
			this.wikiPosts = posts;
			this.wikiHeader = header;
			_localAdrs = HttpHtmlGenerator.CreateLocalAddress();
		}

		/// <summary>
		/// CCu܂B
		/// </summary>
		/// <param name="lineData">sf[^</param>
		/// <param name="replaceType">uCC̎</param>
		/// <returns>CCu̍sf[^</returns>
		/// <remarks>
		/// y[W̖{s̓y[W̍s̃CCu܂B
		/// </remarks>
		public string ReplaceInline(ILineData lineData, ReplaceType replaceType)
		{
			string replaced = lineData.GetContent();

			if (lineData is PreformatLineData) return replaced; 

			replaced = Regex.Replace(replaced, "~{2}(?<escapedWord>[^~]+?)~{2}", new MatchEvaluator(SetEscapedWords));
			replaced = ReplaceSimpleKeyword(replaced);
			if (replaceType == ReplaceType.Contents) replaced = ReplaceLink(replaced);
			replaced = Regex.Replace(replaced, ESCAPED_WORD, new MatchEvaluator(ResetEscapedWords));

			return replaced;
		}

		/// <summary>
		/// CCuGXP[vݒ肵܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>CCuGXP[v</returns>
		private string SetEscapedWords(Match match)
		{
			escapedWords.Enqueue(match.Groups["escapedWord"].Value);

			return ESCAPED_WORD;
		}

		/// <summary>
		/// CCuGXP[v̕ɍĐݒ肵܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>̕</returns>
		private string ResetEscapedWords(Match match)
		{
			return (string) escapedWords.Dequeue();
		}

		/// <summary>
		/// PȃL[[hu܂B
		/// </summary>
		/// <param name="input">u镶</param>
		/// <returns>u̕</returns>
		private string ReplaceSimpleKeyword(string input)
		{
			string replaced = input;
			replaced = Regex.Replace(replaced, "'{3}([^']+?)'{3}", "<em>$1</em>");			// C^bN
			replaced = Regex.Replace(replaced, "'{2}([^']+?)'{2}", "<strong>$1</strong>");	// {[h
			replaced = Regex.Replace(replaced, "%{2}([^%]+?)%{2}", "<del>$1</del>");		// 
			replaced = Regex.Replace(replaced, "_{2}([^_]+?)_{2}", "<ins>$1</ins>");		// 
			replaced = replaced.Replace("&amp;br;","<br />");								// s
			replaced = replaced.Replace("~\r", "<br />\r");									// s
			replaced = replaced.Replace("~\n", "<br />\n");									// s
			replaced = replaced.Replace("&amp;date;",TextUtility.DateTimeToString(wikiDisplaying.Posted));

			replaced = FormatWikiB(replaced,"&amp;size","span","font-size:");// eLXgTCY
			replaced = FormatWikiB(replaced,"&amp;color","span","color:");// eLXgF
			if(enableStyleAttribute) replaced = FormatWikiB(replaced,"&amp;style","span","");
			replaced = replaced.Replace("&amp;local;",_localAdrs);
			replaced = replaced.Replace(_localAdrs+"/",_localAdrs);

			replaced = replaced.Replace("http://ringoch.p2p/",_localAdrs);

			return replaced;
		}

		#region tHgTCYF
		string FormatWikiB(string temp,string head,string ele,string para)
		{
			int begin = 0;
			while(true)
			{
				int pos = temp.IndexOf(head,begin);
				if(pos < 0)break;
				begin = pos + head.Length;

				int ap = temp.IndexOf("(",pos);
				if(ap > 0)
				{
					int bp = temp.IndexOf(")",ap);
					if(bp > 0)
					{
						string v = temp.Substring(ap+1,bp-ap-1).Replace("\"","");
						temp = temp.Remove(pos,bp-pos+2);
						if(enableStyleAttribute)
							temp = temp.Insert(pos,"<" + ele + " style=\"" + para + v + (para=="font-size:" && TextUtility.IsNumeric(v) ? "ex" : "") + "\">");
						else
						{
							if(TextUtility.IsNumeric(v))
							{
								int iv = int.Parse(v);if(iv > 10) iv = 10;
								temp = temp.Insert(pos,"<" + ele + " style=\"" + para + iv.ToString() + "ex" + "\">");
							}
							else
								temp = temp.Insert(pos,"<" + ele + ">");
						}

						int pos2 = temp.IndexOf("};",pos);
						if(pos2 >= 0)
							temp = temp.Remove(pos2,2);
						else
							pos2 = temp.Length;
						temp = temp.Insert(pos2,"</" + ele + ">");
					}
				}
			}
			return temp;
		}
		#endregion

		#region Nu

		/// <summary>
		/// Nu܂B
		/// </summary>
		/// <param name="input">u镶</param>
		/// <returns>u̕</returns>
		private string ReplaceLink(string input)
		{
			string replaced = input;

			StringBuilder builder = new StringBuilder();
			builder.Append("(");
			builder.Append(URL);
			builder.Append("|");
			builder.Append(@"(\[\[([^\]]+)\:" + URL + @"\]\])");
			builder.Append("|");
			builder.Append(@"(\[" + URL + @"\s([^\]]+)\])");
			builder.Append("|");
			builder.Append(@"(" + MAIL_ADDRESS + ")");
			builder.Append("|");
			builder.Append(@"(\[\[([^\]]+)\:(" + MAIL_ADDRESS + @")\]\])");
			builder.Append("|");
			builder.Append(@"(\[" + MAIL_ADDRESS + @"\s([^\]]+)\])");
			builder.Append("|");
			builder.Append(BRACKET_NAME);
			//builder.Append("|");
			//builder.Append(WIKI_NAME);
			builder.Append(")");

			return Regex.Replace(replaced, builder.ToString(), new MatchEvaluator(MakeLink));
		}

		/// <summary>
		/// N쐬܂B
		/// MatchEvaluatorfQ[g\bh
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>N</returns>
		private string MakeLink(Match match)
		{
			string link = match.Groups[0].Value;

			Match normalLinkMatch = Regex.Match(link, IMAGE,RegexOptions.IgnoreCase);
			if (normalLinkMatch.Success) return MakeNormalLink(normalLinkMatch);

			Match mailLinkMatch = Regex.Match(link, @"^(" + MAIL_ADDRESS + ")$");
			if (mailLinkMatch.Success) return MakeMailLink(mailLinkMatch);

			Match embeddedMatch = Regex.Match(link, EMBEDDED_NAME);
			if (embeddedMatch.Success) return MakeEmbedded(embeddedMatch);

			Match aliasLinkMatch1 = Regex.Match(link, 
				@"^\[\[(?<name>[^\]]+)\:(?<url>(https?|ftp|news)([^\]]+))\]\]$");
			if (aliasLinkMatch1.Success) return MakeAliasLink(aliasLinkMatch1);

			Match aliasLinkMatch2 = Regex.Match(link, 
				@"^\[(?<url>(https?|ftp|news)([^\]\s]+))\s(?<name>[^\]]+)\]$");
			if (aliasLinkMatch2.Success) return MakeAliasLink(aliasLinkMatch2);
	
			Match aliasMailLinkMatch1 = Regex.Match(link, 
				@"^(\[\[(?<name>[^\]]+)\:(?<address>" + MAIL_ADDRESS + @")\]\])$");
			if (aliasMailLinkMatch1.Success) return MakeAliasMailLink(aliasMailLinkMatch1);

			Match aliasMailLinkMatch2 = Regex.Match(link, 
				@"^\[(?<address>" + MAIL_ADDRESS + @")\s(?<name>[^\]]+)\]$");
			if (aliasMailLinkMatch2.Success) return MakeAliasMailLink(aliasMailLinkMatch2);

			if(enableEmbedded)
			{
				Match pageLinkMatch = Regex.Match(link, 
					@"^(" + BRACKET_NAME + ")$");
				//@"^(" + BRACKET_NAME + ")$|^(" + WIKI_NAME + ")$");
				if (pageLinkMatch.Success) return MakePageLink(pageLinkMatch);
			}

			return link;
		}
		
		bool IsRemoteAddress(string adrs)
		{
			return !adrs.StartsWith(HttpHtmlGenerator.CreateLocalAddress());
		}

		/// <summary>
		/// ʏ/摜N쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>ʏ/摜N</returns>
		private string MakeNormalLink(Match match)
		{
			if (match.Groups["image"].Length > 0 & !IsRemoteAddress(match.Groups[0].Value))
				return "<a href=\"" + match.Groups[0].Value + "\" target=\"" + linkTarget + "\">" +
					"<img src=\"" + match.Groups[0].Value + "\" border=\"0\" /></a>";
			else
				return "<a href=\"" + match.Groups[0].Value + "\" target=\"" + linkTarget + "\">" + 
					match.Groups[0].Value + "</a>";
		}

		/// <summary>
		/// [N쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>[N</returns>
		private string MakeMailLink(Match match)
		{
			return "<a href=\"mailto:" + match.Groups[0].Value + "\">" + match.Groups[0].Value + "</a>";
		}

		/// <summary>
		/// Rg̓{bNX쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>Rg̓{bNX</returns>
		private string MakeEmbedded(Match match)
		{
			string embedded = match.Groups[0].Value;

			if (enableEmbedded && enableInclude && embedded.StartsWith("[[#include("))
			{
				string temp = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_paragraph.html");
				string wikiname = TextUtility.GetMiddleRangeString(embedded,"(",")");
				WebPostInfo info = HtmlGenerators.WikiGenerator.GetNewestWikiName(wikiPosts,wikiHeader,wikiname,Guid.Empty);
				if(info != null)
				{
					if(wikiname == wikiDisplaying.FileName || info.Body.IndexOf("[[#include(" + wikiDisplaying.FileName + ")]]") >= 0)
					{
						return "#includeG[F[vĂ܂B}CĂ";
					}
					else
					{
						string original = info.Body;
						info.Body = Engine.Transform(info.Body,wikiHeader,wikiPosts,info);
						temp = HtmlGenerators.TemplateFieldReplacer.ReplaceWikiFields(temp,wikiHeader,info,null);
						info.Body = original;
						return temp;
					}
				}
				else
				{
					Match m = Regex.Match("[[" + wikiname + "]]",@"\[\[(.+)\]\]");
					return MakePageLink(m);
				}
			}
			else if(enableEmbedded && enableComment && embedded.StartsWith("[[#comments("))
			{
				string item = TextUtility.GetMiddleRangeString(embedded,"(",")");
				string[] items = item.Split(',');
				string id = items[0];
				string num = (items.Length>1?items[1]:"0");
				return CommentTransformer.Transform(id,num,wikiHeader,wikiDisplaying,wikiPosts);
			}
			else if(enableEmbedded && embedded.StartsWith("[[#edit("))
			{
				string item,name;
				if(embedded.IndexOf("\"") >= 0)
				{
					item = TextUtility.GetMiddleRangeString(embedded,"(\"","\",\"");
					name = TextUtility.GetMiddleRangeString(embedded,"\",\"","\")");
				}
				else
				{
					item = TextUtility.GetMiddleRangeString(embedded,"(",",");
					name = TextUtility.GetMiddleRangeString(embedded,",",")");
				}
				return "<a title=\"" + HttpUtility.HtmlEncode(item) + "ҏW\" href=\"edit.p2p?name=" + HttpUtility.UrlEncode(item) + "\">" + HttpUtility.HtmlEncode(name) + "</a>";
			}
			else if(embedded.StartsWith("[[#img("))
			{
				string path = TextUtility.GetMiddleRangeString(embedded,"(",")]").Replace("\"","");
				if(enableEmbedded)
				{
					WebPostInfo post = HtmlGenerators.WikiGenerator.GetNewestWikiName(wikiPosts,wikiHeader,"\0" + wikiDisplaying.WikiName + ":\0" + path,Guid.Empty);
					if(post != null)
						return "<img src=\"" + post.GUID.ToString() + "\\" + path + "\" />";
					else
						return "<img src=\"" + embedded.Replace("\"","") + "\" />";
				}
				else
				{
					if(BlogMode)
					{
						path = string.Concat("/files/",HttpUtility.UrlEncode(System.IO.Path.GetFileName(path)));
						return string.Concat("<img src=\"",_localAdrs,"",wikiHeader.GUID.ToString(),path,"\" />");
					}
					else
					{
						return string.Concat("<img src=\"",path,"\" />");
					}
				}
			}
			else if(embedded.StartsWith("[[#ref("))
			{
				string path = TextUtility.GetMiddleRangeString(embedded,"(",")]").Replace("\"","");
				if(enableEmbedded)
				{
					WebPostInfo post = HtmlGenerators.WikiGenerator.GetNewestWikiName(wikiPosts,wikiHeader,"\0" + wikiDisplaying.WikiName + ":\0" + path,Guid.Empty);
					if(post != null)
						return "<a href=\"" + post.GUID.ToString() + "\\" + path + "\">" + HttpUtility.HtmlEncode(path) + "</a>";
					else
						return embedded;
				}
				else
				{
					if(BlogMode)
					{
						path = string.Concat("/files/",HttpUtility.UrlEncode(System.IO.Path.GetFileName(path)));
						return string.Concat("<a href=\"",_localAdrs,"",wikiHeader.GUID.ToString(),path,"\">",HttpUtility.HtmlEncode(System.IO.Path.GetFileName(path)),"</a>");
					}
					else
					{
						return string.Concat("<a href=\"",path,"\">",HttpUtility.HtmlEncode(System.IO.Path.GetFileName(path)),"</a>");
					}
				}
			}
			else if(enableEmbedded && embedded.StartsWith("[[#newest("))
			{
				string item = TextUtility.GetMiddleRangeString(embedded,"(",")");
				if(TextUtility.IsNumeric(item))
				{
					int count = int.Parse(item);
					int counter = 0;
					StringBuilder html = new StringBuilder();
					string template = HttpHtmlGenerator.ReadTemplateFile("template\\wiki\\wiki_newest.html");
					string header = template.Substring(0,template.IndexOf("<!--BEGIN LOOP-->"));
					string loop = TextUtility.GetMiddleRangeString(template,"<!--BEGIN LOOP-->","<!--END LOOP-->");
					string footer = template.Substring(template.IndexOf("<!--END LOOP-->")+"<!--END LOOP-->".Length);
					WebPostInfo[] posts = (WebPostInfo[])wikiPosts.Clone();
					Array.Sort(posts,new WebPostInfoComparer());
					Array.Reverse(posts);
					ArrayList wikis = new ArrayList();
					DateTime last = DateTime.MinValue;

					for(int i=0;i<posts.Length;i++)
					{
						WebPostInfo post = posts[i];

						if(!wikis.Contains(post.WikiName))
						{
							// o͍ςWikiXgɒǉ
							wikis.Add(post.WikiName);

							if(!post.WikiName.StartsWith("\0"))
							{
								if(!post.Posted.Date.Equals(last))
								{
									// ŏ̍ڂłȂꍇ̓tb^t
									if(i > 0) html.Append(footer);

									// to
									html.Append(header.Replace("%_newest_date%",post.Posted.ToString("yyyy-MM-dd")));


									last = post.Posted.Date;
								}
							
								// Wikio
								html.Append(HtmlGenerators.TemplateFieldReplacer.ReplaceWikiFields
									(loop,wikiHeader,post,null));

								counter++;

							}
						}

						if(count == counter)
						{
							// ŌɃtb^t
							html.Append(footer);
							break;
						}
					}

					return html.ToString();
				}
				else
					return "";
			}
			else
				return embedded;
		}

		/// <summary>
		/// ʖN쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>ʖN</returns>
		private string MakeAliasLink(Match match)
		{
			Match imageMatch = Regex.Match(match.Groups["name"].Value, IMAGE);	

			if (imageMatch.Success)
			{
				return "<a href=\"" + match.Groups["url"].Value + "\" target=\"" + linkTarget + "\">" +
					"<img src=\"" + match.Groups["name"].Value + "\" border=\"0\" /></a>";
			}
			else
			{
				return "<a href=\"" + match.Groups["url"].Value + "\" target=\"" + linkTarget + "\">" + 
					match.Groups["name"].Value + "</a>";
			}
		}

		/// <summary>
		/// ʖ[N쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>ʖ[N</returns>
		private string MakeAliasMailLink(Match match)
		{
			return "<a href=\"mailto:" + match.Groups["address"].Value + "\">" + 
				match.Groups["name"].Value + "</a>";
		}

		/// <summary>
		/// Wikiy[WN쐬܂B
		/// </summary>
		/// <param name="match">K\ƈvʂMatch</param>
		/// <returns>Wikiy[WN</returns>
		private string MakePageLink(Match match)
		{
			string page = StripBracket(match.Groups[0].Value);
			string title = UnReplaceSpecialChars(page);
			string name = title;

			if(title.IndexOf(">") > 0)
			{
				name = title.Substring(title.IndexOf(">")+1);
				title = title.Substring(0,title.IndexOf(">"));
			}

			bool flag = false;
			foreach(WebPostInfo post in wikiPosts)
				if(post.FileName == name)
				{
					flag = true;
					break;
				}

			if (flag)
				return "<a href=\"wiki.p2p?name=" + HttpUtility.UrlEncode(name) + "\">" + HttpUtility.HtmlEncode(title) + "</a>";
			else
				return HttpUtility.HtmlEncode(title) + "<a title=\"̃y[WҏW܂B\" href=\"edit.p2p?name=" + HttpUtility.UrlEncode(name) + "\">?</a>";
		}

		/// <summary>
		/// uPbgl[uPbgO܂B
		/// </summary>
		/// <param name="bracketName">BracketName</param>
		/// <returns>Wikiy[W</returns>
		private string StripBracket(string bracketName)
		{
			string replaced = bracketName;
			replaced = Regex.Replace(replaced, @"^\[\[(.*)\]\]$", "$1");

			return replaced;
		}

		/// <summary>
		/// uHTMLꕶĒu܂B
		/// </summary>
		/// <param name="input">Ēu镶</param>
		/// <returns>Ēu̕</returns>
		private string UnReplaceSpecialChars(string input)
		{
			string replaced = input;
			replaced = Regex.Replace(replaced, "&amp;", "&");
			replaced = Regex.Replace(replaced, "&quot;", "\"");
			replaced = Regex.Replace(replaced, "&lt;", "<");
			replaced = Regex.Replace(replaced, "&gt;", ">");

			return replaced;
		}

		#endregion

		/// <summary>
		/// ԂNA܂B
		/// </summary>
		/// <remarks>
		/// ԂԂɐݒ肵܂B
		/// </remarks>
		public void ClearState()
		{
			escapedWords.Clear();
		}
	}
}
