using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;

namespace RinGOch.WikiEngine.LineData
{
	/// <summary>
	/// table^Oɑ΂L[[h [,] ܂ލs\NXłB
	/// </summary>
	public class TableLineData : StructuredLineData
	{
		private const string TAG_TABLE = "table"; 

		public TableLineData(string keyword, string content) : base(keyword, content, TAG_TABLE,"")
		{}

		/// <summary>
		/// e擾܂B
		/// </summary>
		/// <returns>e</returns>
		/// <remarks>
		/// s̓e擾܂B^O<tr></tr><td></td>t܂B
		/// sɕt܂B
		/// </remarks>
		public override string GetContent()
		{
			StringBuilder builder = new StringBuilder();
			builder.Append("<tr>");

			ArrayList tableDatas = new ArrayList();

			string content = base._content + ",";
			content = Regex.Replace(content, "&quot;", "\"");

			Match match = Regex.Match(content, "(?<td>\"[^\"]*(?:\"\"[^\"]*)*\"|[^,]*),");

			while (match.Success) 
			{
				tableDatas.Add(new TableData(match.Groups["td"].Value));
				match = match.NextMatch();
			}

			TableData[] tdList = new TableData[tableDatas.Count];
			tableDatas.CopyTo(tdList);

			for (int i = 0; i < tdList.Length; i++)
			{
				if (tdList[i].ColSpan > 0)
				{
					while (i + tdList[i].ColSpan < tdList.Length && tdList[i + tdList[i].ColSpan].Data == TableData.EQUAL ) 
						tdList[i].IncrementColSpan();
                
					builder.Append(tdList[i]);
				}
			}

			builder.Append("</tr>\r\n");

			return builder.ToString();
		}

		#region Ci[NX

		/// <summary>
		/// TableData ̊Tv̐łB
		/// </summary>
		/// <remarks>
		/// alignAcolSpan ̎w肳ꂽtd^O\NXłB
		/// </remarks>
		private class TableData
		{
			/// <summary>
			/// ̃ZƓł邱Ƃw肷L[[hłB
			/// </summary>
			/// <remarks>
			/// pubNXR[v̒萔łB
			/// </remarks>
			public const string EQUAL = "==";

			private const string ALIGN_CENTER = "center"; 
			private const string ALIGN_RIGHT = "right"; 

			private string data;
			private int colSpan;
			private string align;
			
			#region RXgN^

			/// <summary>
			/// w肳ꂽegp
			/// TableDataNX̐VCX^X܂B
			/// </remarks>
			public TableData(string data)
			{
				Match match = Regex.Match(data, "^\"(?<data>.*)\"$");

				if (match.Success) 
					this.data = Regex.Replace(match.Groups["data"].Value, "\"{2}", "\"");
				else
					this.data = data;

				this.align = GetAlign();
				this.colSpan = (data == EQUAL ? 0 : 1);
			}

			/// <summary>
			/// \ʒu̐ݒ擾܂B
			/// </summary>
			private string GetAlign()
			{
				Match centerMatch = Regex.Match(data, @"^\s+(?<data>[^\s]*)\s+$");

				if (centerMatch.Success) 
				{
					data = centerMatch.Groups["data"].Value;
					return ALIGN_CENTER;
				}

				Match rightMatch = Regex.Match(data, @"^\s+(?<data>[^\s]+)$");

				if (rightMatch.Success) 
				{
					data = rightMatch.Groups["data"].Value;
					return ALIGN_RIGHT;
				}
				
				return string.Empty;
			}

			#endregion

			#region vpeB

			/// <summary>
			/// A񐔂擾܂B
			/// </summary>
			public int ColSpan
			{
				get { return colSpan; }
			}

			/// <summary>
			/// e擾܂B
			/// </summary>
			public string Data
			{
				get { return data; }
			}

			#endregion

			/// <summary>
			/// A񐔂Z܂B
			/// </summary>
			public void IncrementColSpan()
			{
				colSpan++;
			}

			/// <summary>
			/// IuWFNg̕\擾܂B
			/// </summary>
			public override string ToString()
			{
				string alignFormat = (align.Length > 0 ?  string.Format(" align=\"{0}\"", align) : string.Empty);
				string colSpanFormat = (colSpan > 1 ? string.Format(" colspan=\"{0}\"", colSpan.ToString()) : string.Empty);
				string replacedData = Regex.Replace(data, "\"", "&quot;");

				return string.Format("<td{0}{1}>{2}</td>", alignFormat, colSpanFormat, replacedData);
			}
		}

		#endregion
	}
}
